@@ -676,6 +676,7 @@ abstract class CControllerPopupItemTest extends CController {
 				: [];
 
 			if (count($interfaces) > 0) {
+				// Macros in interface details are not resolved.
 				$interfaces = CMacrosResolverHelper::resolveHostInterfaces($interfaces);
 
 				$interface_data = [
@@ -1011,6 +1012,24 @@ abstract class CControllerPopupItemTest extends CController {
 			}
 		}
 
+		// Resolve interface details (SNMP) macros separately.
+		if (array_key_exists('interface', $inputs) && array_key_exists('details', $inputs['interface'])) {
+			foreach ($inputs['interface']['details'] as &$field) {
+				if (strstr($field, '{') !== false) {
+					$matched_macros = (new CMacrosResolverGeneral)->getMacroPositions($field, ['usermacros' => true]);
+
+					foreach (array_reverse($matched_macros, true) as $pos => $macro) {
+						$macro_value = array_key_exists($macro, $macros_posted)
+							? $macros_posted[$macro]
+							: '';
+
+						$field = substr_replace($field, $macro_value, $pos, strlen($macro));
+					}
+				}
+			}
+			unset($field);
+		}
+
 		return $inputs;
 	}
 
