@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 $table = (new CTableInfo())->setNoDataMessage(_('No screens added.'));
 
 foreach ($data['screens'] as $screen) {
@@ -35,6 +39,7 @@ foreach ($data['screens'] as $screen) {
 			->onClick($on_click)
 			->addClass(ZBX_STYLE_REMOVE_BTN)
 			->setAttribute('aria-label', _xs('Remove, %1$s', 'screen reader', $screen['label']))
+			->removeId()
 	]);
 }
 
@@ -52,4 +57,4 @@ if ($data['user']['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
 	$output['debug'] = CProfiler::getInstance()->make()->toString();
 }
 
-echo (new CJson())->encode($output);
+echo json_encode($output);
