@@ -209,7 +209,7 @@ if (count($data['steps']) > 0) {
 			->addVar('steps['.$i.'][error_handler_params]', $step['error_handler_params']);
 
 		// Temporary solution to fix "\n\n1" conversion to "\n1" in the hidden textarea field after jQuery.append().
-		if ($step['type'] == ZBX_PREPROC_CSV_TO_JSON) {
+		if ($step['type'] == ZBX_PREPROC_CSV_TO_JSON || $step['type'] == ZBX_PREPROC_VALIDATE_RANGE) {
 			$form->addItem(new CInput('hidden', 'steps['.$i.'][params]', $step['params']));
 		}
 		else {
@@ -269,7 +269,7 @@ $templates = [
 			(new CDiv(
 				(new CSpan('#{result}'))
 					->addClass(ZBX_STYLE_LINK_ACTION)
-					->setHint('#{result}', 'hintbox-scrollable', true, 'max-width:'.ZBX_ACTIONS_POPUP_MAX_WIDTH.'px;')
+					->setHint('#{result}', 'hintbox-wrap')
 			))
 				->addStyle('max-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;')
 				->addClass(ZBX_STYLE_OVERFLOW_ELLIPSIS)
@@ -283,7 +283,7 @@ $templates = [
 				(new CDiv(
 					(new CSpan('#{failed}'))
 						->addClass(ZBX_STYLE_LINK_ACTION)
-						->setHint('#{failed}', '', true, 'max-width:'.ZBX_ACTIONS_POPUP_MAX_WIDTH.'px; ')
+						->setHint('#{failed}', 'hintbox-wrap')
 				))
 					->addStyle('max-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;')
 					->addClass(ZBX_STYLE_OVERFLOW_ELLIPSIS)
@@ -294,10 +294,19 @@ $templates = [
 		)
 ];
 
+$warning_message = [[
+	'message' => _('Item contains user defined macros with type "Secret text". Values of these macros should be entered manually.')
+]];
+$warning_box = $data['show_warning']
+	? makeMessageBox(false, $warning_message, null, true, false)
+		->removeAttribute('class')
+		->addClass(ZBX_STYLE_MSG_WARNING)
+	: null;
+
 $output = [
 	'header' => $data['title'],
 	'script_inline' => $this->readJsFile('popup.itemtestedit.view.js.php'),
-	'body' => (new CDiv([$form, $templates]))->toString(),
+	'body' => (new CDiv([$warning_box, $form, $templates]))->toString(),
 	'cancel_action' => 'return saveItemTestInputs();',
 	'buttons' => [
 		[
