@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -122,6 +122,20 @@ class API {
 		return self::getApi('application');
 	}
 
+	/**
+	 * @return CAuditLog
+	 */
+	public static function AuditLog() {
+		return self::getApi('auditlog');
+	}
+
+	/**
+	 * @return CAutoregistration
+	 */
+	public static function Autoregistration() {
+		return self::getApi('autoregistration');
+	}
+
 	/**
 	 * @return CConfiguration
 	 */
@@ -276,6 +290,13 @@ class API {
 		return self::getApi('maintenance');
 	}
 
+	/**
+	 * @return CModule
+	 */
+	public static function Module() {
+		return self::getApi('module');
+	}
+
 	/**
 	 * @return CMap
 	 */
