@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -121,15 +121,10 @@ class CDService extends CApiService {
 		if (!is_null($options['dcheckids'])) {
 			zbx_value2array($options['dcheckids']);
 
-			$sqlParts['from']['dhosts'] = 'dhosts dh';
-			$sqlParts['from']['dchecks'] = 'dchecks dc';
-
-			$sqlParts['where'][] = dbConditionInt('dc.dcheckid', $options['dcheckids']);
-			$sqlParts['where']['dhds'] = 'dh.dhostid=ds.dhostid';
-			$sqlParts['where']['dcdh'] = 'dc.druleid=dh.druleid';
+			$sqlParts['where'][] = dbConditionInt('ds.dcheckid', $options['dcheckids']);
 
 			if ($options['groupCount']) {
-				$sqlParts['group']['dcheckid'] = 'dc.dcheckid';
+				$sqlParts['group']['dcheckid'] = 'ds.dcheckid';
 			}
 		}
 
@@ -165,7 +160,7 @@ class CDService extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($dservice = DBfetch($res)) {
 			if ($options['countOutput']) {
 				if ($options['groupCount']) {
