@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -31,23 +31,21 @@ class CHostInterface extends CApiService {
 	/**
 	 * Get interface data.
 	 *
-	 * @param array   $options
-	 * @param array   $options['hostids']		Interface IDs
-	 * @param boolean $options['editable']		only with read-write permission. Ignored for SuperAdmins
-	 * @param boolean $options['selectHosts']	select Interface hosts
-	 * @param boolean $options['selectItems']	select Items
-	 * @param int     $options['count']			count Interfaces, returned column name is rowscount
-	 * @param string  $options['pattern']		search hosts by pattern in Interface name
-	 * @param int     $options['limit']			limit selection
-	 * @param string  $options['sortfield']		field to sort by
-	 * @param string  $options['sortorder']		sort order
+	 * @param array  $options
+	 * @param array  $options['hostids']		Interface IDs
+	 * @param bool   $options['editable']		only with read-write permission. Ignored for SuperAdmins
+	 * @param bool   $options['selectHosts']	select Interface hosts
+	 * @param bool   $options['selectItems']	select Items
+	 * @param int    $options['count']			count Interfaces, returned column name is rowscount
+	 * @param string $options['pattern']		search hosts by pattern in Interface name
+	 * @param int    $options['limit']			limit selection
+	 * @param string $options['sortfield']		field to sort by
+	 * @param string $options['sortorder']		sort order
 	 *
 	 * @return array|boolean Interface data as array or false if error
 	 */
 	public function get(array $options = []) {
 		$result = [];
-		$userType = self::$userData['type'];
-		$userId = self::$userData['userid'];
 
 		$sqlParts = [
 			'select'	=> ['interface' => 'hi.interfaceid'],
@@ -64,22 +62,22 @@ class CHostInterface extends CApiService {
 			'interfaceids'				=> null,
 			'itemids'					=> null,
 			'triggerids'				=> null,
-			'editable'					=> null,
+			'editable'					=> false,
 			'nopermissions'				=> null,
 			// filter
 			'filter'					=> null,
 			'search'					=> null,
 			'searchByAny'				=> null,
-			'startSearch'				=> null,
-			'excludeSearch'				=> null,
+			'startSearch'				=> false,
+			'excludeSearch'				=> false,
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
 			'selectHosts'				=> null,
 			'selectItems'				=> null,
-			'countOutput'				=> null,
-			'groupCount'				=> null,
-			'preservekeys'				=> null,
+			'countOutput'				=> false,
+			'groupCount'				=> false,
+			'preservekeys'				=> false,
 			'sortfield'					=> '',
 			'sortorder'					=> '',
 			'limit'						=> null,
@@ -88,10 +86,9 @@ class CHostInterface extends CApiService {
 		$options = zbx_array_merge($defOptions, $options);
 
 		// editable + PERMISSION CHECK
-		if ($userType != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
 			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
-
-			$userGroups = getUserGroupsByUserId($userId);
+			$userGroups = getUserGroupsByUserId(self::$userData['userid']);
 
 			$sqlParts['where'][] = 'EXISTS ('.
 				'SELECT NULL'.
@@ -116,6 +113,10 @@ class CHostInterface extends CApiService {
 		if (!is_null($options['hostids'])) {
 			zbx_value2array($options['hostids']);
 			$sqlParts['where']['hostid'] = dbConditionInt('hi.hostid', $options['hostids']);
+
+			if ($options['groupCount']) {
+				$sqlParts['group']['hostid'] = 'hi.hostid';
+			}
 		}
 
 		// itemids
@@ -153,12 +154,17 @@ class CHostInterface extends CApiService {
 			$sqlParts['limit'] = $options['limit'];
 		}
 
+		if ($this->outputIsRequested('details', $options['output'])) {
+			$sqlParts['left_join'][] = ['alias' => 'his', 'table' => 'interface_snmp', 'using' => 'interfaceid'];
+			$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
+		}
+
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($interface = DBfetch($res)) {
-			if (!is_null($options['countOutput'])) {
-				if (!is_null($options['groupCount'])) {
+			if ($options['countOutput']) {
+				if ($options['groupCount']) {
 					$result[] = $interface;
 				}
 				else {
@@ -170,7 +176,7 @@ class CHostInterface extends CApiService {
 			}
 		}
 
-		if (!is_null($options['countOutput'])) {
+		if ($options['countOutput']) {
 			return $result;
 		}
 
@@ -180,10 +186,53 @@ class CHostInterface extends CApiService {
 		}
 
 		// removing keys (hash -> array)
-		if (is_null($options['preservekeys'])) {
+		if (!$options['preservekeys']) {
 			$result = zbx_cleanHashes($result);
 		}
 
+		// Moving additional fields to separate object.
+		if ($this->outputIsRequested('details', $options['output'])) {
+			foreach ($result as &$value) {
+				$snmp_fields = ['version', 'bulk', 'community', 'securityname', 'securitylevel', 'authpassphrase',
+					'privpassphrase', 'authprotocol', 'privprotocol', 'contextname'
+				];
+
+				$interface_type = $value['type'];
+
+				if (!$this->outputIsRequested('type', $options['output'])) {
+					unset($value['type']);
+				}
+
+				$details = [];
+
+				// Handle SNMP related fields.
+				if ($interface_type == INTERFACE_TYPE_SNMP) {
+					foreach ($snmp_fields as $field_name) {
+						$details[$field_name] = $value[$field_name];
+						unset($value[$field_name]);
+					}
+
+					if ($details['version'] == SNMP_V1 || $details['version'] == SNMP_V2C) {
+						foreach (['securityname', 'securitylevel', 'authpassphrase', 'privpassphrase', 'authprotocol',
+								'privprotocol', 'contextname'] as $snmp_field_name) {
+							unset($details[$snmp_field_name]);
+						}
+					}
+					else {
+						unset($details['community']);
+					}
+				}
+				else {
+					foreach ($snmp_fields as $field_name) {
+						unset($value[$field_name]);
+					}
+				}
+
+				$value['details'] = $details;
+			}
+			unset($value);
+		}
+
 		return $result;
 	}
 
@@ -231,6 +280,7 @@ class CHostInterface extends CApiService {
 			'preservekeys' => true
 		]);
 
+		$check_have_items = [];
 		foreach ($interfaces as &$interface) {
 			if (!check_db_fields($interfaceDBfields, $interface)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
@@ -246,6 +296,10 @@ class CHostInterface extends CApiService {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Cannot switch host for interface.'));
 				}
 
+				if (array_key_exists('type', $interface) && $interface['type'] != $dbInterface['type']) {
+					$check_have_items[] = $interface['interfaceid'];
+				}
+
 				$interface['hostid'] = $dbInterface['hostid'];
 
 				// we check all fields on "updated" interface
@@ -261,7 +315,7 @@ class CHostInterface extends CApiService {
 					$interface['type'] = INTERFACE_TYPE_UNKNOWN;
 				}
 				elseif (!isset($interface['type'])) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to method.'));
+					self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
 				}
 			}
 
@@ -303,8 +357,6 @@ class CHostInterface extends CApiService {
 				$this->checkPort($interface);
 			}
 
-			$this->checkBulk($interface);
-
 			if ($update) {
 				$interface = $updInterface;
 			}
@@ -314,16 +366,104 @@ class CHostInterface extends CApiService {
 		// check if any of the affected hosts are discovered
 		if ($update) {
 			$interfaces = $this->extendObjects('interface', $interfaces, ['hostid']);
+
+			if ($check_have_items) {
+				$this->checkIfInterfaceHasItems($check_have_items);
+			}
 		}
 		$this->checkValidator(zbx_objectValues($interfaces, 'hostid'), new CHostNormalValidator([
 			'message' => _('Cannot update interface for discovered host "%1$s".')
 		]));
 	}
 
+	/**
+	 * Check SNMP related inputs.
+	 *
+	 * @param array $interfaces
+	 */
+	protected function checkSnmpInput(array $interfaces) {
+		foreach ($interfaces as $interface) {
+			if (!array_key_exists('type', $interface) || $interface['type'] != INTERFACE_TYPE_SNMP) {
+				continue;
+			}
+
+			if (!array_key_exists('details', $interface)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+			}
+
+			$this->checkSnmpVersion($interface);
+
+			$this->checkSnmpCommunity($interface);
+
+			$this->checkSnmpBulk($interface);
+
+			$this->checkSnmpSecurityLevel($interface);
+
+			$this->checkSnmpAuthProtocol($interface);
+
+			$this->checkSnmpPrivProtocol($interface);
+		}
+	}
+
+	/**
+	 * Sanitize SNMP fields by version.
+	 *
+	 * @param array $interfaces
+	 *
+	 * @return array
+	 */
+	protected function sanitizeSnmpFields(array $interfaces): array {
+		$default_fields = [
+			'community' => '',
+			'securityname' => '',
+			'securitylevel' => DB::getDefault('interface_snmp', 'securitylevel'),
+			'authpassphrase' => '',
+			'privpassphrase' => '',
+			'authprotocol' => DB::getDefault('interface_snmp', 'authprotocol'),
+			'privprotocol' => DB::getDefault('interface_snmp', 'privprotocol'),
+			'contextname' => ''
+		];
+
+		foreach ($interfaces as &$interface) {
+			if ($interface['version'] == SNMP_V1 || $interface['version'] == SNMP_V2C) {
+				unset($interface['securityname'], $interface['securitylevel'], $interface['authpassphrase'],
+					$interface['privpassphrase'], $interface['authprotocol'], $interface['privprotocol'],
+					$interface['contextname']
+				);
+			}
+			else {
+				unset($interface['community']);
+			}
+
+			$interface = $interface + $default_fields;
+		}
+
+		return $interfaces;
+	}
+
+	/**
+	 * Create SNMP interfaces.
+	 *
+	 * @param array $interfaces
+	 */
+	protected function createSnmpInterfaceDetails(array $interfaces) {
+		if (count($interfaces)) {
+			if (count(array_column($interfaces, 'interfaceid')) != count($interfaces)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+			}
+
+			$interfaces = $this->sanitizeSnmpFields($interfaces);
+
+			foreach ($interfaces as $interface) {
+				DB::insert('interface_snmp', [$interface], false);
+			}
+		}
+	}
+
 	/**
 	 * Add interfaces.
 	 *
-	 * @param array $interfaces multidimensional array with Interfaces data
+	 * @param array $interfaces  multidimensional array with Interfaces data
 	 *
 	 * @return array
 	 */
@@ -331,17 +471,79 @@ class CHostInterface extends CApiService {
 		$interfaces = zbx_toArray($interfaces);
 
 		$this->checkInput($interfaces, __FUNCTION__);
+		$this->checkSnmpInput($interfaces);
 		$this->checkMainInterfacesOnCreate($interfaces);
 
-		$interfaceIds = DB::insert('interface', $interfaces);
+		$interfaceids = DB::insert('interface', $interfaces);
 
-		return ['interfaceids' => $interfaceIds];
+		$snmp_interfaces = [];
+		foreach ($interfaceids as $key => $id) {
+			if ($interfaces[$key]['type'] == INTERFACE_TYPE_SNMP) {
+				$snmp_interfaces[] = ['interfaceid' => $id] + $interfaces[$key]['details'];
+			}
+		}
+
+		$this->createSnmpInterfaceDetails($snmp_interfaces);
+
+		return ['interfaceids' => $interfaceids];
+	}
+
+	protected function updateInterfaces(array $interfaces): bool {
+		$data = [];
+
+		foreach ($interfaces as $interface) {
+			$data[] = [
+				'values' => $interface,
+				'where' => ['interfaceid' => $interface['interfaceid']]
+			];
+		}
+
+		DB::update('interface', $data);
+
+		return true;
+	}
+
+	protected function updateInterfaceDetails(array $interfaces): bool {
+		$db_interfaces = $this->get([
+			'output' => ['type', 'details'],
+			'interfaceids' => array_column($interfaces, 'interfaceid'),
+			'preservekeys' => true
+		]);
+		DB::delete('interface_snmp', ['interfaceid' => array_column($interfaces, 'interfaceid')]);
+
+		$snmp_interfaces = [];
+		foreach ($interfaces as $interface) {
+			$interfaceid = $interface['interfaceid'];
+
+			// Check new interface type or, if interface type not present, check type from db.
+			if ((!array_key_exists('type', $interface) && $db_interfaces[$interfaceid]['type'] != INTERFACE_TYPE_SNMP)
+					|| (array_key_exists('type', $interface) && $interface['type'] != INTERFACE_TYPE_SNMP)) {
+				continue;
+			}
+			else {
+				// Type is required for SNMP validation.
+				$interface['type'] = INTERFACE_TYPE_SNMP;
+			}
+
+			// Merge details with db values or set only values from db.
+			$interface['details'] = array_key_exists('details', $interface)
+				? $interface['details'] + $db_interfaces[$interfaceid]['details']
+				: $db_interfaces[$interfaceid]['details'];
+
+			$this->checkSnmpInput([$interface]);
+
+			$snmp_interfaces[] = ['interfaceid' => $interfaceid] + $interface['details'];
+		}
+
+		$this->createSnmpInterfaceDetails($snmp_interfaces);
+
+		return true;
 	}
 
 	/**
 	 * Update interfaces.
 	 *
-	 * @param array $interfaces multidimensional array with Interfaces data
+	 * @param array $interfaces   multidimensional array with Interfaces data
 	 *
 	 * @return array
 	 */
@@ -351,28 +553,11 @@ class CHostInterface extends CApiService {
 		$this->checkInput($interfaces, __FUNCTION__);
 		$this->checkMainInterfacesOnUpdate($interfaces);
 
-		$data = [];
-		foreach ($interfaces as $interface) {
-			$data[] = [
-				'values' => $interface,
-				'where' => ['interfaceid' => $interface['interfaceid']]
-			];
-		}
-		DB::update('interface', $data);
+		$this->updateInterfaces($interfaces);
 
-		return ['interfaceids' => zbx_objectValues($interfaces, 'interfaceid')];
-	}
+		$this->updateInterfaceDetails($interfaces);
 
-	protected function clearValues(array $interface) {
-		if (isset($interface['port']) && $interface['port'] != '') {
-			$interface['port'] = ltrim($interface['port'], '0');
-
-			if ($interface['port'] == '') {
-				$interface['port'] = 0;
-			}
-		}
-
-		return $interface;
+		return ['interfaceids' => array_column($interfaces, 'interfaceid')];
 	}
 
 	/**
@@ -404,6 +589,7 @@ class CHostInterface extends CApiService {
 		$this->checkMainInterfacesOnDelete($interfaceids);
 
 		DB::delete('interface', ['interfaceid' => $interfaceids]);
+		DB::delete('interface_snmp', ['interfaceid' => $interfaceids]);
 
 		return ['interfaceids' => $interfaceids];
 	}
@@ -437,21 +623,21 @@ class CHostInterface extends CApiService {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
 			}
 
-			$this->checkDns($interface);
-			$this->checkIp($interface);
-			$this->checkPort($interface);
-			$this->checkBulk($interface);
+			$filter = [
+				'hostid' => $data['hostids'],
+				'ip' => $interface['ip'],
+				'dns' => $interface['dns'],
+				'port' => $interface['port']
+			];
+
+			if (array_key_exists('bulk', $interface)) {
+				$filter['bulk'] = $interface['bulk'];
+			}
 
 			// check main interfaces
 			$interfacesToRemove = API::getApiService()->select($this->tableName(), [
 				'output' => ['interfaceid'],
-				'filter' => [
-					'hostid' => $data['hostids'],
-					'ip' => $interface['ip'],
-					'dns' => $interface['dns'],
-					'port' => $interface['port'],
-					'bulk' => $interface['bulk']
-				]
+				'filter' => $filter
 			]);
 			if ($interfacesToRemove) {
 				$this->checkMainInterfacesOnDelete(zbx_objectValues($interfacesToRemove, 'interfaceid'));
@@ -483,8 +669,7 @@ class CHostInterface extends CApiService {
 					'hostid' => $data['hostids'],
 					'ip' => $interface['ip'],
 					'dns' => $interface['dns'],
-					'port' => $interface['port'],
-					'bulk' => $interface['bulk']
+					'port' => $interface['port']
 				],
 				'editable' => true,
 				'preservekeys' => true
@@ -506,7 +691,7 @@ class CHostInterface extends CApiService {
 	/**
 	 * Replace existing interfaces with input interfaces.
 	 *
-	 * @param $host
+	 * @param array $host
 	 */
 	public function replaceHostInterfaces(array $host) {
 		if (isset($host['interfaces']) && !is_null($host['interfaces'])) {
@@ -514,50 +699,66 @@ class CHostInterface extends CApiService {
 
 			$this->checkHostInterfaces($host['interfaces'], $host['hostid']);
 
-			$interfacesToDelete = API::HostInterface()->get([
-				'hostids' => $host['hostid'],
-				'output' => API_OUTPUT_EXTEND,
-				'preservekeys' => true,
-				'nopermissions' => true
+			$interfaces_delete = DB::select('interface', [
+				'output' => [],
+				'filter' => ['hostid' => $host['hostid']],
+				'preservekeys' => true
 			]);
 
-			$interfacesToAdd = [];
-			$interfacesToUpdate = [];
+			$interfaces_add = [];
+			$interfaces_update = [];
 
 			foreach ($host['interfaces'] as $interface) {
 				$interface['hostid'] = $host['hostid'];
 
-				if (!isset($interface['interfaceid'])) {
-					$interfacesToAdd[] = $interface;
+				if (!array_key_exists('interfaceid', $interface)) {
+					$interfaces_add[] = $interface;
 				}
-				elseif (isset($interfacesToDelete[$interface['interfaceid']])) {
-					$interfacesToUpdate[] = $interface;
-					unset($interfacesToDelete[$interface['interfaceid']]);
+				elseif (array_key_exists($interface['interfaceid'], $interfaces_delete)) {
+					$interfaces_update[] = $interface;
+					unset($interfaces_delete[$interface['interfaceid']]);
 				}
 			}
 
-			if ($interfacesToUpdate) {
-				API::HostInterface()->checkInput($interfacesToUpdate, 'update');
+			if ($interfaces_update) {
+				$this->checkInput($interfaces_update, 'update');
 
-				$data = [];
-				foreach ($interfacesToUpdate as $interface) {
-					$data[] = [
-						'values' => $interface,
-						'where' => ['interfaceid' => $interface['interfaceid']]
-					];
+				$this->updateInterfaces($interfaces_update);
+
+				$this->updateInterfaceDetails($interfaces_update);
+			}
+
+			if ($interfaces_add) {
+				$this->checkInput($interfaces_add, 'create');
+				$interfaceids = DB::insert('interface', $interfaces_add);
+
+				$this->checkSnmpInput($interfaces_add);
+
+				$snmp_interfaces = [];
+				foreach ($interfaceids as $key => $id) {
+					if ($interfaces_add[$key]['type'] == INTERFACE_TYPE_SNMP) {
+						$snmp_interfaces[] = ['interfaceid' => $id] + $interfaces_add[$key]['details'];
+					}
 				}
-				DB::update('interface', $data);
+
+				$this->createSnmpInterfaceDetails($snmp_interfaces);
+
+				foreach ($host['interfaces'] as &$interface) {
+					if (!array_key_exists('interfaceid', $interface)) {
+						$interface['interfaceid'] = array_shift($interfaceids);
+					}
+				}
+				unset($interface);
 			}
 
-			if ($interfacesToAdd) {
-				$this->checkInput($interfacesToAdd, 'create');
-				DB::insert('interface', $interfacesToAdd);
+			if ($interfaces_delete) {
+				$this->delete(array_keys($interfaces_delete));
 			}
 
-			if ($interfacesToDelete) {
-				$this->delete(zbx_objectValues($interfacesToDelete, 'interfaceid'));
-			}
+			return ['interfaceids' => array_column($host['interfaces'], 'interfaceid')];
 		}
+
+		return ['interfaceids' => []];
 	}
 
 	/**
@@ -578,7 +779,7 @@ class CHostInterface extends CApiService {
 		if (!preg_match('/^'.ZBX_PREG_DNS_FORMAT.'$/', $interface['dns'])
 				&& $user_macro_parser->parse($interface['dns']) != CParser::PARSE_SUCCESS) {
 			self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Incorrect interface DNS parameter "%s" provided.', $interface['dns'])
+				_s('Incorrect interface DNS parameter "%1$s" provided.', $interface['dns'])
 			);
 		}
 	}
@@ -603,9 +804,10 @@ class CHostInterface extends CApiService {
 			return;
 		}
 
-		$ipValidator = new CIPValidator();
-		if (!$ipValidator->validate($interface['ip'])) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, $ipValidator->getError());
+		$ip_parser = new CIPParser(['v6' => ZBX_HAVE_IPV6]);
+
+		if ($ip_parser->parse($interface['ip']) != CParser::PARSE_SUCCESS) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid IP address "%1$s".', $interface['ip']));
 		}
 	}
 
@@ -621,7 +823,9 @@ class CHostInterface extends CApiService {
 			self::exception(ZBX_API_ERROR_PARAMETERS, _('Port cannot be empty for host interface.'));
 		}
 		elseif (!validatePortNumberOrMacro($interface['port'])) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect interface port "%s" provided.', $interface['port']));
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Incorrect interface port "%1$s" provided.', $interface['port'])
+			);
 		}
 	}
 
@@ -630,30 +834,23 @@ class CHostInterface extends CApiService {
 	 *
 	 * @throws APIException if the user doesn't have write permissions for the given hosts
 	 *
-	 * @param array $hostIds	an array of host IDs
+	 * @param array $hostids	an array of host IDs
 	 */
-	protected function checkHostPermissions(array $hostIds) {
-		if (!API::Host()->isWritable($hostIds)) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
-		}
-	}
+	protected function checkHostPermissions(array $hostids) {
+		if ($hostids) {
+			$hostids = array_unique($hostids);
 
-	/**
-	 * Validates interface "bulk" field.
-	 * For SNMP interfaces bulk value should be either 0 (disabled) or 1 (enabled).
-	 * For other non-SNMP interfaces bulk value should be 1 (default).
-	 *
-	 * @throws APIException if bulk field is incorrect.
-	 *
-	 * @param array $interface
-	 */
-	protected function checkBulk(array $interface) {
-		if ($interface['type'] !== null && (($interface['type'] != INTERFACE_TYPE_SNMP && isset($interface['bulk'])
-				&& $interface['bulk'] != SNMP_BULK_ENABLED)
-				|| ($interface['type'] == INTERFACE_TYPE_SNMP && isset($interface['bulk'])
-					&& (zbx_empty($interface['bulk'])
-						|| ($interface['bulk'] != SNMP_BULK_DISABLED && $interface['bulk'] != SNMP_BULK_ENABLED))))) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect bulk value for interface.'));
+			$count = API::Host()->get([
+				'countOutput' => true,
+				'hostids' => $hostids,
+				'editable' => true
+			]);
+
+			if ($count != count($hostids)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
 		}
 	}
 
@@ -702,32 +899,20 @@ class CHostInterface extends CApiService {
 		$this->checkMainInterfaces($interfaces);
 	}
 
+	/**
+	 * Prepares data to validate main interface for every interface type. Executes main interface validation.
+	 *
+	 * @param array $interfaces                     Array of interfaces to validate.
+	 * @param int   $interfaces[]['hostid']         Updated interface's hostid.
+	 * @param int   $interfaces[]['interfaceid']    Updated interface's interfaceid.
+	 *
+	 * @throws APIException
+	 */
 	private function checkMainInterfacesOnUpdate(array $interfaces) {
-		$interfaceidsWithoutHostIds = [];
-
-		// gather all hostids where interfaces should be checked
-		foreach ($interfaces as $interface) {
-			if (isset($interface ['type']) || isset($interface['main'])) {
-				if (isset($interface['hostid'])) {
-					$hostids[$interface['hostid']] = $interface['hostid'];
-				}
-				else {
-					$interfaceidsWithoutHostIds[] = $interface['interfaceid'];
-				}
-			}
-		}
-
-		// gather missing host ids
-		$hostIds = [];
-		if ($interfaceidsWithoutHostIds) {
-			$dbResult = DBselect('SELECT DISTINCT i.hostid FROM interface i WHERE '.dbConditionInt('i.interfaceid', $interfaceidsWithoutHostIds));
-			while ($hostData = DBfetch($dbResult)) {
-				$hostIds[$hostData['hostid']] = $hostData['hostid'];
-			}
-		}
+		$hostids = array_keys(array_flip(zbx_objectValues($interfaces, 'hostid')));
 
 		$dbInterfaces = API::HostInterface()->get([
-			'hostids' => $hostIds,
+			'hostids' => $hostids,
 			'output' => ['hostid', 'main', 'type'],
 			'preservekeys' => true,
 			'nopermissions' => true
@@ -738,7 +923,7 @@ class CHostInterface extends CApiService {
 			if (isset($dbInterfaces[$interface['interfaceid']])) {
 				$dbInterfaces[$interface['interfaceid']] = array_merge(
 					$dbInterfaces[$interface['interfaceid']],
-					$interfaces[$interface['interfaceid']]
+					$interface
 				);
 			}
 		}
@@ -834,10 +1019,141 @@ class CHostInterface extends CApiService {
 		}
 	}
 
+	/**
+	 * Check if SNMP version is valid. Valid versions: SNMP_V1, SNMP_V2C, SNMP_V3.
+	 *
+	 * @param array $interface
+	 *
+	 * @throws APIException if "version" value is incorrect.
+	 */
+	protected function checkSnmpVersion(array $interface) {
+		if (!array_key_exists('version', $interface['details'])
+				|| !in_array($interface['details']['version'], [SNMP_V1, SNMP_V2C, SNMP_V3])) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+		}
+	}
+
+	/**
+	 * Check SNMP community. For SNMPv1 and SNMPv2c it required.
+	 *
+	 * @param array $interface
+	 *
+	 * @throws APIException if "community" value is incorrect.
+	 */
+	protected function checkSnmpCommunity(array $interface) {
+		if (($interface['details']['version'] == SNMP_V1 || $interface['details']['version'] == SNMP_V2C)
+				&& (!array_key_exists('community', $interface['details'])
+					|| $interface['details']['community'] === '')) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+		}
+	}
+
+	/**
+	 * Validates SNMP interface "bulk" field.
+	 *
+	 * @param array $interface
+	 *
+	 * @throws APIException if "bulk" value is incorrect.
+	 */
+	protected function checkSnmpBulk(array $interface) {
+		if ($interface['type'] !== null && (($interface['type'] != INTERFACE_TYPE_SNMP
+					&& isset($interface['details']['bulk']) && $interface['details']['bulk'] != SNMP_BULK_ENABLED)
+					|| ($interface['type'] == INTERFACE_TYPE_SNMP && isset($interface['details']['bulk'])
+						&& (zbx_empty($interface['details']['bulk'])
+							|| ($interface['details']['bulk'] != SNMP_BULK_DISABLED
+							&& $interface['details']['bulk'] != SNMP_BULK_ENABLED))))) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect bulk value for interface.'));
+		}
+	}
+
+	/**
+	 * Check SNMP Security level field.
+	 *
+	 * @param array $interface
+	 * @param array $interface['details']
+	 * @param array $interface['details']['version']        SNMP version
+	 * @param array $interface['details']['securitylevel']  SNMP security level
+	 *
+	 * @throws APIException if "securitylevel" value is incorrect.
+	 */
+	protected function checkSnmpSecurityLevel(array $interface) {
+		if ($interface['details']['version'] == SNMP_V3 && (array_key_exists('securitylevel', $interface['details'])
+					&& !in_array($interface['details']['securitylevel'], [ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV,
+						ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV, ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV]))) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+		}
+	}
+
+	/**
+	 * Check SNMP authentication  protocol.
+	 *
+	 * @param array $interface
+	 * @param array $interface['details']
+	 * @param array $interface['details']['version']       SNMP version
+	 * @param array $interface['details']['authprotocol']  SNMP authentication protocol
+	 *
+	 * @throws APIException if "authprotocol" value is incorrect.
+	 */
+	protected function checkSnmpAuthProtocol(array $interface) {
+		if ($interface['details']['version'] == SNMP_V3 && (array_key_exists('authprotocol', $interface['details'])
+					&& !in_array($interface['details']['authprotocol'], [ITEM_AUTHPROTOCOL_MD5,
+						ITEM_AUTHPROTOCOL_SHA]))) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+		}
+	}
+
+	/**
+	 * Check SNMP Privacy protocol.
+	 *
+	 * @param array $interface
+	 * @param array $interface['details']
+	 * @param array $interface['details']['version']       SNMP version
+	 * @param array $interface['details']['privprotocol']  SNMP privacy protocol
+	 *
+	 * @throws APIException if "privprotocol" value is incorrect.
+	 */
+	protected function checkSnmpPrivProtocol(array $interface) {
+		if ($interface['details']['version'] == SNMP_V3 && (array_key_exists('privprotocol', $interface['details'])
+				&& !in_array($interface['details']['privprotocol'], [ITEM_PRIVPROTOCOL_DES, ITEM_PRIVPROTOCOL_AES]))) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,  _('Incorrect arguments passed to function.'));
+		}
+	}
+
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
 		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
 
-		if ($options['countOutput'] === null && $options['selectHosts'] !== null) {
+		if ($this->outputIsRequested('details', $options['output'])) {
+			// Select interface type to check show details array or not.
+			$sqlParts = $this->addQuerySelect('hi.type', $sqlParts);
+
+			$sqlParts = $this->addQuerySelect(dbConditionCoalesce('his.version', SNMP_V2C, 'version'), $sqlParts);
+			$sqlParts = $this->addQuerySelect(dbConditionCoalesce('his.bulk', SNMP_BULK_ENABLED, 'bulk'), $sqlParts);
+			$sqlParts = $this->addQuerySelect(dbConditionCoalesce('his.community', '', 'community'), $sqlParts);
+			$sqlParts = $this->addQuerySelect(dbConditionCoalesce('his.securityname', '', 'securityname'), $sqlParts);
+			$sqlParts = $this->addQuerySelect(
+				dbConditionCoalesce('his.securitylevel', ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV, 'securitylevel'),
+				$sqlParts
+			);
+			$sqlParts = $this->addQuerySelect(
+				dbConditionCoalesce('his.authpassphrase', '', 'authpassphrase'),
+				$sqlParts
+			);
+			$sqlParts = $this->addQuerySelect(
+				dbConditionCoalesce('his.privpassphrase', '', 'privpassphrase'),
+				$sqlParts
+			);
+			$sqlParts = $this->addQuerySelect(
+				dbConditionCoalesce('his.authprotocol', ITEM_AUTHPROTOCOL_MD5, 'authprotocol'),
+				$sqlParts
+			);
+			$sqlParts = $this->addQuerySelect(
+				dbConditionCoalesce('his.privprotocol', ITEM_PRIVPROTOCOL_DES, 'privprotocol'),
+				$sqlParts
+			);
+			$sqlParts = $this->addQuerySelect(dbConditionCoalesce('his.contextname', '', 'contextname'), $sqlParts);
+		}
+
+		if (!$options['countOutput'] && $options['selectHosts'] !== null) {
 			$sqlParts = $this->addQuerySelect('hi.hostid', $sqlParts);
 		}
 
@@ -884,8 +1200,10 @@ class CHostInterface extends CApiService {
 					'groupCount' => true
 				]);
 				$items = zbx_toHash($items, 'interfaceid');
-				foreach ($result as $interfaceId => $interface) {
-					$result[$interfaceId]['items'] = isset($items[$interfaceId]) ? $items[$interfaceId]['rowscount'] : 0;
+				foreach ($result as $interfaceid => $interface) {
+					$result[$interfaceid]['items'] = array_key_exists($interfaceid, $items)
+						? $items[$interfaceid]['rowscount']
+						: '0';
 				}
 			}
 		}
