@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -58,40 +58,24 @@ abstract class CMapElement extends CApiService {
 			}
 
 			if (array_key_exists('urls', $selement)) {
+				$url_validate_options = ['allow_user_macro' => false];
+				if ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_HOST) {
+					$url_validate_options['allow_inventory_macro'] = INVENTORY_URL_MACRO_HOST;
+				}
+				elseif ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_TRIGGER) {
+					$url_validate_options['allow_inventory_macro'] = INVENTORY_URL_MACRO_TRIGGER;
+				}
+				else {
+					$url_validate_options['allow_inventory_macro'] = INVENTORY_URL_MACRO_NONE;
+				}
+
 				foreach ($selement['urls'] as $url_data) {
-					if (!CHtmlUrlValidator::validate($url_data['url'], false)) {
+					if (!CHtmlUrlValidator::validate($url_data['url'], $url_validate_options)) {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _('Wrong value for url field.'));
 					}
 				}
 			}
 
-			/*
-			 * @deprecated  As of version 3.4, use elements data array.
-			 */
-			if (array_key_exists('elementid', $selement)) {
-				$this->deprecated('elementid is deprecated parameter.');
-
-				switch ($selement['elementtype']) {
-					case SYSMAP_ELEMENT_TYPE_HOST_GROUP:
-						$selement['elements'][0]['groupid'] = $selement['elementid'];
-						break;
-
-					case SYSMAP_ELEMENT_TYPE_HOST:
-						$selement['elements'][0]['hostid'] = $selement['elementid'];
-						break;
-
-					case SYSMAP_ELEMENT_TYPE_TRIGGER:
-						$selement['elements'][0]['triggerid'] = $selement['elementid'];
-						break;
-
-					case SYSMAP_ELEMENT_TYPE_MAP:
-						$selement['elements'][0]['sysmapid'] = $selement['elementid'];
-						break;
-				}
-
-				unset($selement['elementid']);
-			}
-
 			if ($update && array_key_exists('selementid', $selement)
 						&& array_key_exists($selement['selementid'], $db_selements)) {
 				$db_selement = $db_selements[$selement['selementid']];
@@ -124,12 +108,6 @@ abstract class CMapElement extends CApiService {
 				}
 			}
 
-			if (array_key_exists('elements', $selement)) {
-				if (!is_array($selement['elements'])) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
-				}
-			}
-
 			if ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_TRIGGER) {
 				foreach ($selement['elements'] as $element) {
 					if (!array_key_exists('triggerid', $element)) {
@@ -195,9 +173,9 @@ abstract class CMapElement extends CApiService {
 			}
 
 			if (isset($selement['iconid_off']) && $selement['iconid_off'] == 0) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('No icon for map element "%s".', array_key_exists('label', $selement) ? $selement['label'] : '')
-				);
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('No icon for map element ""%1$s".',
+					array_key_exists('label', $selement) ? $selement['label'] : ''
+				));
 			}
 
 			if ($create) {
@@ -208,9 +186,7 @@ abstract class CMapElement extends CApiService {
 
 		// check permissions to used objects
 		if (!CMapHelper::checkSelementPermissions($selements)) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS,
-					_('No permissions to referred object or it does not exist!')
-			);
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
 		}
 
 		return $update ? $db_selements : true;
