@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -139,7 +139,7 @@ class CUserGroup extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($usrgrp = DBfetch($res)) {
 			if ($options['countOutput']) {
 				$result = $usrgrp['rowscount'];
@@ -932,10 +932,14 @@ class CUserGroup extends CApiService {
 		if ($options['selectUsers'] !== null && $options['selectUsers'] != API_OUTPUT_COUNT) {
 			$relationMap = $this->createRelationMap($result, 'usrgrpid', 'userid', 'users_groups');
 
+			$get_access = ($this->outputIsRequested('gui_access', $options['selectUsers'])
+				|| $this->outputIsRequested('debug_mode', $options['selectUsers'])
+				|| $this->outputIsRequested('users_status', $options['selectUsers'])) ? true : null;
+
 			$dbUsers = API::User()->get([
 				'output' => $options['selectUsers'],
 				'userids' => $relationMap->getRelatedIds(),
-				'getAccess' => ($options['selectUsers'] == API_OUTPUT_EXTEND) ? true : null,
+				'getAccess' => $get_access,
 				'preservekeys' => true
 			]);
 
