@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -46,7 +46,6 @@ class CUserMacro extends CApiService {
 	 */
 	public function get($options = []) {
 		$result = [];
-		$userType = self::$userData['type'];
 		$userid = self::$userData['userid'];
 
 		$sqlParts = [
@@ -72,22 +71,22 @@ class CUserMacro extends CApiService {
 			'globalmacroids'			=> null,
 			'templateids'				=> null,
 			'globalmacro'				=> null,
-			'editable'					=> null,
+			'editable'					=> false,
 			'nopermissions'				=> null,
 			// filter
 			'filter'					=> null,
 			'search'					=> null,
 			'searchByAny'				=> null,
-			'startSearch'				=> null,
-			'excludeSearch'				=> null,
+			'startSearch'				=> false,
+			'excludeSearch'				=> false,
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
 			'selectGroups'				=> null,
 			'selectHosts'				=> null,
 			'selectTemplates'			=> null,
-			'countOutput'				=> null,
-			'preservekeys'				=> null,
+			'countOutput'				=> false,
+			'preservekeys'				=> false,
 			'sortfield'					=> '',
 			'sortorder'					=> '',
 			'limit'						=> null
@@ -95,8 +94,8 @@ class CUserMacro extends CApiService {
 		$options = zbx_array_merge($defOptions, $options);
 
 		// editable + PERMISSION CHECK
-		if ($userType != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
-			if (!is_null($options['editable']) && !is_null($options['globalmacro'])) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
+			if ($options['editable'] && !is_null($options['globalmacro'])) {
 				return [];
 			}
 			else {
@@ -167,22 +166,6 @@ class CUserMacro extends CApiService {
 			$sqlParts['where']['hht'] = 'hm.hostid=ht.hostid';
 		}
 
-		// search
-		if (is_array($options['search'])) {
-			zbx_db_search('hostmacro hm', $options, $sqlParts);
-			zbx_db_search('globalmacro gm', $options, $sqlPartsGlobal);
-		}
-
-		// filter
-		if (is_array($options['filter'])) {
-			if (isset($options['filter']['macro'])) {
-				zbx_value2array($options['filter']['macro']);
-
-				$sqlParts['where'][] = dbConditionString('hm.macro', $options['filter']['macro']);
-				$sqlPartsGlobal['where'][] = dbConditionString('gm.macro', $options['filter']['macro']);
-			}
-		}
-
 		// sorting
 		$sqlParts = $this->applyQuerySortOptions('hostmacro', 'hm', $options, $sqlParts);
 		$sqlPartsGlobal = $this->applyQuerySortOptions('globalmacro', 'gm', $options, $sqlPartsGlobal);
@@ -195,8 +178,9 @@ class CUserMacro extends CApiService {
 
 		// init GLOBALS
 		if (!is_null($options['globalmacro'])) {
+			$sqlPartsGlobal = $this->applyQueryFilterOptions('globalmacro', 'gm', $options, $sqlPartsGlobal);
 			$sqlPartsGlobal = $this->applyQueryOutputOptions('globalmacro', 'gm', $options, $sqlPartsGlobal);
-			$res = DBselect($this->createSelectQueryFromParts($sqlPartsGlobal), $sqlPartsGlobal['limit']);
+			$res = DBselect(self::createSelectQueryFromParts($sqlPartsGlobal), $sqlPartsGlobal['limit']);
 			while ($macro = DBfetch($res)) {
 				if ($options['countOutput']) {
 					$result = $macro['rowscount'];
@@ -208,8 +192,9 @@ class CUserMacro extends CApiService {
 		}
 		// init HOSTS
 		else {
+			$sqlParts = $this->applyQueryFilterOptions('hostmacro', 'hm', $options, $sqlParts);
 			$sqlParts = $this->applyQueryOutputOptions('hostmacro', 'hm', $options, $sqlParts);
-			$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+			$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 			while ($macro = DBfetch($res)) {
 				if ($options['countOutput']) {
 					$result = $macro['rowscount'];
@@ -220,17 +205,17 @@ class CUserMacro extends CApiService {
 			}
 		}
 
-		if (!is_null($options['countOutput'])) {
+		if ($options['countOutput']) {
 			return $result;
 		}
 
 		if ($result) {
 			$result = $this->addRelatedObjects($options, $result);
-			$result = $this->unsetExtraFields($result, ['hostid'], $options['output']);
+			$result = $this->unsetExtraFields($result, ['hostid', 'type'], $options['output']);
 		}
 
 		// removing keys (hash -> array)
-		if (is_null($options['preservekeys'])) {
+		if (!$options['preservekeys']) {
 			$result = zbx_cleanHashes($result);
 		}
 
@@ -238,165 +223,245 @@ class CUserMacro extends CApiService {
 	}
 
 	/**
-	 * Validates the input parameters for the createGlobal() method.
-	 *
-	 * @param array $globalmacros
-	 *
-	 * @throws APIException if the input is invalid.
-	 */
-	protected function validateCreateGlobal(array $globalmacros) {
-		$this->checkGlobalMacrosPermissions(_('Only Super Admins can create global macros.'));
-
-		foreach ($globalmacros as $globalmacro) {
-			$this->checkMacro($globalmacro);
-			$this->checkUnsupportedFields('globalmacro', $globalmacro,
-				_s('Wrong fields for macro "%1$s".', $globalmacro['macro'])
-			);
-		}
-
-		$this->checkDuplicateMacros($globalmacros);
-		$this->checkIfGlobalMacrosDontRepeat($globalmacros);
-	}
-
-	/**
-	 * Add global macros.
-	 *
 	 * @param array $globalmacros
 	 *
 	 * @return array
 	 */
 	public function createGlobal(array $globalmacros) {
-		$globalmacros = zbx_toArray($globalmacros);
-
 		$this->validateCreateGlobal($globalmacros);
 
-		$globalmacroids = DB::insert('globalmacro', $globalmacros);
+		$globalmacroids = DB::insertBatch('globalmacro', $globalmacros);
 
-		$globalmacroid = reset($globalmacroids);
-
-		foreach ($globalmacros as $globalmacro) {
-			$value = array_key_exists('value', $globalmacro) ? $globalmacro['value'] : '';
-
-			add_audit_ext(AUDIT_ACTION_ADD, AUDIT_RESOURCE_MACRO, $globalmacroid,
-				$globalmacro['macro'].' &rArr; '.$value, null, null, null
-			);
-
-			$globalmacroid = next($globalmacroids);
+		foreach ($globalmacros as $index => &$globalmacro) {
+			$globalmacro['globalmacroid'] = $globalmacroids[$index];
 		}
+		unset($globalmacro);
+
+		$this->addAuditBulk(AUDIT_ACTION_ADD, AUDIT_RESOURCE_MACRO, $globalmacros);
 
 		return ['globalmacroids' => $globalmacroids];
 	}
 
 	/**
-	 * Updates global macros.
+	 * @param array $globalmacros
 	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	private function validateCreateGlobal(array &$globalmacros) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
+		}
+
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['macro']], 'fields' => [
+			'macro' =>			['type' => API_USER_MACRO, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('globalmacro', 'macro')],
+			'value' =>			['type' => API_STRING_UTF8, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('globalmacro', 'value')],
+			'type' =>			['type' => API_INT32, 'in' => ZBX_MACRO_TYPE_TEXT.','.ZBX_MACRO_TYPE_SECRET],
+			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('globalmacro', 'description')]
+		]];
+		if (!CApiInputValidator::validate($api_input_rules, $globalmacros, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
+		$this->checkDuplicates(zbx_objectValues($globalmacros, 'macro'));
+	}
+
+	/**
 	 * @param array $globalmacros
 	 *
 	 * @return array
 	 */
 	public function updateGlobal(array $globalmacros) {
-		$globalmacros = zbx_toArray($globalmacros);
+		$this->validateUpdateGlobal($globalmacros, $db_globalmacros);
 
-		// validate macros
-		$this->checkGlobalMacrosPermissions(_('Only Super Admins can update global macros.'));
-
-		$required_fields = ['globalmacroid'];
+		$upd_globalmacros = [];
 
 		foreach ($globalmacros as $globalmacro) {
-			$missing_keys = array_diff($required_fields, array_keys($globalmacro));
+			$db_globalmacro = $db_globalmacros[$globalmacro['globalmacroid']];
 
-			if ($missing_keys) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('User macro missing parameters: %1$s', implode(', ', $missing_keys))
-				);
-			}
-		}
+			$upd_globalmacro = [];
 
-		$ex_globalmacros = $this->extendObjects('globalmacro', $globalmacros, ['macro', 'value']);
-
-		foreach ($ex_globalmacros as $ex_globalmacro) {
-			$this->checkMacro($ex_globalmacro);
-			$this->checkUnsupportedFields('globalmacro', $ex_globalmacro,
-				_s('Wrong fields for macro "%1$s".', $ex_globalmacro['macro']));
-		}
-
-		$this->checkDuplicateMacros($ex_globalmacros);
-		$this->checkIfGlobalMacrosExist(zbx_objectValues($globalmacros, 'globalmacroid'));
-		$this->checkIfGlobalMacrosDontRepeat($ex_globalmacros);
-
-		// update macros
-		$data = [];
-
-		foreach ($globalmacros as $globalmacro) {
-			$values = [];
-			foreach (['macro', 'value'] as $field) {
-				if (array_key_exists($field, $globalmacro)) {
-					$values[$field] = $globalmacro[$field];
+			// strings
+			foreach (['macro', 'value', 'description', 'type'] as $field_name) {
+				if (array_key_exists($field_name, $globalmacro)
+						&& $globalmacro[$field_name] !== $db_globalmacro[$field_name]) {
+					$upd_globalmacro[$field_name] = $globalmacro[$field_name];
 				}
 			}
 
-			if ($values) {
-				$data[] = [
-					'values'=> $values,
+			if (array_key_exists('type', $globalmacro) && $globalmacro['type'] != $db_globalmacro['type']
+					&& $db_globalmacro['type'] == ZBX_MACRO_TYPE_SECRET) {
+				$upd_globalmacro += ['value' => ''];
+			}
+
+			if ($upd_globalmacro) {
+				$upd_globalmacros[] = [
+					'values'=> $upd_globalmacro,
 					'where'=> ['globalmacroid' => $globalmacro['globalmacroid']]
 				];
 			}
 		}
 
-		DB::update('globalmacro', $data);
-
-		foreach ($ex_globalmacros as $ex_globalmacro) {
-			add_audit_ext(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_MACRO, $ex_globalmacro['globalmacroid'],
-				$ex_globalmacro['macro'].' &rArr; '.$ex_globalmacro['value'], null, null, null
-			);
+		if ($upd_globalmacros) {
+			DB::update('globalmacro', $upd_globalmacros);
 		}
 
+		$this->addAuditBulk(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_MACRO, $globalmacros, $db_globalmacros);
+
 		return ['globalmacroids' => zbx_objectValues($globalmacros, 'globalmacroid')];
 	}
 
 	/**
-	 * Validates the input parameters for the deleteGlobal() method.
+	 * Returns macro without spaces and curly braces.
 	 *
-	 * @param array $globalmacroids
+	 * @param string $macro
 	 *
-	 * @throws APIException if the input is invalid.
+	 * @return string
 	 */
-	protected function validateDeleteGlobal(array $globalmacroids) {
-		if (!$globalmacroids) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Empty input parameter.'));
+	private function trimMacro($macro) {
+		$user_macro_parser = new CUserMacroParser();
+
+		$user_macro_parser->parse($macro);
+
+		$macro = $user_macro_parser->getMacro();
+		$context = $user_macro_parser->getContext();
+		$regex = $user_macro_parser->getRegex();
+
+		if ($context !== null) {
+			$macro .= ':'.$context;
+		}
+		elseif ($regex !== null) {
+			$macro .= ':'.CUserMacroParser::REGEX_PREFIX.$regex;
 		}
 
-		$this->checkGlobalMacrosPermissions(_('Only Super Admins can delete global macros.'));
-		$this->checkIfGlobalMacrosExist($globalmacroids);
+		return $macro;
 	}
 
 	/**
-	 * Delete global macros.
+	 * @param array $globalmacros
+	 * @param array $db_globalmacros
 	 *
+	 * @throws APIException if the input is invalid
+	 */
+	private function validateUpdateGlobal(array &$globalmacros, array &$db_globalmacros = null) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
+		}
+
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['globalmacroid'], ['macro']], 'fields' => [
+			'globalmacroid' =>	['type' => API_ID, 'flags' => API_REQUIRED],
+			'macro' =>			['type' => API_USER_MACRO, 'length' => DB::getFieldLength('globalmacro', 'macro')],
+			'value' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('globalmacro', 'value')],
+			'type' =>			['type' => API_INT32, 'in' => ZBX_MACRO_TYPE_TEXT.','.ZBX_MACRO_TYPE_SECRET],
+			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('globalmacro', 'description')]
+		]];
+		if (!CApiInputValidator::validate($api_input_rules, $globalmacros, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
+		$db_globalmacros = DB::select('globalmacro', [
+			'output' => ['globalmacroid', 'macro', 'value', 'description', 'type'],
+			'globalmacroids' => zbx_objectValues($globalmacros, 'globalmacroid'),
+			'preservekeys' => true
+		]);
+
+		$macros = [];
+
+		foreach ($globalmacros as $globalmacro) {
+			if (!array_key_exists($globalmacro['globalmacroid'], $db_globalmacros)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
+
+			$db_globalmacro = $db_globalmacros[$globalmacro['globalmacroid']];
+
+			if (array_key_exists('macro', $globalmacro)
+					&& $this->trimMacro($globalmacro['macro']) !== $this->trimMacro($db_globalmacro['macro'])) {
+				$macros[] = $globalmacro['macro'];
+			}
+		}
+
+		if ($macros) {
+			$this->checkDuplicates($macros);
+		}
+	}
+
+	/**
+	 * Check for duplicated macros.
+	 *
+	 * @param array $macros
+	 *
+	 * @throws APIException if macros already exists.
+	 */
+	private function checkDuplicates(array $macros) {
+		$user_macro_parser = new CUserMacroParser();
+
+		$db_globalmacros = DB::select('globalmacro', [
+			'output' => ['macro']
+		]);
+
+		$uniq_macros = [];
+
+		foreach ($db_globalmacros as $db_globalmacro) {
+			$uniq_macros[$this->trimMacro($db_globalmacro['macro'])] = true;
+		}
+
+		foreach ($macros as $macro) {
+			$macro_orig = $macro;
+			$macro = $this->trimMacro($macro);
+
+			if (array_key_exists($macro, $uniq_macros)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Macro "%1$s" already exists.', $macro_orig));
+			}
+			$uniq_macros[$macro] = true;
+		}
+	}
+
+	/**
 	 * @param array $globalmacroids
 	 *
 	 * @return array
 	 */
 	public function deleteGlobal(array $globalmacroids) {
-		$this->validateDeleteGlobal($globalmacroids);
+		$this->validateDeleteGlobal($globalmacroids, $db_globalmacros);
 
-		$db_globalmacros = API::getApiService()->select('globalmacro', [
-			'output' => ['globalmacroid', 'macro', 'value'],
-			'globalmacroids' => $globalmacroids
-		]);
-
-		// delete macros
 		DB::delete('globalmacro', ['globalmacroid' => $globalmacroids]);
 
-		foreach ($db_globalmacros as $db_globalmacro) {
-			add_audit_ext(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_MACRO, $db_globalmacro['globalmacroid'],
-				$db_globalmacro['macro'].' &rArr; '.$db_globalmacro['value'], null, null, null
-			);
-		}
+		$this->addAuditBulk(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_MACRO, $db_globalmacros);
 
 		return ['globalmacroids' => $globalmacroids];
 	}
 
+	/**
+	 * @param array $globalmacroids
+	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	private function validateDeleteGlobal(array &$globalmacroids, array &$db_globalmacros = null) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
+		}
+
+		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
+		if (!CApiInputValidator::validate($api_input_rules, $globalmacroids, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
+		$db_globalmacros = DB::select('globalmacro', [
+			'output' => ['globalmacroid', 'macro'],
+			'globalmacroids' => $globalmacroids,
+			'preservekeys' => true
+		]);
+
+		foreach ($globalmacroids as $globalmacroid) {
+			if (!array_key_exists($globalmacroid, $db_globalmacros)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
+		}
+	}
+
 	/**
 	 * Validates the input parameters for the create() method.
 	 *
@@ -416,6 +481,7 @@ class CUserMacro extends CApiService {
 			$this->checkMacro($hostmacro);
 			$this->checkUnsupportedFields('hostmacro', $hostmacro,
 				_s('Wrong fields for macro "%1$s".', $hostmacro['macro']));
+			$this->checkMacroType($hostmacro);
 		}
 
 		$this->checkDuplicateMacros($hostmacros);
@@ -487,6 +553,7 @@ class CUserMacro extends CApiService {
 			$this->checkUnsupportedFields('hostmacro', $hostmacro,
 				_s('Wrong fields for macro "%1$s".', $hostmacro['macro'])
 			);
+			$this->checkMacroType($hostmacro);
 		}
 
 		$this->checkDuplicateMacros($hostmacros);
@@ -505,9 +572,21 @@ class CUserMacro extends CApiService {
 
 		$this->validateUpdate($hostmacros);
 
+		$db_macros = DB::select('hostmacro', [
+			'output' => ['type'],
+			'filter' => ['hostmacroid' => array_column($hostmacros, 'hostmacroid')],
+			'preservekeys' => true
+		]);
 		$data = [];
 
 		foreach ($hostmacros as $macro) {
+			$db_macro = $db_macros[$macro['hostmacroid']];
+
+			if (array_key_exists('type', $macro) && $macro['type'] != $db_macro['type']
+					&& $db_macro['type'] == ZBX_MACRO_TYPE_SECRET) {
+				$macro += ['value' => ''];
+			}
+
 			$hostmacroid = $macro['hostmacroid'];
 			unset($macro['hostmacroid']);
 
@@ -519,7 +598,7 @@ class CUserMacro extends CApiService {
 
 		DB::update('hostmacro', $data);
 
-		return ['hostmacroids' => zbx_objectValues($hostmacros, 'hostmacroid')];
+		return ['hostmacroids' => array_column($hostmacros, 'hostmacroid')];
 	}
 
 	/**
@@ -651,6 +730,18 @@ class CUserMacro extends CApiService {
 		}
 	}
 
+	/**
+	 * Validate the "type" field.
+	 *
+	 * @param array $macro
+	 */
+	protected function checkMacroType(array $macro) {
+		if (array_key_exists('type', $macro) && $macro['type'] != ZBX_MACRO_TYPE_TEXT
+				&& $macro['type'] != ZBX_MACRO_TYPE_SECRET) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid type for macro "%1$s".', $macro['macro']));
+		}
+	}
+
 	/**
 	 * Validates the "hostid" field.
 	 *
@@ -676,8 +767,46 @@ class CUserMacro extends CApiService {
 	 * @throws APIException if the user doesn't have write permissions for the given hosts.
 	 */
 	protected function checkHostPermissions(array $hostids) {
-		if (!API::Host()->isWritable($hostids)) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
+		if ($hostids) {
+			$hostids = array_unique($hostids);
+
+			$count = API::Host()->get([
+				'countOutput' => true,
+				'hostids' => $hostids,
+				'filter' => [
+					'flags' => ZBX_FLAG_DISCOVERY_NORMAL
+				],
+				'editable' => true
+			]);
+
+			if ($count == count($hostids)) {
+				return;
+			}
+
+			$count += API::Template()->get([
+				'countOutput' => true,
+				'templateids' => $hostids,
+				'filter' => [
+					'flags' => ZBX_FLAG_DISCOVERY_NORMAL
+				],
+				'editable' => true
+			]);
+
+			if ($count == count($hostids)) {
+				return;
+			}
+
+			$count += API::HostPrototype()->get([
+				'countOutput' => true,
+				'hostids' => $hostids,
+				'editable' => true
+			]);
+
+			if ($count != count($hostids)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
 		}
 	}
 
@@ -704,17 +833,30 @@ class CUserMacro extends CApiService {
 
 			$macro_name = $user_macro_parser->getMacro();
 			$context = $user_macro_parser->getContext();
+			$regex = $user_macro_parser->getRegex();
 
 			/*
 			 * Macros with same name can have different contexts. A macro with no context is not the same
 			 * as a macro with an empty context.
 			 */
-			if (array_key_exists($hostid, $existing_macros) && array_key_exists($macro_name, $existing_macros[$hostid])
-					&& in_array($context, $existing_macros[$hostid][$macro_name], true)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Macro "%1$s" is not unique.', $macro['macro']));
+			if (array_key_exists($hostid, $existing_macros)
+					&& array_key_exists($macro_name, $existing_macros[$hostid])) {
+				$has_context = in_array($context, array_column($existing_macros[$hostid][$macro_name], 'context'),
+					true
+				);
+				$context_exists = ($context !== null && $has_context);
+
+				$has_regex = in_array($regex, array_column($existing_macros[$hostid][$macro_name], 'regex'), true);
+				$regex_exists = ($regex !== null && $has_regex);
+
+				$is_macro_without_context = ($context === null && $regex === null);
+
+				if (($is_macro_without_context && $has_context && $has_regex) || ($context_exists || $regex_exists)) {
+					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Macro "%1$s" is not unique.', $macro['macro']));
+				}
 			}
 
-			$existing_macros[$hostid][$macro_name][] = $context;
+			$existing_macros[$hostid][$macro_name][] = ['context' => $context, 'regex' => $regex];
 		}
 	}
 
@@ -726,7 +868,7 @@ class CUserMacro extends CApiService {
 	 * @param array $hostmacros
 	 * @param int $hostmacros[]['hostmacroid']
 	 * @param int $hostmacros[]['hostid']
-	 * @paramt string $hostmacros['macro']
+	 * @param string $hostmacros['macro']
 	 *
 	 * @throws APIException if any of the given macros already exist.
 	 */
@@ -744,8 +886,9 @@ class CUserMacro extends CApiService {
 
 			$macro_name = $user_macro_parser->getMacro();
 			$context = $user_macro_parser->getContext();
+			$regex = $user_macro_parser->getRegex();
 
-			if ($context === null) {
+			if ($context === null && $regex === null) {
 				$macro_names['{$'.$macro_name] = true;
 			}
 			else {
@@ -757,7 +900,7 @@ class CUserMacro extends CApiService {
 		// When updating with empty array, don't select any data from database.
 		$db_hostmacros = API::getApiService()->select($this->tableName(), [
 			'output' => ['hostmacroid', 'hostid', 'macro'],
-			'filter' => ['hostid' => array_unique(zbx_objectValues($hostmacros, 'hostid'))],
+			'filter' => ['hostid' => array_unique(array_column($hostmacros, 'hostid'))],
 			'search' => ['macro' => array_keys($macro_names)],
 			'searchByAny' => true
 		]);
@@ -770,8 +913,10 @@ class CUserMacro extends CApiService {
 
 			$macro_name = $user_macro_parser->getMacro();
 			$context = $user_macro_parser->getContext();
+			$regex = $user_macro_parser->getRegex();
 
-			$existing_macros[$db_hostmacro['hostid']][$macro_name][$db_hostmacro['hostmacroid']] = $context;
+			$existing_macros[$db_hostmacro['hostid']][$macro_name][$db_hostmacro['hostmacroid']] =
+				['context' => $context, 'regex' => $regex];
 		}
 
 		// Compare each macro name and context to existing one.
@@ -782,21 +927,32 @@ class CUserMacro extends CApiService {
 
 			$macro_name = $user_macro_parser->getMacro();
 			$context = $user_macro_parser->getContext();
+			$regex = $user_macro_parser->getRegex();
 
-			if (array_key_exists($hostid, $existing_macros) && array_key_exists($macro_name, $existing_macros[$hostid])
-					&& in_array($context, $existing_macros[$hostid][$macro_name], true)) {
-				foreach ($existing_macros[$hostid][$macro_name] as $hostmacroid => $existing_macro_context) {
-					if ((!array_key_exists('hostmacroid', $hostmacro)
-							|| bccomp($hostmacro['hostmacroid'], $hostmacroid) != 0)
-								&& $context === $existing_macro_context) {
-						$hosts = API::getApiService()->select('hosts', [
-							'output' => ['name'],
-							'hostids' => $hostmacro['hostid']
-						]);
+			if (array_key_exists($hostid, $existing_macros)
+					&& array_key_exists($macro_name, $existing_macros[$hostid])) {
+				$has_context = ($context !== null && in_array($context,
+					array_column($existing_macros[$hostid][$macro_name], 'context'), true
+				));
+				$has_regex = ($regex !== null && in_array($regex,
+					array_column($existing_macros[$hostid][$macro_name], 'regex'), true
+				));
+				$is_macro_without_context = ($context === null && $regex === null);
 
-						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_s('Macro "%1$s" already exists on "%2$s".', $hostmacro['macro'], $hosts[0]['name'])
-						);
+				if ($is_macro_without_context || $has_context || $has_regex) {
+					foreach ($existing_macros[$hostid][$macro_name] as $hostmacroid => $macro_details) {
+						if ((!array_key_exists('hostmacroid', $hostmacro)
+									|| bccomp($hostmacro['hostmacroid'], $hostmacroid) != 0)
+								&& $context === $macro_details['context'] && $regex === $macro_details['regex']) {
+							$hosts = API::getApiService()->select('hosts', [
+								'output' => ['name'],
+								'hostids' => $hostmacro['hostid']
+							]);
+
+							self::exception(ZBX_API_ERROR_PARAMETERS,
+								_s('Macro "%1$s" already exists on "%2$s".', $hostmacro['macro'], $hosts[0]['name'])
+							);
+						}
 					}
 				}
 			}
@@ -824,123 +980,12 @@ class CUserMacro extends CApiService {
 		}
 	}
 
-	/**
-	 * Checks if any of the given host global macros already exist. If the macros are updated and
-	 * the "globalmacroid" field is set, the method will only fail, if a macro with a different globalmacroid exists.
-	 * Assumes the "macro", "hostmacroid" fields are valid.
-	 *
-	 * @param array $globalmacros
-	 * @param int $globalmacros[]['globalmacroid']
-	 * @param string $globalmacros[]['macro']
-	 * @param string $globalmacros[]['value']
-	 *
-	 * @throws APIException if any of the given macros already exist.
-	 */
-	protected function checkIfGlobalMacrosDontRepeat(array $globalmacros) {
-		if (!$globalmacros) {
-			return;
-		}
-
-		$macro_names = [];
-		$user_macro_parser = new CUserMacroParser();
-
-		// Parse each macro, get unique names and, if context exists, narrow down the search.
-		foreach ($globalmacros as $globalmacro) {
-			$user_macro_parser->parse($globalmacro['macro']);
-
-			$macro_name = $user_macro_parser->getMacro();
-			$context = $user_macro_parser->getContext();
-
-			if ($context === null) {
-				$macro_names['{$'.$macro_name] = true;
-			}
-			else {
-				// Narrow down the search for macros with contexts.
-				$macro_names['{$'.$macro_name.':'] = true;
-			}
-		}
-
-		// When updating with empty array, don't select any data from database.
-		$db_macros = API::getApiService()->select('globalmacro', [
-			'output' => ['globalmacroid', 'macro'],
-			'search' => ['macro' => array_keys($macro_names)],
-			'searchByAny' => true
-		]);
-
-		$existing_macros = [];
-
-		// Collect existing unique macro names and their contexts.
-		foreach ($db_macros as $db_macro) {
-			$user_macro_parser->parse($db_macro['macro']);
-
-			$macro_name = $user_macro_parser->getMacro();
-			$context = $user_macro_parser->getContext();
-
-			$existing_macros[$macro_name][$db_macro['globalmacroid']] = $context;
-		}
-
-		// Compare each macro name and context to existing one.
-		foreach ($globalmacros as $globalmacro) {
-			$user_macro_parser->parse($globalmacro['macro']);
-
-			$macro_name = $user_macro_parser->getMacro();
-			$context = $user_macro_parser->getContext();
-
-			if (array_key_exists($macro_name, $existing_macros)
-					&& in_array($context, $existing_macros[$macro_name], true)) {
-				foreach ($existing_macros[$macro_name] as $globalmacroid => $existing_macro_context) {
-					if ((!array_key_exists('globalmacroid', $globalmacro)
-							|| bccomp($globalmacro['globalmacroid'], $globalmacroid) != 0)
-								&& $context === $existing_macro_context) {
-						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_s('Macro "%1$s" already exists.', $globalmacro['macro'])
-						);
-					}
-				}
-			}
-		}
-	}
-
-	/**
-	 * Checks if the user has the permissions to edit global macros.
-	 *
-	 * @param string $error a message that will be used as the error text
-	 *
-	 * @throws APIException if the user doesn't have the required permissions.
-	 */
-	protected function checkGlobalMacrosPermissions($error) {
-		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, $error);
-		}
-	}
-
-	/**
-	 * Checks if all of the global macros with globalmacroids given in $globalmacroids are present in $globalmacros.
-	 * Assumes the "globalmacroids" field is valid.
-	 *
-	 * @param array $globalmacroids
-	 *
-	 * @throws APIException if any of the global macros is not present in $globalmacros.
-	 */
-	protected function checkIfGlobalMacrosExist(array $globalmacroids) {
-		if ($globalmacroids) {
-			$db_globamacros = API::getApiService()->select('globalmacro', [
-				'output' => ['globalmacroid'],
-				'globalmacroids' => $globalmacroids,
-				'preservekeys' => true
-			]);
-
-			foreach ($globalmacroids as $globalmacroid) {
-				if (!array_key_exists($globalmacroid, $db_globamacros)) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Macro with globalmacroid "%1$s" does not exist.', $globalmacroid)
-					);
-				}
-			}
-		}
-	}
-
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
+		// Added type to query because it required to check macro is secret or not.
+		if (!$this->outputIsRequested('type', $options['output'])) {
+			$options['output'][] = 'type';
+		}
+
 		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
 
 		if ($options['output'] != API_OUTPUT_COUNT && $options['globalmacro'] === null) {
@@ -952,6 +997,47 @@ class CUserMacro extends CApiService {
 		return $sqlParts;
 	}
 
+	/**
+	 * @inheritdoc
+	 */
+	protected function applyQueryFilterOptions($table, $alias, array $options, $sql_parts) {
+		if (is_array($options['search'])) {
+			// Do not allow to search by value for macro of type ZBX_MACRO_TYPE_SECRET.
+			if (array_key_exists('value', $options['search'])) {
+				$sql_parts['where']['search'] = $alias.'.type!='.ZBX_MACRO_TYPE_SECRET;
+				zbx_db_search($table.' '.$alias, [
+						'searchByAny' => false,
+						'search' => ['value' => $options['search']['value']]
+					] + $options, $sql_parts
+				);
+				unset($options['search']['value']);
+			}
+
+			if ($options['search']) {
+				zbx_db_search($table.' '.$alias, $options, $sql_parts);
+			}
+		}
+
+		if (is_array($options['filter'])) {
+			// Do not allow to filter by value for macro of type ZBX_MACRO_TYPE_SECRET.
+			if (array_key_exists('value', $options['filter'])) {
+				$sql_parts['where']['filter'] = $alias.'.type!='.ZBX_MACRO_TYPE_SECRET;
+				$this->dbFilter($table.' '.$alias, [
+						'searchByAny' => false,
+						'filter' => ['value' => $options['filter']['value']]
+					] + $options, $sql_parts
+				);
+				unset($options['filter']['value']);
+			}
+
+			if ($options['filter']) {
+				$this->dbFilter($table.' '.$alias, $options, $sql_parts);
+			}
+		}
+
+		return $sql_parts;
+	}
+
 	protected function addRelatedObjects(array $options, array $result) {
 		$result = parent::addRelatedObjects($options, $result);
 
@@ -1007,4 +1093,15 @@ class CUserMacro extends CApiService {
 
 		return $result;
 	}
+
+	protected function unsetExtraFields(array $objects, array $fields, $output) {
+		foreach ($objects as &$object) {
+			if ($object['type'] == ZBX_MACRO_TYPE_SECRET) {
+				unset($object['value']);
+			}
+		}
+		unset($object);
+
+		return parent::unsetExtraFields($objects, $fields, $output);
+	}
 }
