@@ -38,6 +38,7 @@ class DB {
 	const FIELD_TYPE_UINT = 'uint';
 	const FIELD_TYPE_BLOB = 'blob';
 	const FIELD_TYPE_TEXT = 'text';
+	const FIELD_TYPE_NCLOB = 'nclob';
 
 	private static $schema = null;
 
@@ -65,9 +66,6 @@ class DB {
 				case ZBX_DB_ORACLE:
 					self::$dbBackend = new OracleDbBackend();
 					break;
-				case ZBX_DB_DB2:
-					self::$dbBackend = new Db2DbBackend();
-					break;
 			}
 		}
 
@@ -269,7 +267,11 @@ class DB {
 		$schema = self::getSchema($table_name);
 
 		if ($schema['fields'][$field_name]['type'] == self::FIELD_TYPE_TEXT) {
-			return ($DB['TYPE'] == ZBX_DB_DB2 || $DB['TYPE'] == ZBX_DB_ORACLE) ? 2048 : 65536;
+			return ($DB['TYPE'] == ZBX_DB_ORACLE) ? 2048 : 65535;
+		}
+
+		if ($schema['fields'][$field_name]['type'] == self::FIELD_TYPE_NCLOB) {
+			return 65535;
 		}
 
 		return $schema['fields'][$field_name]['length'];
@@ -280,7 +282,8 @@ class DB {
 
 		if ($DB['TYPE'] == ZBX_DB_MYSQL) {
 			foreach ($tableSchema['fields'] as $name => $field) {
-				if ($field['type'] == self::FIELD_TYPE_TEXT && !$field['null']) {
+				if (($field['type'] == self::FIELD_TYPE_TEXT || $field['type'] == self::FIELD_TYPE_NCLOB)
+						&& !$field['null']) {
 					foreach ($values as &$value) {
 						if (!isset($value[$name])) {
 							$value[$name] = '';
@@ -381,7 +384,7 @@ class DB {
 						$values[$field] = zbx_dbstr($values[$field]);
 						break;
 					case self::FIELD_TYPE_TEXT:
-						if ($DB['TYPE'] == ZBX_DB_DB2 || $DB['TYPE'] == ZBX_DB_ORACLE) {
+						if ($DB['TYPE'] == ZBX_DB_ORACLE) {
 							$length = mb_strlen($values[$field]);
 
 							if ($length > 2048) {
@@ -391,11 +394,41 @@ class DB {
 						}
 						$values[$field] = zbx_dbstr($values[$field]);
 						break;
+					case self::FIELD_TYPE_NCLOB:
+						// Using strlen because 4000 bytes is largest possible string literal in oracle query.
+						if ($DB['TYPE'] == ZBX_DB_ORACLE && strlen($values[$field]) > ORACLE_MAX_STRING_SIZE) {
+							$chunks = zbx_dbstr(self::chunkMultibyteStr($values[$field], ORACLE_MAX_STRING_SIZE));
+							$values[$field] = 'TO_NCLOB('.implode(') || TO_NCLOB(', $chunks).')';
+						}
+						else {
+							$values[$field] = zbx_dbstr($values[$field]);
+						}
+						break;
 				}
 			}
 		}
 	}
 
+	/**
+	 * @param string $str
+	 * @param int $chunk_size
+	 *
+	 * @return array
+	 */
+	public static function chunkMultibyteStr(string $str, int $chunk_size): array {
+		$chunks = [];
+		$offset = 0;
+		$size = strlen($str);
+
+		while ($offset < $size) {
+			$chunk = mb_strcut($str, $offset, $chunk_size);
+			$chunks[] = $chunk;
+			$offset = strlen($chunk) + $offset;
+		}
+
+		return $chunks;
+	}
+
 	/**
 	 * Returns the records that match the given criteria.
 	 *
@@ -1075,7 +1108,7 @@ class DB {
 
 			$field_schema = $table_schema['fields'][$field_name];
 
-			if ($field_schema['type'] == self::FIELD_TYPE_TEXT) {
+			if ($field_schema['type'] == self::FIELD_TYPE_TEXT || $field_schema['type'] == self::FIELD_TYPE_NCLOB) {
 				self::exception(self::SCHEMA_ERROR,
 					vsprintf('%s: field "%s.%s" has an unsupported type.', [__FUNCTION__, $table_name, $field_name])
 				);
