@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,8 +28,8 @@ class CForm extends CTag {
 		$this->setEnctype($enctype);
 		$this->setAttribute('accept-charset', 'utf-8');
 
-		if (isset($_COOKIE['zbx_sessionid'])) {
-			$this->addVar('sid', substr($_COOKIE['zbx_sessionid'], 16, 16));
+		if (array_key_exists(ZBX_SESSION_NAME, $_COOKIE)) {
+			$this->addVar('sid', substr($_COOKIE[ZBX_SESSION_NAME], 16, 16));
 		}
 		$this->addVar('form_refresh', getRequest('form_refresh', 0) + 1);
 	}
@@ -65,4 +65,17 @@ class CForm extends CTag {
 		}
 		return $this;
 	}
+
+	/**
+	 * Prevent browser from auto fill inputs with type password.
+	 *
+	 * @return CForm
+	 */
+	public function disablePasswordAutofill() {
+		$this->addItem((new CDiv([
+			(new CInput('password', null, null))->setAttribute('tabindex', '-1')->removeId()
+		]))->addStyle('display: none;'));
+
+		return $this;
+	}
 }
