@@ -25,6 +25,18 @@ class CMultiSelect extends CTag {
 	 */
 	const ZBX_STYLE_CLASS = 'multiselect-control';
 
+	/**
+	 * Search method used for autocomplete requests.
+	 */
+	const SEARCH_METHOD = 'multiselect.get';
+
+	/**
+	 * Supported preselect types.
+	 *
+	 * @param array
+	 */
+	protected $preselect_fields = ['hosts', 'hostgroups'];
+
 	/**
 	 * @param array $options['objectOptions']  An array of parameters to be added to the request URL.
 	 * @param bool  $options['multiple']       Allows multiple selections.
@@ -54,8 +66,8 @@ class CMultiSelect extends CTag {
 		// Autocomplete url.
 		$url = (new CUrl('jsrpc.php'))
 			->setArgument('type', PAGE_TYPE_TEXT_RETURN_JSON)
-			->setArgument('method', 'multiselect.get')
-			->setArgument('objectName', $options['objectName']);
+			->setArgument('method', static::SEARCH_METHOD)
+			->setArgument('object_name', $options['object_name']);
 
 		if (array_key_exists('objectOptions', $options)) {
 			foreach ($options['objectOptions'] as $option_name => $option_value) {
@@ -79,13 +91,23 @@ class CMultiSelect extends CTag {
 			$params['data'] = zbx_cleanHashes($options['data']);
 		}
 
-		foreach (['defaultValue', 'disabled', 'selectedLimit', 'addNew', 'styles'] as $option) {
+		foreach (['defaultValue', 'disabled', 'selectedLimit', 'addNew', 'styles', 'placeholder'] as $option) {
 			if (array_key_exists($option, $options)) {
 				$params[$option] = $options[$option];
 			}
 		}
 
+		if (array_key_exists('autosuggest', $options)) {
+			if (array_key_exists('filter_preselect_fields', $options['autosuggest'])) {
+				$params['autosuggest']['filter_preselect_fields'] = $options['autosuggest']['filter_preselect_fields'];
+			}
+		}
+
 		if (array_key_exists('popup', $options)) {
+			if (array_key_exists('filter_preselect_fields', $options['popup'])) {
+				$params['popup']['filter_preselect_fields'] = $options['popup']['filter_preselect_fields'];
+			}
+
 			if (array_key_exists('parameters', $options['popup'])) {
 				$params['popup']['parameters'] = $options['popup']['parameters'];
 
@@ -116,7 +138,7 @@ class CMultiSelect extends CTag {
 	}
 
 	public function getPostJS() {
-		return 'jQuery("#'.$this->getAttribute('id').'").multiSelect('.CJs::encodeJson($this->params).');';
+		return 'jQuery("#'.$this->getAttribute('id').'").multiSelect('.json_encode($this->params).');';
 	}
 
 	/**
@@ -126,9 +148,9 @@ class CMultiSelect extends CTag {
 	 *
 	 * @return array
 	 */
-	private function mapOptions(array $options) {
+	protected function mapOptions(array $options) {
 		$valid_fields = ['name', 'object_name', 'multiple', 'disabled', 'default_value', 'data', 'add_new',
-			'add_post_js', 'styles', 'popup'
+			'add_post_js', 'styles', 'popup', 'placeholder', 'autosuggest'
 		];
 
 		foreach ($options as $field => $value) {
@@ -140,13 +162,14 @@ class CMultiSelect extends CTag {
 		$mapped_options = [];
 		$mappings = [
 			'name' => 'name',
-			'object_name' => 'objectName',
+			'object_name' => 'object_name',
 			'disabled' => 'disabled',
 			'default_value' => 'defaultValue',
 			'data' => 'data',
 			'add_new' => 'addNew',
 			'add_post_js' => 'add_post_js',
-			'styles' => 'styles'
+			'styles' => 'styles',
+			'placeholder' => 'placeholder'
 		];
 
 		foreach ($mappings as $new_field => $old_field) {
@@ -163,8 +186,34 @@ class CMultiSelect extends CTag {
 		$autocomplete_parameters = [];
 		$popup_parameters = [];
 
+		if (array_key_exists('autosuggest', $options)) {
+			$valid_fields = ['filter_preselect_fields'];
+
+			foreach (array_keys($options['autosuggest']) as $field) {
+				if (!in_array($field, $valid_fields)) {
+					error('unsupported option: $options[\'autosuggest\'][\''.$field.'\']');
+				}
+			}
+
+			if (array_key_exists('filter_preselect_fields', $options['autosuggest'])) {
+				if (is_array($options['autosuggest']['filter_preselect_fields'])) {
+					foreach ($options['autosuggest']['filter_preselect_fields'] as $field => $value) {
+						if (in_array($field, $this->preselect_fields) && is_string($value) && $value !== '') {
+							$mapped_options['autosuggest']['filter_preselect_fields'][$field] = $value;
+						}
+						else {
+							error('invalid property: $options[\'autosuggest\'][\'filter_preselect_fields\'][\''.$field.'\']');
+						}
+					}
+				}
+				else {
+					error('invalid property: $options[\'autosuggest\'][\'filter_preselect_fields\']');
+				}
+			}
+		}
+
 		if (array_key_exists('popup', $options)) {
-			$valid_fields = ['parameters'];
+			$valid_fields = ['parameters', 'filter_preselect_fields'];
 
 			foreach ($options['popup'] as $field => $value) {
 				if (!in_array($field, $valid_fields)) {
@@ -172,13 +221,32 @@ class CMultiSelect extends CTag {
 				}
 			}
 
+			if (array_key_exists('filter_preselect_fields', $options['popup'])) {
+				if (is_array($options['popup']['filter_preselect_fields'])) {
+					foreach ($options['popup']['filter_preselect_fields'] as $field => $value) {
+						if (in_array($field, $this->preselect_fields) && is_string($value) && $value !== '') {
+							$mapped_options['popup']['filter_preselect_fields'][$field] = $value;
+						}
+						else {
+							error('invalid property: $options[\'popup\'][\'filter_preselect_fields\'][\''.$field.'\']');
+						}
+					}
+				}
+				else {
+					error('invalid property: $options[\'popup\'][\'filter_preselect_fields\']');
+				}
+			}
+
 			if (array_key_exists('parameters', $options['popup'])) {
 				$parameters = $options['popup']['parameters'];
 
 				$valid_fields = ['srctbl', 'srcfld1', 'srcfld2', 'dstfrm', 'dstfld1', 'real_hosts', 'monitored_hosts',
-					'with_monitored_triggers', 'noempty', 'editable', 'templated_hosts', 'hostid', 'webitems',
-					'normal_only', 'numeric', 'with_simple_graph_items', 'with_triggers', 'value_types', 'excludeids',
-					'disableids', 'enrich_parent_groups'];
+					'with_monitored_triggers', 'noempty', 'editable', 'templated_hosts', 'hostid', 'parent_discoveryid',
+					'webitems', 'normal_only', 'numeric', 'with_graphs', 'with_graph_prototypes', 'with_items',
+					'with_simple_graph_items', 'with_simple_graph_item_prototypes', 'with_triggers', 'value_types',
+					'excludeids', 'disableids', 'enrich_parent_groups', 'orig_names', 'with_monitored_items',
+					'with_httptests', 'with_hosts_and_templates'
+				];
 
 				foreach ($parameters as $field => $value) {
 					if (!in_array($field, $valid_fields)) {
@@ -209,6 +277,15 @@ class CMultiSelect extends CTag {
 					$autocomplete_parameters['hostid'] = (string) $parameters['hostid'];
 				}
 
+				if (array_key_exists('groupid', $parameters) && $parameters['groupid'] > 0) {
+					$popup_parameters['groupid'] = (string) $parameters['groupid'];
+				}
+
+				if (array_key_exists('parent_discoveryid', $parameters) && $parameters['parent_discoveryid'] > 0) {
+					$popup_parameters['parent_discoveryid'] = $parameters['parent_discoveryid'];
+					$autocomplete_parameters['parent_discoveryid'] = $parameters['parent_discoveryid'];
+				}
+
 				if (array_key_exists('numeric', $parameters) && $parameters['numeric']) {
 					$popup_parameters['numeric'] = '1';
 					$autocomplete_parameters['filter']['value_type'] = [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64];
@@ -234,20 +311,26 @@ class CMultiSelect extends CTag {
 				}
 
 				if (array_key_exists('templated_hosts', $parameters) && $parameters['templated_hosts']) {
+					$popup_parameters['templated_hosts'] = '1';
 					$autocomplete_parameters['templated_hosts'] = true;
 				}
 
-				if (array_key_exists('with_simple_graph_items', $parameters) && $parameters['with_simple_graph_items']) {
-					$popup_parameters['with_simple_graph_items'] = '1';
+				if (array_key_exists('with_hosts_and_templates', $parameters) && $parameters['with_hosts_and_templates']) {
+					$popup_parameters['with_hosts_and_templates'] = '1';
+					$autocomplete_parameters['with_hosts_and_templates'] = true;
 				}
 
-				if (array_key_exists('with_triggers', $parameters) && $parameters['with_triggers']) {
-					$popup_parameters['with_triggers'] = '1';
+				foreach (['with_graphs', 'with_graph_prototypes', 'with_simple_graph_items',
+						'with_simple_graph_item_prototypes', 'with_triggers'] as $name) {
+					if (array_key_exists($name, $parameters) && $parameters[$name]) {
+						$popup_parameters[$name] = '1';
+						$autocomplete_parameters[$name] = true;
+					}
 				}
 
 				if (array_key_exists('webitems', $parameters) && $parameters['webitems']) {
 					$popup_parameters['with_webitems'] = '1';
-					$autocomplete_parameters['webitems'] = '1';
+					$autocomplete_parameters['webitems'] = true;
 				}
 
 				if (array_key_exists('editable', $parameters) && $parameters['editable']) {
@@ -262,7 +345,28 @@ class CMultiSelect extends CTag {
 
 				if (array_key_exists('with_monitored_triggers', $parameters) && $parameters['with_monitored_triggers']) {
 					$popup_parameters['with_monitored_triggers'] = '1';
-					$autocomplete_parameters['monitored'] = true;
+
+					if ($popup_parameters['srctbl'] === 'triggers') {
+						$autocomplete_parameters['monitored'] = true;
+					}
+					else {
+						$autocomplete_parameters['with_monitored_triggers'] = true;
+					}
+				}
+
+				if (array_key_exists('with_monitored_items', $parameters) && $parameters['with_monitored_items']) {
+					$popup_parameters['with_monitored_items'] = '1';
+					$autocomplete_parameters['with_monitored_items'] = true;
+				}
+
+				if (array_key_exists('with_httptests', $parameters) && $parameters['with_httptests']) {
+					$popup_parameters['with_httptests'] = '1';
+					$autocomplete_parameters['with_httptests'] = true;
+				}
+
+				if (array_key_exists('with_items', $parameters) && $parameters['with_items']) {
+					$popup_parameters['with_items'] = '1';
+					$autocomplete_parameters['with_items'] = true;
 				}
 
 				if (array_key_exists('excludeids', $parameters) && $parameters['excludeids']) {
@@ -275,7 +379,11 @@ class CMultiSelect extends CTag {
 
 				if (array_key_exists('enrich_parent_groups', $parameters) && $parameters['enrich_parent_groups']) {
 					$popup_parameters['enrich_parent_groups'] = '1';
-					$autocomplete_parameters['enrich_parent_groups'] = '1';
+					$autocomplete_parameters['enrich_parent_groups'] = true;
+				}
+
+				if (array_key_exists('orig_names', $parameters) && $parameters['orig_names']) {
+					$popup_parameters['orig_names'] = '1';
 				}
 			}
 		}
