@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -65,13 +65,13 @@ class CConfigurationImport {
 	 */
 	public function __construct(array $options = [], CImportReferencer $referencer,
 			CImportedObjectContainer $importedObjectContainer) {
-		$this->options = [
+		$default_options = [
 			'groups' => ['createMissing' => false],
 			'hosts' => ['updateExisting' => false, 'createMissing' => false],
 			'templates' => ['updateExisting' => false, 'createMissing' => false],
 			'templateScreens' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
 			'applications' => ['createMissing' => false, 'deleteMissing' => false],
-			'templateLinkage' => ['createMissing' => false],
+			'templateLinkage' => ['createMissing' => false, 'deleteMissing' => false],
 			'items' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
 			'discoveryRules' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
 			'triggers' => ['updateExisting' => false, 'createMissing' => false, 'deleteMissing' => false],
@@ -80,10 +80,40 @@ class CConfigurationImport {
 			'screens' => ['updateExisting' => false, 'createMissing' => false],
 			'maps' => ['updateExisting' => false, 'createMissing' => false],
 			'images' => ['updateExisting' => false, 'createMissing' => false],
+			'mediaTypes' => ['updateExisting' => false, 'createMissing' => false],
 			'valueMaps' => ['updateExisting' => false, 'createMissing' => false]
 		];
 
-		$this->options = array_merge($this->options, $options);
+		$options = array_merge($default_options, $options);
+
+		$object_options = (
+			$options['templateLinkage']['createMissing']
+			|| $options['templateLinkage']['deleteMissing']
+			|| $options['applications']['createMissing']
+			|| $options['applications']['deleteMissing']
+			|| $options['items']['updateExisting']
+			|| $options['items']['createMissing']
+			|| $options['items']['deleteMissing']
+			|| $options['discoveryRules']['updateExisting']
+			|| $options['discoveryRules']['createMissing']
+			|| $options['discoveryRules']['deleteMissing']
+			|| $options['triggers']['deleteMissing']
+			|| $options['graphs']['deleteMissing']
+			|| $options['httptests']['updateExisting']
+			|| $options['httptests']['createMissing']
+			|| $options['httptests']['deleteMissing']
+		);
+		$options['process_templates'] = (
+			!$options['templates']['updateExisting']
+			&& ($object_options
+				|| $options['templateScreens']['updateExisting']
+				|| $options['templateScreens']['createMissing']
+				|| $options['templateScreens']['deleteMissing']
+			)
+		);
+		$options['process_hosts'] = (!$options['hosts']['updateExisting'] && $object_options);
+
+		$this->options = $options;
 		$this->referencer = $referencer;
 		$this->importedObjectContainer = $importedObjectContainer;
 	}
@@ -111,13 +141,12 @@ class CConfigurationImport {
 		$this->deleteMissingTriggers();
 		$this->deleteMissingGraphs();
 		$this->deleteMissingItems();
-		$this->deleteMissingApplications();
 
 		// import objects
 		$this->processApplications();
 		$this->processValueMaps();
-		$this->processItems();
 		$this->processHttpTests();
+		$this->processItems();
 		$this->processTriggers();
 		$this->processDiscoveryRules();
 		$this->processGraphs();
@@ -125,6 +154,10 @@ class CConfigurationImport {
 		$this->processMaps();
 		$this->processTemplateScreens();
 		$this->processScreens();
+		$this->processMediaTypes();
+
+		// Missing applications should be deleted after new application changes are done to all inherited items.
+		$this->deleteMissingApplications();
 
 		return true;
 	}
@@ -291,6 +324,19 @@ class CConfigurationImport {
 						$templatesRefs[$template['name']] = $template['name'];
 					}
 				}
+
+				if ($discoveryRule['overrides']) {
+					foreach ($discoveryRule['overrides'] as $override) {
+						foreach ($override['operations'] as $operation) {
+							if ($operation['operationobject'] == OPERATION_OBJECT_HOST_PROTOTYPE
+									&& array_key_exists('optemplate', $operation)) {
+								foreach ($operation['optemplate'] as $template) {
+									$templatesRefs[$template['name']] = $template['name'];
+								}
+							}
+						}
+					}
+				}
 			}
 		}
 
@@ -420,10 +466,6 @@ class CConfigurationImport {
 						case SCREEN_RESOURCE_MAP:
 							$mapsRefs[$resource['name']] = $resource['name'];
 							break;
-
-						case SCREEN_RESOURCE_SCREEN:
-							$screensRefs[$resource['name']] = $resource['name'];
-							break;
 					}
 				}
 			}
@@ -539,8 +581,10 @@ class CConfigurationImport {
 	 * @throws Exception
 	 */
 	protected function processTemplates() {
-		if ($this->options['templates']['updateExisting'] || $this->options['templates']['createMissing']) {
+		if ($this->options['templates']['updateExisting'] || $this->options['templates']['createMissing']
+				|| $this->options['process_templates']) {
 			$templates = $this->getFormattedTemplates();
+
 			if ($templates) {
 				$templateImporter = new CTemplateImporter($this->options, $this->referencer,
 					$this->importedObjectContainer
@@ -560,8 +604,10 @@ class CConfigurationImport {
 	 * @throws Exception
 	 */
 	protected function processHosts() {
-		if ($this->options['hosts']['updateExisting'] || $this->options['hosts']['createMissing']) {
+		if ($this->options['hosts']['updateExisting'] || $this->options['hosts']['createMissing']
+				|| $this->options['process_hosts']) {
 			$hosts = $this->getFormattedHosts();
+
 			if ($hosts) {
 				$hostImporter = new CHostImporter($this->options, $this->referencer, $this->importedObjectContainer);
 				$hostImporter->import($hosts);
@@ -692,8 +738,10 @@ class CConfigurationImport {
 					$applicationsIds = [];
 
 					foreach ($item['applications'] as $application) {
-						if ($applicationId = $this->referencer->resolveApplication($hostId, $application['name'])) {
-							$applicationsIds[] = $applicationId;
+						$applicationid = $this->referencer->resolveApplication($hostId, $application['name']);
+
+						if ($applicationid) {
+							$applicationsIds[] = $applicationid;
 						}
 						else {
 							throw new Exception(_s('Item "%1$s" on "%2$s": application "%3$s" does not exist.',
@@ -705,7 +753,16 @@ class CConfigurationImport {
 				}
 
 				if (array_key_exists('interface_ref', $item) && $item['interface_ref']) {
-					$item['interfaceid'] = $this->referencer->interfacesCache[$hostId][$item['interface_ref']];
+					$interfaceid = $this->referencer->resolveInterface($hostId, $item['interface_ref']);
+
+					if ($interfaceid) {
+						$item['interfaceid'] = $interfaceid;
+					}
+					else {
+						throw new Exception(_s('Cannot find interface "%1$s" used for item "%2$s" on "%3$s".',
+							$item['interface_ref'], $item['name'], $host
+						));
+					}
 				}
 
 				if (isset($item['valuemap']) && $item['valuemap']) {
@@ -724,6 +781,12 @@ class CConfigurationImport {
 				}
 
 				if ($item['type'] == ITEM_TYPE_DEPENDENT) {
+					if (!array_key_exists('key', $item[$xml_itemkey])) {
+						throw new Exception(_s('Incorrect value for field "%1$s": %2$s.', 'master_itemid',
+							_('cannot be empty')
+						));
+					}
+
 					$master_itemid = $this->referencer->resolveItem($hostId, $item[$xml_itemkey]['key']);
 
 					if ($master_itemid !== false) {
@@ -735,6 +798,24 @@ class CConfigurationImport {
 					unset($item[$xml_itemkey]);
 				}
 
+				if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
+					$headers = [];
+
+					foreach ($item['headers'] as $header) {
+						$headers[$header['name']] = $header['value'];
+					}
+
+					$item['headers'] = $headers;
+
+					$query_fields = [];
+
+					foreach ($item['query_fields'] as $query_field) {
+						$query_fields[] = [$query_field['name'] => $query_field['value']];
+					}
+
+					$item['query_fields'] = $query_fields;
+				}
+
 				$itemsId = $this->referencer->resolveItem($hostId, $item['key_']);
 
 				if ($itemsId) {
@@ -794,6 +875,7 @@ class CConfigurationImport {
 				}
 			}
 			unset($entity);
+
 			$entityids = $entity_service->create($entities)['itemids'];
 			$entityid = reset($entityids);
 
@@ -867,6 +949,10 @@ class CConfigurationImport {
 			}
 		}
 
+		if ($this->options['discoveryRules']['updateExisting']) {
+			$this->deleteMissingPrototypes($allDiscoveryRules);
+		}
+
 		$itemsToCreate = [];
 		$itemsToUpdate = [];
 
@@ -877,7 +963,16 @@ class CConfigurationImport {
 				$item['hostid'] = $hostId;
 
 				if (array_key_exists('interface_ref', $item) && $item['interface_ref']) {
-					$item['interfaceid'] = $this->referencer->interfacesCache[$hostId][$item['interface_ref']];
+					$interfaceid = $this->referencer->resolveInterface($hostId, $item['interface_ref']);
+
+					if ($interfaceid) {
+						$item['interfaceid'] = $interfaceid;
+					}
+					else {
+						throw new Exception(_s('Cannot find interface "%1$s" used for discovery rule "%2$s" on "%3$s".',
+							$item['interface_ref'], $item['name'], $host
+						));
+					}
 				}
 
 				unset($item['item_prototypes']);
@@ -887,11 +982,78 @@ class CConfigurationImport {
 
 				$itemId = $this->referencer->resolveItem($hostId, $item['key_']);
 
+				if ($item['type'] == ITEM_TYPE_HTTPAGENT) {
+					$headers = [];
+
+					foreach ($item['headers'] as $header) {
+						$headers[$header['name']] = $header['value'];
+					}
+
+					$item['headers'] = $headers;
+
+					$query_fields = [];
+
+					foreach ($item['query_fields'] as $query_field) {
+						$query_fields[] = [$query_field['name'] => $query_field['value']];
+					}
+
+					$item['query_fields'] = $query_fields;
+				}
+
+				if ($item['type'] == ITEM_TYPE_DEPENDENT) {
+					if (!array_key_exists('key', $item['master_item'])) {
+						throw new Exception( _s('Incorrect value for field "%1$s": %2$s.', 'master_itemid',
+							_('cannot be empty')
+						));
+					}
+
+					$item['master_itemid'] = $this->referencer->resolveItem($hostId,  $item['master_item']['key']);
+				}
+
+				unset($item['master_item']);
+
+				if ($item['overrides']) {
+					foreach ($item['overrides'] as &$override) {
+						foreach ($override['operations'] as &$operation) {
+							if ($operation['operationobject'] == OPERATION_OBJECT_HOST_PROTOTYPE
+									&& array_key_exists('optemplate', $operation)) {
+								foreach ($operation['optemplate'] as &$template) {
+									$templateid = $this->referencer->resolveTemplate($template['name']);
+
+									if (!$templateid) {
+										throw new Exception(_s(
+											'Cannot find template "%1$s" for override "%2$s" of discovery rule "%3$s" on "%4$s".',
+											$template['name'],
+											$override['name'],
+											$item['name'],
+											$host
+										));
+									}
+
+									$template['templateid'] = $templateid;
+
+									unset($template['name']);
+								}
+							}
+						}
+						unset($operation);
+					}
+					unset($override);
+				}
+
 				if ($itemId) {
 					$item['itemid'] = $itemId;
 					$itemsToUpdate[] = $item;
 				}
 				else {
+					/*
+					 * The array key "lld_macro_paths" must exist at this point. It is processed by chain conversion.
+					 * Unlike discoveryrule.update method, discoveryrule.create does not allow "lld_macro_paths"
+					 * to be empty.
+					 */
+					if (!$item['lld_macro_paths']) {
+						unset($item['lld_macro_paths']);
+					}
 					$itemsToCreate[] = $item;
 				}
 			}
@@ -925,13 +1087,15 @@ class CConfigurationImport {
 		// refresh discovery rules because templated ones can be inherited to host and used for prototypes
 		$this->referencer->refreshItems();
 
-		$xml_itemprototype_key = 'master_item_prototype';
-		$order_tree = $this->getDiscoveryRulesItemsOrder($xml_itemprototype_key);
+		$xml_item_key = 'master_item';
+		$order_tree = $this->getDiscoveryRulesItemsOrder($xml_item_key);
+
 		// process prototypes
 		$prototypes_to_update = [];
 		$prototypes_to_create = [];
-		$hostPrototypesToUpdate = [];
+		$host_prototypes_to_update = [];
 		$hostPrototypesToCreate = [];
+		$ex_group_prototypes_where = [];
 
 		foreach ($allDiscoveryRules as $host => $discoveryRules) {
 			$hostId = $this->referencer->resolveHostOrTemplate($host);
@@ -948,6 +1112,7 @@ class CConfigurationImport {
 
 				// prototypes
 				$item_prototypes = $item['item_prototypes'] ? $order_tree[$host][$item_key] : [];
+
 				foreach ($item_prototypes as $index => $level) {
 					$prototype = $item['item_prototypes'][$index];
 					$prototype['hostid'] = $hostId;
@@ -964,7 +1129,20 @@ class CConfigurationImport {
 					}
 
 					if (array_key_exists('interface_ref', $prototype) && $prototype['interface_ref']) {
-						$prototype['interfaceid'] = $this->referencer->interfacesCache[$hostId][$prototype['interface_ref']];
+						$interfaceid = $this->referencer->resolveInterface($hostId, $prototype['interface_ref']);
+
+						if ($interfaceid) {
+							$prototype['interfaceid'] = $interfaceid;
+						}
+						else {
+							throw new Exception(_s(
+								'Cannot find interface "%1$s" used for item prototype "%2$s" of discovery rule "%3$s" on "%4$s".',
+								$prototype['interface_ref'],
+								$prototype['name'],
+								$item['name'],
+								$host
+							));
+						}
 					}
 
 					if ($prototype['valuemap']) {
@@ -984,17 +1162,41 @@ class CConfigurationImport {
 					}
 
 					if ($prototype['type'] == ITEM_TYPE_DEPENDENT) {
+						if (!array_key_exists('key', $prototype[$xml_item_key])) {
+							throw new Exception( _s('Incorrect value for field "%1$s": %2$s.', 'master_itemid',
+								_('cannot be empty')
+							));
+						}
+
 						$master_itemprototypeid = $this->referencer->resolveItem($hostId,
-							$prototype[$xml_itemprototype_key]['key']
+							$prototype[$xml_item_key]['key']
 						);
 
 						if ($master_itemprototypeid !== false) {
 							$prototype['master_itemid'] = $master_itemprototypeid;
-							unset($prototype[$xml_itemprototype_key]);
+							unset($prototype[$xml_item_key]);
 						}
 					}
 					else {
-						unset($prototype[$xml_itemprototype_key]);
+						unset($prototype[$xml_item_key]);
+					}
+
+					if ($prototype['type'] == ITEM_TYPE_HTTPAGENT) {
+						$headers = [];
+
+						foreach ($prototype['headers'] as $header) {
+							$headers[$header['name']] = $header['value'];
+						}
+
+						$prototype['headers'] = $headers;
+
+						$query_fields = [];
+
+						foreach ($prototype['query_fields'] as $query_field) {
+							$query_fields[] = [$query_field['name'] => $query_field['value']];
+						}
+
+						$prototype['query_fields'] = $query_fields;
 					}
 
 					$prototypeId = $this->referencer->resolveItem($hostId, $prototype['key_']);
@@ -1065,48 +1267,71 @@ class CConfigurationImport {
 					$hostPrototypeId = $this->referencer->resolveHostPrototype($hostId, $itemId, $hostPrototype['host']);
 
 					if ($hostPrototypeId) {
+						if ($hostPrototype['groupPrototypes']) {
+							$ex_group_prototypes_where[] = '('.dbConditionInt('hostid', [$hostPrototypeId]).
+								' AND '.dbConditionString('name',
+									zbx_objectValues($hostPrototype['groupPrototypes'], 'name')
+								).
+							')';
+						}
+
 						$hostPrototype['hostid'] = $hostPrototypeId;
-						$hostPrototypesToUpdate[] = $hostPrototype;
+						$host_prototypes_to_update[] = $hostPrototype;
 					}
 					else {
 						$hostPrototype['ruleid'] = $itemId;
 						$hostPrototypesToCreate[] = $hostPrototype;
 					}
 				}
+			}
+		}
 
-				if (array_key_exists('interface_ref', $item) && $item['interface_ref']) {
-					$item['interfaceid'] = $this->referencer->interfacesCache[$hostId][$item['interface_ref']];
-				}
-				unset($item['item_prototypes']);
-				unset($item['trigger_prototypes']);
-				unset($item['graph_prototypes']);
-				unset($item['host_prototypes']);
+		// Attach existing host group prototype ids.
+		if ($ex_group_prototypes_where) {
+			$db_group_prototypes = DBFetchArray(DBselect(
+				'SELECT group_prototypeid,name,hostid'.
+					' FROM group_prototype'.
+					' WHERE '.implode(' OR ', $ex_group_prototypes_where)
+			));
 
-				$itemsId = $this->referencer->resolveItem($hostId, $item['key_']);
+			if ($db_group_prototypes) {
+				$groups_hash = [];
+				foreach ($db_group_prototypes as $group) {
+					$groups_hash[$group['hostid']][$group['name']] = $group['group_prototypeid'];
+				}
 
-				if ($itemsId) {
-					$item['itemid'] = $itemsId;
-					$itemsToUpdate[] = $item;
-				}
-				else {
-					$itemsToCreate[] = $item;
+				foreach ($host_prototypes_to_update as &$host_prototype) {
+					if (!array_key_exists($host_prototype['hostid'], $groups_hash)) {
+						continue;
+					}
+
+					$hash = $groups_hash[$host_prototype['hostid']];
+					foreach ($host_prototype['groupPrototypes'] as &$group_prototype) {
+						if (array_key_exists($group_prototype['name'], $hash)) {
+							$group_prototype['group_prototypeid'] = $hash[$group_prototype['name']];
+						}
+					}
+					unset($group_prototype, $hash);
 				}
+				unset($host_prototype, $groups_hash);
 			}
 		}
 
 		if ($prototypes_to_create) {
-			$this->createEntitiesWithDependency($xml_itemprototype_key, $prototypes_to_create, API::ItemPrototype());
+			ksort($prototypes_to_create);
+			$this->createEntitiesWithDependency($xml_item_key, $prototypes_to_create, API::ItemPrototype());
 		}
 
 		if ($prototypes_to_update) {
-			$this->updateEntitiesWithDependency($xml_itemprototype_key, $prototypes_to_update, API::ItemPrototype());
+			ksort($prototypes_to_update);
+			$this->updateEntitiesWithDependency($xml_item_key, $prototypes_to_update, API::ItemPrototype());
 		}
 
 		if ($hostPrototypesToCreate) {
 			API::HostPrototype()->create($hostPrototypesToCreate);
 		}
-		if ($hostPrototypesToUpdate) {
-			API::HostPrototype()->update($hostPrototypesToUpdate);
+		if ($host_prototypes_to_update) {
+			API::HostPrototype()->update($host_prototypes_to_update);
 		}
 
 		// refresh prototypes because templated ones can be inherited to host and used in triggers prototypes or graph prototypes
@@ -1418,33 +1643,31 @@ class CConfigurationImport {
 				$graph['ymax_itemid'] = $itemId;
 			}
 
-			if (isset($graph['gitems']) && $graph['gitems']) {
-				foreach ($graph['gitems'] as &$gitem) {
-					$gitemHostId = $this->referencer->resolveHostOrTemplate($gitem['item']['host']);
-					$gitem['itemid'] = $gitemHostId
-						? $this->referencer->resolveItem($gitemHostId, $gitem['item']['key'])
-						: false;
+			foreach ($graph['gitems'] as &$gitem) {
+				$gitemHostId = $this->referencer->resolveHostOrTemplate($gitem['item']['host']);
+				$gitem['itemid'] = $gitemHostId
+					? $this->referencer->resolveItem($gitemHostId, $gitem['item']['key'])
+					: false;
 
-					if (!$gitem['itemid']) {
-						throw new Exception(_s(
-							'Cannot find item "%1$s" on "%2$s" used in graph "%3$s".',
-							$gitem['item']['key'],
-							$gitem['item']['host'],
-							$graph['name']
-						));
-					}
+				if (!$gitem['itemid']) {
+					throw new Exception(_s(
+						'Cannot find item "%1$s" on "%2$s" used in graph "%3$s".',
+						$gitem['item']['key'],
+						$gitem['item']['host'],
+						$graph['name']
+					));
 				}
-				unset($gitem);
+			}
+			unset($gitem);
 
-				$graphId = $this->referencer->resolveGraph($gitemHostId, $graph['name']);
+			$graphId = $this->referencer->resolveGraph($gitemHostId, $graph['name']);
 
-				if ($graphId) {
-					$graph['graphid'] = $graphId;
-					$graphsToUpdate[] = $graph;
-				}
-				else {
-					$graphsToCreate[] = $graph;
-				}
+			if ($graphId) {
+				$graph['graphid'] = $graphId;
+				$graphsToUpdate[] = $graph;
+			}
+			else {
+				$graphsToCreate[] = $graph;
 			}
 		}
 
@@ -1576,8 +1799,7 @@ class CConfigurationImport {
 	 * @return null
 	 */
 	protected function processImages() {
-		if (CWebUser::$data['type'] != USER_TYPE_SUPER_ADMIN
-				|| (!$this->options['images']['updateExisting'] && !$this->options['images']['createMissing'])) {
+		if (!$this->options['images']['updateExisting'] && !$this->options['images']['createMissing']) {
 			return;
 		}
 
@@ -1662,6 +1884,50 @@ class CConfigurationImport {
 		}
 	}
 
+	/**
+	 * Import media types.
+	 */
+	protected function processMediaTypes() {
+		if (!$this->options['mediaTypes']['updateExisting'] && !$this->options['mediaTypes']['createMissing']) {
+			return;
+		}
+
+		$all_media_types = $this->getFormattedMediaTypes();
+
+		if (!$all_media_types) {
+			return;
+		}
+
+		$all_media_types = zbx_toHash($all_media_types, 'name');
+
+		$db_media_types = API::MediaType()->get([
+			'output' => ['mediatypeid', 'name'],
+			'filter' => ['name' => array_keys($all_media_types)]
+		]);
+		$db_media_types = zbx_toHash($db_media_types, 'name');
+
+		$upd_media_types = [];
+		$new_media_types = [];
+
+		foreach ($all_media_types as $name => $media_type) {
+			if (array_key_exists($name, $db_media_types)) {
+				$media_type['mediatypeid'] = $db_media_types[$name]['mediatypeid'];
+				$upd_media_types[] = $media_type;
+			}
+			else {
+				$new_media_types[] = $media_type;
+			}
+		}
+
+		if ($this->options['mediaTypes']['createMissing'] && $new_media_types) {
+			API::MediaType()->create($new_media_types);
+		}
+
+		if ($this->options['mediaTypes']['updateExisting'] && $upd_media_types) {
+			API::MediaType()->update($upd_media_types);
+		}
+	}
+
 	/**
 	 * Deletes items from DB that are missing in XML.
 	 *
@@ -1913,7 +2179,7 @@ class CConfigurationImport {
 	}
 
 	/**
-	 * Deletes discovery rules and prototypes from DB that are missing in XML.
+	 * Deletes discovery rules from DB that are missing in XML.
 	 *
 	 * @return null
 	 */
@@ -1936,16 +2202,14 @@ class CConfigurationImport {
 
 		$allDiscoveryRules = $this->getFormattedDiscoveryRules();
 
-		if ($allDiscoveryRules) {
-			foreach ($allDiscoveryRules as $host => $discoveryRules) {
-				$hostId = $this->referencer->resolveHostOrTemplate($host);
+		foreach ($allDiscoveryRules as $host => $discoveryRules) {
+			$hostId = $this->referencer->resolveHostOrTemplate($host);
 
-				foreach ($discoveryRules as $discoveryRule) {
-					$discoveryRuleId = $this->referencer->resolveItem($hostId, $discoveryRule['key_']);
+			foreach ($discoveryRules as $discoveryRule) {
+				$discoveryRuleId = $this->referencer->resolveItem($hostId, $discoveryRule['key_']);
 
-					if ($discoveryRuleId) {
-						$discoveryRuleIdsXML[$discoveryRuleId] = $discoveryRuleId;
-					}
+				if ($discoveryRuleId) {
+					$discoveryRuleIdsXML[$discoveryRuleId] = $discoveryRuleId;
 				}
 			}
 		}
@@ -1966,56 +2230,66 @@ class CConfigurationImport {
 
 		// refresh discovery rules because templated ones can be inherited to host and used for prototypes
 		$this->referencer->refreshItems();
+	}
 
-		$hostPrototypeIdsXML = [];
-		$triggerPrototypeIdsXML = [];
-		$itemPrototypeIdsXML = [];
-		$graphPrototypeIdsXML = [];
+	/**
+	 * Deletes prototypes from DB that are missing in XML.
+	 *
+	 * @param array $all_discovery_rules
+	 */
+	protected function deleteMissingPrototypes(array $all_discovery_rules) {
+		$xml_discoveryids = [];
+		$xml_host_prototypeids = [];
+		$xml_trigger_prototypeids = [];
+		$xml_graph_prototypeids = [];
+		$xml_item_prototypeids = [];
 
-		foreach ($allDiscoveryRules as $host => $discoveryRules) {
-			$hostId = $this->referencer->resolveHostOrTemplate($host);
+		foreach ($all_discovery_rules as $host => $discovery_rules) {
+			$hostid = $this->referencer->resolveHostOrTemplate($host);
 
-			foreach ($discoveryRules as $discoveryRule) {
-				$discoveryRuleId = $this->referencer->resolveItem($hostId, $discoveryRule['key_']);
+			foreach ($discovery_rules as $discovery_rule) {
+				$discoveryid = $this->referencer->resolveItem($hostid, $discovery_rule['key_']);
+
+				if ($discoveryid) {
+					$xml_discoveryids[$discoveryid] = $discoveryid;
 
-				if ($discoveryRuleId) {
 					// gather host prototype IDs to delete
-					foreach ($discoveryRule['host_prototypes'] as $hostPrototype) {
-						$hostPrototypeId = $this->referencer->resolveHostPrototype($hostId, $discoveryRuleId,
-							$hostPrototype['host']
+					foreach ($discovery_rule['host_prototypes'] as $host_prototype) {
+						$host_prototypeid = $this->referencer->resolveHostPrototype($hostid, $discoveryid,
+							$host_prototype['host']
 						);
 
-						if ($hostPrototypeId) {
-							$hostPrototypeIdsXML[$hostPrototypeId] = $hostPrototypeId;
+						if ($host_prototypeid) {
+							$xml_host_prototypeids[$host_prototypeid] = $host_prototypeid;
 						}
 					}
 
 					// gather trigger prototype IDs to delete
-					foreach ($discoveryRule['trigger_prototypes'] as $triggerPrototype) {
-						$triggerPrototypeId = $this->referencer->resolveTrigger($triggerPrototype['description'],
-							$triggerPrototype['expression'], $triggerPrototype['recovery_expression']
+					foreach ($discovery_rule['trigger_prototypes'] as $trigger_prototype) {
+						$trigger_prototypeid = $this->referencer->resolveTrigger($trigger_prototype['description'],
+							$trigger_prototype['expression'], $trigger_prototype['recovery_expression']
 						);
 
-						if ($triggerPrototypeId) {
-							$triggerPrototypeIdsXML[$triggerPrototypeId] = $triggerPrototypeId;
+						if ($trigger_prototypeid) {
+							$xml_trigger_prototypeids[$trigger_prototypeid] = $trigger_prototypeid;
 						}
 					}
 
 					// gather graph prototype IDs to delete
-					foreach ($discoveryRule['graph_prototypes'] as $graphPrototype) {
-						$graphPrototypeId = $this->referencer->resolveGraph($hostId, $graphPrototype['name']);
+					foreach ($discovery_rule['graph_prototypes'] as $graph_prototype) {
+						$graph_prototypeid = $this->referencer->resolveGraph($hostid, $graph_prototype['name']);
 
-						if ($graphPrototypeId) {
-							$graphPrototypeIdsXML[$graphPrototypeId] = $graphPrototypeId;
+						if ($graph_prototypeid) {
+							$xml_graph_prototypeids[$graph_prototypeid] = $graph_prototypeid;
 						}
 					}
 
 					// gather item prototype IDs to delete
-					foreach ($discoveryRule['item_prototypes'] as $itemPrototype) {
-						$itemPrototypeId = $this->referencer->resolveItem($hostId, $itemPrototype['key_']);
+					foreach ($discovery_rule['item_prototypes'] as $item_prototype) {
+						$item_prototypeid = $this->referencer->resolveItem($hostid, $item_prototype['key_']);
 
-						if ($itemPrototypeId) {
-							$itemPrototypeIdsXML[$itemPrototypeId] = $itemPrototypeId;
+						if ($item_prototypeid) {
+							$xml_item_prototypeids[$item_prototypeid] = $item_prototypeid;
 						}
 					}
 				}
@@ -2023,65 +2297,67 @@ class CConfigurationImport {
 		}
 
 		// delete missing host prototypes
-		$dbHostPrototypeIds = API::HostPrototype()->get([
+		$db_host_prototypes = API::HostPrototype()->get([
 			'output' => ['hostid'],
-			'discoveryids' => $discoveryRuleIdsXML,
+			'discoveryids' => $xml_discoveryids,
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false
 		]);
 
-		$hostPrototypesToDelete = array_diff_key($dbHostPrototypeIds, $hostPrototypeIdsXML);
+		$del_host_prototypes = array_diff_key($db_host_prototypes, $xml_host_prototypeids);
 
-		if ($hostPrototypesToDelete) {
-			API::HostPrototype()->delete(array_keys($hostPrototypesToDelete));
+		if ($del_host_prototypes) {
+			API::HostPrototype()->delete(array_keys($del_host_prototypes));
 		}
 
 		// delete missing trigger prototypes
-		$dbTriggerPrototypeIds = API::TriggerPrototype()->get([
+		$db_trigger_prototypes = API::TriggerPrototype()->get([
 			'output' => ['triggerid'],
-			'hostids' => $processedHostIds,
+			'discoveryids' => $xml_discoveryids,
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false
 		]);
 
-		$triggerPrototypesToDelete = array_diff_key($dbTriggerPrototypeIds, $triggerPrototypeIdsXML);
+		$del_trigger_prototypes = array_diff_key($db_trigger_prototypes, $xml_trigger_prototypeids);
 
 		// unlike triggers that belong to multiple hosts, trigger prototypes do not, so we just delete them
-		if ($triggerPrototypesToDelete) {
-			API::TriggerPrototype()->delete(array_keys($triggerPrototypesToDelete));
+		if ($del_trigger_prototypes) {
+			API::TriggerPrototype()->delete(array_keys($del_trigger_prototypes));
 		}
 
 		// delete missing graph prototypes
-		$dbGraphPrototypeIds = API::GraphPrototype()->get([
+		$db_graph_prototypes = API::GraphPrototype()->get([
 			'output' => ['graphid'],
-			'hostids' => $processedHostIds,
+			'discoveryids' => $xml_discoveryids,
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false
 		]);
 
-		$graphPrototypesToDelete = array_diff_key($dbGraphPrototypeIds, $graphPrototypeIdsXML);
+		$del_graph_prototypes = array_diff_key($db_graph_prototypes, $xml_graph_prototypeids);
 
 		// unlike graphs that belong to multiple hosts, graph prototypes do not, so we just delete them
-		if ($graphPrototypesToDelete) {
-			API::GraphPrototype()->delete(array_keys($graphPrototypesToDelete));
+		if ($del_graph_prototypes) {
+			API::GraphPrototype()->delete(array_keys($del_graph_prototypes));
 		}
 
 		// delete missing item prototypes
-		$dbItemPrototypeIds = API::ItemPrototype()->get([
+		$db_item_prototypes = API::ItemPrototype()->get([
 			'output' => ['itemid'],
-			'hostids' => $processedHostIds,
+			'discoveryids' => $xml_discoveryids,
 			'preservekeys' => true,
 			'nopermissions' => true,
 			'inherited' => false
 		]);
 
-		$itemPrototypesToDelete = array_diff_key($dbItemPrototypeIds, $itemPrototypeIdsXML);
+		$del_item_prototypes = array_diff_key($db_item_prototypes, $xml_item_prototypeids);
 
-		if ($itemPrototypesToDelete) {
-			API::ItemPrototype()->delete(array_keys($itemPrototypesToDelete));
+		if ($del_item_prototypes) {
+			API::ItemPrototype()->delete(array_keys($del_item_prototypes));
+
+			$this->referencer->refreshGraphs();
 		}
 	}
 
@@ -2333,6 +2609,19 @@ class CConfigurationImport {
 		return $this->formattedData['templateScreens'];
 	}
 
+	/**
+	 * Get formatted media types.
+	 *
+	 * @return array
+	 */
+	protected function getFormattedMediaTypes() {
+		if (!isset($this->formattedData['mediaTypes'])) {
+			$this->formattedData['mediaTypes'] = $this->adapter->getMediaTypes();
+		}
+
+		return $this->formattedData['mediaTypes'];
+	}
+
 	/**
 	 * Get items keys order tree, to ensure that master item will be inserted or updated before any of it dependent
 	 * item. Returns associative array where key is item index and value is item dependency level.
@@ -2343,8 +2632,8 @@ class CConfigurationImport {
 	 */
 	protected function getItemsOrder($master_key_identifier) {
 		$entities = $this->getFormattedItems();
-		$data_provider = API::Item();
-		return $this->getEntitiesOrder($master_key_identifier, $entities, $data_provider);
+
+		return $this->getEntitiesOrder($master_key_identifier, $entities);
 	}
 
 	/**
@@ -2358,14 +2647,13 @@ class CConfigurationImport {
 	 */
 	protected function getDiscoveryRulesItemsOrder($master_key_identifier) {
 		$discovery_rules = $this->getFormattedDiscoveryRules();
-		$data_povider = API::ItemPrototype();
 		$entities_order = [];
 
 		foreach ($discovery_rules as $host_key => $items) {
 			foreach ($items as $item) {
 				if ($item['item_prototypes']) {
 					$item_prototypes = [$host_key => $item['item_prototypes']];
-					$item_prototypes = $this->getEntitiesOrder($master_key_identifier, $item_prototypes, $data_povider);
+					$item_prototypes = $this->getEntitiesOrder($master_key_identifier, $item_prototypes, true);
 					$entities_order[$host_key][$item['key_']] = $item_prototypes[$host_key];
 				}
 			}
@@ -2380,15 +2668,15 @@ class CConfigurationImport {
 	 * Returns associative array where key is entity index in source array grouped by host key and value is entity
 	 * dependency level.
 	 *
-	 * @param string               $master_key_identifier   String containing master key name to identify item master.
-	 * @param array                $host_entities           Associative array of host key and host items.
-	 * @param CItem|CItemPrototype $data_provider           Service to get master items not found in supplied input.
+	 * @param string $master_key_identifier  String containing master key name to identify item master.
+	 * @param array  $host_entities          Associative array of host key and host items.
+	 * @param bool   $get_prototypes         Option to get also master item prototypes not found in supplied input.
 	 *
 	 * @throws Exception if data is invalid.
 	 *
 	 * @return array
 	 */
-	protected function getEntitiesOrder($master_key_identifier, $host_entities, $data_provider) {
+	protected function getEntitiesOrder($master_key_identifier, array $host_entities, $get_prototypes = false) {
 		$find_keys = [];
 		$find_hosts = [];
 		$resolved_masters_cache = [];
@@ -2429,16 +2717,26 @@ class CConfigurationImport {
 
 		// There are entities to resolve from database, resolve and cache them recursively.
 		if ($find_keys) {
-			// For existing items, 'referencer' should be initialized before 'addItemRef' method will be used.
-			// Registering reference when property 'itemRefs' is empty, will not allow first call of
-			// 'resolveValueMap' method update references to existing items.
+			/*
+			 * For existing items, 'referencer' should be initialized before 'addItemRef' method will be used.
+			 * Registering reference when property 'itemRefs' is empty, will not allow first call of 'resolveValueMap'
+			 * method update references to existing items.
+			 */
 			$this->referencer->initItemsReferences();
-			$resolved_entities = $data_provider->get([
+
+			$options = [
 				'output' => ['key_', 'type', 'hostid', 'master_itemid'],
 				'hostids' => array_keys($find_hosts),
 				'filter' => ['key_' => array_keys($find_keys)],
 				'preservekeys' => true
-			]);
+			];
+
+			$resolved_entities = API::Item()->get($options + ['webitems' => true]);
+
+			if ($get_prototypes) {
+				$resolved_entities += API::ItemPrototype()->get($options);
+			}
+
 			$resolve_entity_keys = [];
 			$host_entity_idkey_hash = [];
 
@@ -2457,6 +2755,7 @@ class CConfigurationImport {
 
 					if ($entity['type'] == ITEM_TYPE_DEPENDENT) {
 						$master_itemid = $entity['master_itemid'];
+
 						if (array_key_exists($master_itemid, $host_entity_idkey_hash[$host_key])) {
 							$cache_entity[$master_key_identifier] = [
 								'key' => $host_entity_idkey_hash[$host_key][$master_itemid]
@@ -2471,15 +2770,21 @@ class CConfigurationImport {
 							];
 						}
 					}
+
 					$resolved_masters_cache[$host_key][$entity['key']] = $cache_entity;
 				}
 
 				if ($find_ids) {
-					$resolved_entities = $data_provider->get([
+					$options = [
 						'output' => ['key_', 'type', 'hostid', 'master_itemid'],
 						'itemids' => $find_ids,
 						'preservekeys' => true
-					]);
+					];
+					$resolved_entities = API::Item()->get($options + ['webitems' => true]);
+
+					if ($get_prototypes) {
+						$resolved_entities += API::ItemPrototype()->get($options);
+					}
 				}
 				else {
 					break;
@@ -2501,14 +2806,17 @@ class CConfigurationImport {
 						_s('value "%1$s" not found', $master_itemid)
 					));
 				}
+
 				$master_key = $host_entity_idkey_hash[$entity['host']][$master_itemid];
 				$resolved_masters_cache[$entity['host']][$entity['key']] += [
 					$master_key_identifier => ['key' => $master_key]
 				];
 			}
+
 			unset($resolve_entity_keys);
 			unset($host_entity_idkey_hash);
 		}
+
 		// Resolve every entity dependency level.
 		$tree = [];
 
@@ -2521,6 +2829,12 @@ class CConfigurationImport {
 				$traversal_path = [$entity['key_']];
 
 				while ($entity && $entity['type'] == ITEM_TYPE_DEPENDENT) {
+					if (!array_key_exists('key', $entity[$master_key_identifier])) {
+						throw new Exception(_s('Incorrect value for field "%1$s": %2$s.', 'master_itemid',
+							_('cannot be empty')
+						));
+					}
+
 					$master_key = $entity[$master_key_identifier]['key'];
 
 					if (array_key_exists($host_key, $resolved_masters_cache)
@@ -2545,17 +2859,19 @@ class CConfigurationImport {
 					}
 
 					if ($level > ZBX_DEPENDENT_ITEM_MAX_LEVELS) {
-						$traversal = '"'. implode('" => "', $traversal_path) . '"';
 						throw new Exception(_s('Incorrect value for field "%1$s": %2$s.', 'master_itemid',
 							_('maximum number of dependency levels reached')
 						));
 					}
 				}
+
 				$host_items_tree[$entity_index] = $level;
 			}
+
 			$tree[$host_key] = $host_items_tree;
 		}
-		// Order item indexes in descending order by nesting level
+
+		// Order item indexes in descending order by nesting level.
 		foreach ($tree as $host_key => &$item_indexes) {
 			asort($item_indexes);
 		}
