@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -22,14 +22,14 @@
 class CJsonImportReader extends CImportReader {
 
 	/**
-	 * convert string with data in JSON format to php array.
+	 * Convert string with data in JSON format to php array.
 	 *
 	 * @param string $string
 	 *
 	 * @return array
 	 */
 	public function read($string) {
-		$data = (new CJson)->decode($string, true);
+		$data = json_decode($string, true);
 
 		if ($data === null){
 			throw new Exception(_s('Cannot read JSON: %1$s.', json_last_error_msg()));
