@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,6 +21,21 @@
 
 class CLdap {
 
+	const ERR_PHP_EXTENSION = 1;
+	const ERR_SERVER_UNAVAILABLE = 2;
+	const ERR_BIND_FAILED = 3;
+	const ERR_BIND_ANON_FAILED = 4;
+	const ERR_USER_NOT_FOUND = 5;
+	const ERR_OPT_PROTOCOL_FAILED = 10;
+	const ERR_OPT_TLS_FAILED = 11;
+	const ERR_OPT_REFERRALS_FAILED = 12;
+	const ERR_OPT_DEREF_FAILED = 13;
+
+	/**
+	 * @var int
+	 */
+	public $error;
+
 	public function __construct($arg = []) {
 		$this->ds = false;
 		$this->info = [];
@@ -48,15 +63,23 @@ class CLdap {
 			$this->cnf = zbx_array_merge($this->cnf, $arg);
 		}
 
-		$ldap_status = (new CFrontendSetup())->checkPhpLdapModule();
+		$this->error = $this->moduleEnabled() ? 0 : static::ERR_PHP_EXTENSION;
+	}
 
-		if ($ldap_status['result'] != CFrontendSetup::CHECK_OK) {
-			error($ldap_status['error']);
-			return false;
-		}
+	/**
+	 * Check is the PHP extension enabled.
+	 *
+	 * @return bool
+	 */
+	public function moduleEnabled() {
+		return function_exists('ldap_connect') && function_exists('ldap_set_option') && function_exists('ldap_bind')
+			&& function_exists('ldap_search') && function_exists('ldap_get_entries')
+			&& function_exists('ldap_free_result') && function_exists('ldap_start_tls');
 	}
 
 	public function connect() {
+		$this->error = 0;
+
 		// connection already established
 		if ($this->ds) {
 			return true;
@@ -64,41 +87,43 @@ class CLdap {
 
 		$this->bound = 0;
 
-		if (!$this->ds = ldap_connect($this->cnf['host'], $this->cnf['port'])) {
-			error('LDAP: couldn\'t connect to LDAP server.');
+		if (!$this->ds = @ldap_connect($this->cnf['host'], $this->cnf['port'])) {
+			$this->error = static::ERR_SERVER_UNAVAILABLE;
 
 			return false;
 		}
 
-		// set protocol version and dependend options
+		// Set protocol version and dependent options.
 		if ($this->cnf['version']) {
-			if (!ldap_set_option($this->ds, LDAP_OPT_PROTOCOL_VERSION, $this->cnf['version'])) {
-				error('Setting LDAP Protocol version '.$this->cnf['version'].' failed.');
+			if (!@ldap_set_option($this->ds, LDAP_OPT_PROTOCOL_VERSION, $this->cnf['version'])) {
+				$this->error = static::ERR_OPT_PROTOCOL_FAILED;
 			}
 			else {
 				// use TLS (needs version 3)
-				if (isset($this->cnf['starttls']) && !ldap_start_tls($this->ds)) {
-					error('Starting TLS failed.');
+				if (isset($this->cnf['starttls']) && !@ldap_start_tls($this->ds)) {
+					$this->error = static::ERR_OPT_TLS_FAILED;
 				}
 
 				// needs version 3
 				if (!zbx_empty($this->cnf['referrals'])
-						&& !ldap_set_option($this->ds, LDAP_OPT_REFERRALS, $this->cnf['referrals'])) {
-					error('Setting LDAP referrals to off failed.');
+						&& !@ldap_set_option($this->ds, LDAP_OPT_REFERRALS, $this->cnf['referrals'])) {
+					$this->error = static::ERR_OPT_REFERRALS_FAILED;
 				}
 			}
 		}
 
 		// set deref mode
-		if (isset($this->cnf['deref']) && !ldap_set_option($this->ds, LDAP_OPT_DEREF, $this->cnf['deref'])) {
-			error('Setting LDAP Deref mode '.$this->cnf['deref'].' failed.');
+		if (isset($this->cnf['deref']) && !@ldap_set_option($this->ds, LDAP_OPT_DEREF, $this->cnf['deref'])) {
+			$this->error = static::ERR_OPT_DEREF_FAILED;
 		}
 
-		return true;
+		return !$this->error;
 	}
 
 	public function checkPass($user, $pass) {
 		if (!$pass) {
+			$this->error = static::ERR_USER_NOT_FOUND;
+
 			return false;
 		}
 
@@ -111,8 +136,8 @@ class CLdap {
 		// indirect user bind
 		if (!empty($this->cnf['bind_dn']) && !empty($this->cnf['bind_password'])) {
 			// use superuser credentials
-			if (!ldap_bind($this->ds, $this->cnf['bind_dn'], $this->cnf['bind_password'])) {
-				error('LDAP: cannot bind by given Bind DN.');
+			if (!@ldap_bind($this->ds, $this->cnf['bind_dn'], $this->cnf['bind_password'])) {
+				$this->error = static::ERR_BIND_FAILED;
 
 				return false;
 			}
@@ -129,8 +154,8 @@ class CLdap {
 		}
 		else {
 			// anonymous bind
-			if (!ldap_bind($this->ds)) {
-				error('LDAP: can not bind anonymously.');
+			if (!@ldap_bind($this->ds)) {
+				$this->error = static::ERR_BIND_ANON_FAILED;
 
 				return false;
 			}
@@ -139,7 +164,9 @@ class CLdap {
 		// try to bind to with the dn if we have one.
 		if ($dn) {
 			// user/password bind
-			if (!ldap_bind($this->ds, $dn, $pass)) {
+			if (!@ldap_bind($this->ds, $dn, $pass)) {
+				$this->error = static::ERR_USER_NOT_FOUND;
+
 				return false;
 			}
 
@@ -159,7 +186,9 @@ class CLdap {
 			}
 
 			// try to bind with the dn provided
-			if (!ldap_bind($this->ds, $dn, $pass)) {
+			if (!@ldap_bind($this->ds, $dn, $pass)) {
+				$this->error = static::ERR_USER_NOT_FOUND;
+
 				return false;
 			}
 
@@ -178,7 +207,9 @@ class CLdap {
 
 		// force superuser bind if wanted and not bound as superuser yet
 		if (!empty($this->cnf['bind_dn']) && !empty($this->cnf['bind_password']) && ($this->bound < 2)) {
-			if (!ldap_bind($this->ds, $this->cnf['bind_dn'], $this->cnf['bind_password'])) {
+			if (!@ldap_bind($this->ds, $this->cnf['bind_dn'], $this->cnf['bind_password'])) {
+				$this->error = static::ERR_BIND_FAILED;
+
 				return false;
 			}
 			$this->bound = 2;
@@ -197,12 +228,13 @@ class CLdap {
 		else {
 			$filter = '(ObjectClass=*)';
 		}
-		$sr = ldap_search($this->ds, $base, $filter);
-		$result = ldap_get_entries($this->ds, $sr);
+		$sr = @ldap_search($this->ds, $base, $filter);
+		$result = is_resource($sr) ? @ldap_get_entries($this->ds, $sr) : [];
 
 		// don't accept more or less than one response
-		if ($result['count'] != 1) {
-			error('LDAP: User not found.');
+		if (!$result || $result['count'] != 1) {
+			$this->error = $result ? static::ERR_USER_NOT_FOUND : static::ERR_BIND_FAILED;
+
 			return false;
 		}
 
@@ -222,31 +254,6 @@ class CLdap {
 		}
 		$user_result = zbx_array_merge($info,$user_result);
 
-		// get groups for given user if grouptree is given
-		if (isset($this->cnf['grouptree']) && isset($this->cnf['groupfilter'])) {
-			$base = $this->makeFilter($this->cnf['grouptree'], $user_result);
-			$filter = $this->makeFilter($this->cnf['groupfilter'], $user_result);
-			$sr = ldap_search($this->ds, $base, $filter, [$this->cnf['groupkey']]);
-
-			if (!$sr) {
-				error('LDAP: Reading group memberships failed.');
-				return false;
-			}
-
-			$result = ldap_get_entries($this->ds, $sr);
-
-			foreach ($result as $grp) {
-				if (!empty($grp[$this->cnf['groupkey']][0])) {
-					$info['grps'][] = $grp[$this->cnf['groupkey']][0];
-				}
-			}
-		}
-
-		// always add the default group to the list of groups
-		if (isset($conf['defaultgroup']) && !str_in_array($conf['defaultgroup'], $info['grps'])) {
-			$info['grps'][] = $conf['defaultgroup'];
-		}
-
 		return $info;
 	}
 
