@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -53,31 +53,30 @@ class CMacrosResolverGeneral {
 	 * @return array
 	 */
 	protected function resolveTriggerReferences($expression, $references) {
-		$matched_macros = $this->getMacroPositions($expression, ['usermacros' => true]);
+		$values = [];
+		$expression_data = new CTriggerExpression(['collapsed_expression' => true]);
 
-		// Replace user macros with string 'macro' to make values search easier.
-		foreach (array_reverse($matched_macros, true) as $pos => $macro) {
-			$expression = substr_replace($expression, 'macro', $pos, strlen($macro));
+		if (($result = $expression_data->parse($expression)) !== false) {
+			foreach ($result->getTokens() as $token) {
+				switch ($token['type']) {
+					case CTriggerExprParserResult::TOKEN_TYPE_NUMBER:
+					case CTriggerExprParserResult::TOKEN_TYPE_USER_MACRO:
+						$values[] = $token['value'];
+						break;
+
+					case CTriggerExprParserResult::TOKEN_TYPE_STRING:
+						$values[] = $token['data']['string'];
+						break;
+				}
+			}
 		}
 
-		// Replace functionids with string 'function' to make values search easier.
-		$expression = preg_replace('/\{[0-9]+\}/', 'function', $expression);
-
-		// Replace whitespace with emptyness to make value search easier.
-		$expression = str_replace(" \r\n\t", '', $expression);
-
-		// Search for numeric values in expression.
-		preg_match_all('/((?<![\)\.0-9]|[\.0-9]['.ZBX_BYTE_SUFFIXES.ZBX_TIME_SUFFIXES.']|function)\-?'.
-				'([.][0-9]+|[0-9]+[.]?[0-9]*)['.ZBX_BYTE_SUFFIXES.ZBX_TIME_SUFFIXES.']?)/', $expression, $values);
-
-		$macro_values = [];
-
-		foreach (array_keys($references) as $reference) {
-			$i = (int) $reference[1] - 1;
-			$macro_values[$reference] = array_key_exists($i, $values[0]) ? $values[0][$i] : '';
+		foreach ($references as $macro => $value) {
+			$i = (int) $macro[1] - 1;
+			$references[$macro] = array_key_exists($i, $values) ? $values[$i] : '';
 		}
 
-		return $macro_values;
+		return $references;
 	}
 
 	/**
@@ -106,7 +105,7 @@ class CMacrosResolverGeneral {
 	 * @return array
 	 */
 	protected function transformToPositionTypes(array $types) {
-		foreach (['macros', 'macros_n', 'macro_funcs_n'] as $type) {
+		foreach (['macros', 'macros_n', 'macros_an', 'macro_funcs_n'] as $type) {
 			if (array_key_exists($type, $types)) {
 				$patterns = [];
 				foreach ($types[$type] as $key => $_patterns) {
@@ -127,6 +126,7 @@ class CMacrosResolverGeneral {
 	 * @param bool   $types['usermacros']
 	 * @param array  $types['macros'][<macro_patterns>]
 	 * @param array  $types['macros_n'][<macro_patterns>]
+	 * @param array  $types['macros_an'][<macro_patterns>]
 	 * @param array  $types['macro_funcs_n'][<macro_patterns>]
 	 * @param bool   $types['references']
 	 * @param bool   $types['lldmacros']
@@ -140,6 +140,7 @@ class CMacrosResolverGeneral {
 		$extract_usermacros = array_key_exists('usermacros', $types);
 		$extract_macros = array_key_exists('macros', $types);
 		$extract_macros_n = array_key_exists('macros_n', $types);
+		$extract_macros_an = array_key_exists('macros_an', $types);
 		$extract_macro_funcs_n = array_key_exists('macro_funcs_n', $types);
 		$extract_references = array_key_exists('references', $types);
 		$extract_lldmacros = array_key_exists('lldmacros', $types);
@@ -155,11 +156,19 @@ class CMacrosResolverGeneral {
 		}
 
 		if ($extract_macros_n) {
-			$macro_n_parser = new CMacroParser($types['macros_n'], ['allow_reference' => true]);
+			$macro_n_parser = new CMacroParser($types['macros_n'], ['ref_type' => CMacroParser::REFERENCE_NUMERIC]);
+		}
+
+		if ($extract_macros_an) {
+			$macro_an_parser = new CMacroParser($types['macros_an'],
+				['ref_type' => CMacroParser::REFERENCE_ALPHANUMERIC]
+			);
 		}
 
 		if ($extract_macro_funcs_n) {
-			$macro_func_n_parser = new CMacroFunctionParser($types['macro_funcs_n'], ['allow_reference' => true]);
+			$macro_func_n_parser = new CMacroFunctionParser($types['macro_funcs_n'],
+				['ref_type' => CMacroParser::REFERENCE_NUMERIC]
+			);
 		}
 
 		if ($extract_references) {
@@ -192,6 +201,10 @@ class CMacrosResolverGeneral {
 				$macros[$pos] = $macro_n_parser->getMatch();
 				$pos += $macro_n_parser->getLength() - 1;
 			}
+			elseif ($extract_macros_an && $macro_an_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
+				$macros[$pos] = $macro_an_parser->getMatch();
+				$pos += $macro_an_parser->getLength() - 1;
+			}
 			elseif ($extract_macro_funcs_n && $macro_func_n_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
 				$macros[$pos] = $macro_func_n_parser->getMatch();
 				$pos += $macro_func_n_parser->getLength() - 1;
@@ -229,6 +242,7 @@ class CMacrosResolverGeneral {
 	 * @param bool   $types['usermacros']
 	 * @param array  $types['macros'][][<macro_patterns>]
 	 * @param array  $types['macros_n'][][<macro_patterns>]
+	 * @param array  $types['macros_an'][][<macro_patterns>]
 	 * @param array  $types['macro_funcs_n'][][<macro_patterns>]
 	 * @param bool   $types['references']
 	 * @param bool   $types['lldmacros']
@@ -236,11 +250,12 @@ class CMacrosResolverGeneral {
 	 *
 	 * @return array
 	 */
-	protected function extractMacros(array $texts, array $types) {
+	public static function extractMacros(array $texts, array $types) {
 		$macros = [];
 		$extract_usermacros = array_key_exists('usermacros', $types);
 		$extract_macros = array_key_exists('macros', $types);
 		$extract_macros_n = array_key_exists('macros_n', $types);
+		$extract_macros_an = array_key_exists('macros_an', $types);
 		$extract_macro_funcs_n = array_key_exists('macro_funcs_n', $types);
 		$extract_references = array_key_exists('references', $types);
 		$extract_lldmacros = array_key_exists('lldmacros', $types);
@@ -265,16 +280,31 @@ class CMacrosResolverGeneral {
 			$macros['macros_n'] = [];
 
 			foreach ($types['macros_n'] as $key => $macro_patterns) {
-				$types['macros_n'][$key] = new CMacroParser($macro_patterns, ['allow_reference' => true]);
+				$types['macros_n'][$key] = new CMacroParser($macro_patterns,
+					['ref_type' => CMacroParser::REFERENCE_NUMERIC]
+				);
 				$macros['macros_n'][$key] = [];
 			}
 		}
 
+		if ($extract_macros_an) {
+			$macros['macros_an'] = [];
+
+			foreach ($types['macros_an'] as $key => $macro_patterns) {
+				$types['macros_an'][$key] = new CMacroParser($macro_patterns,
+					['ref_type' => CMacroParser::REFERENCE_ALPHANUMERIC]
+				);
+				$macros['macros_an'][$key] = [];
+			}
+		}
+
 		if ($extract_macro_funcs_n) {
 			$macros['macro_funcs_n'] = [];
 
 			foreach ($types['macro_funcs_n'] as $key => $macro_patterns) {
-				$types['macro_funcs_n'][$key] = new CMacroFunctionParser($macro_patterns, ['allow_reference' => true]);
+				$types['macro_funcs_n'][$key] = new CMacroFunctionParser($macro_patterns,
+					['ref_type' => CMacroParser::REFERENCE_NUMERIC]
+				);
 				$macros['macro_funcs_n'][$key] = [];
 			}
 		}
@@ -321,7 +351,7 @@ class CMacrosResolverGeneral {
 						if ($macro_n_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
 							$macros['macros_n'][$key][$macro_n_parser->getMatch()] = [
 								'macro' => $macro_n_parser->getMacro(),
-								'f_num' => $macro_n_parser->getN()
+								'f_num' => $macro_n_parser->getReference()
 							];
 							$pos += $macro_n_parser->getLength() - 1;
 							continue 2;
@@ -329,6 +359,19 @@ class CMacrosResolverGeneral {
 					}
 				}
 
+				if ($extract_macros_an) {
+					foreach ($types['macros_an'] as $key => $macro_an_parser) {
+						if ($macro_an_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
+							$macros['macros_an'][$key][$macro_an_parser->getMatch()] = [
+								'macro' => $macro_an_parser->getMacro(),
+								'f_num' => $macro_an_parser->getReference()
+							];
+							$pos += $macro_an_parser->getLength() - 1;
+							continue 2;
+						}
+					}
+				}
+
 				if ($extract_macro_funcs_n) {
 					foreach ($types['macro_funcs_n'] as $key => $macro_func_n_parser) {
 						if ($macro_func_n_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
@@ -350,7 +393,7 @@ class CMacrosResolverGeneral {
 
 							$macros['macro_funcs_n'][$key][$macro_func_n_parser->getMatch()] = [
 								'macro' => $macro_n_parser->getMacro(),
-								'f_num' => $macro_n_parser->getN(),
+								'f_num' => $macro_n_parser->getReference(),
 								'function' => $function_parser->getFunction(),
 								'parameters' => $function_parameters
 							];
@@ -443,7 +486,7 @@ class CMacrosResolverGeneral {
 			$item_key_parameters = $this->getItemKeyParameters($item_key_parser->getParamsRaw());
 		}
 
-		return $this->extractMacros($item_key_parameters, $types);
+		return self::extractMacros($item_key_parameters, $types);
 	}
 
 	/**
@@ -472,7 +515,7 @@ class CMacrosResolverGeneral {
 			}
 		}
 
-		return $this->extractMacros($function_parameters, $types);
+		return self::extractMacros($function_parameters, $types);
 	}
 
 	/**
@@ -688,31 +731,52 @@ class CMacrosResolverGeneral {
 	 * @param array $macros[<functionid>][<macro>]  An array of the tokens.
 	 * @param array $macro_values
 	 * @param array $triggers
-	 * @param bool  $events                         Resolve {ITEM.VALUE} macro using 'clock' and 'ns' fields.
+	 * @param array $options
+	 * @param bool  $options['events]               Resolve {ITEM.VALUE} macro using 'clock' and 'ns' fields.
+	 * @param bool  $options['html]
 	 *
 	 * @return array
 	 */
-	protected function getItemMacros(array $macros, array $macro_values, array $triggers = [], $events = false) {
+	protected function getItemMacros(array $macros, array $macro_values, array $triggers = [], array $options = []) {
 		if (!$macros) {
 			return $macro_values;
 		}
 
+		$options += [
+			'events' => false,
+			'html' => false
+		];
+
 		$functions = DBfetchArray(DBselect(
-			'SELECT f.triggerid,f.functionid,i.itemid,i.value_type,i.units,i.valuemapid'.
+			'SELECT f.triggerid,f.functionid,i.itemid,i.hostid,i.name,i.key_,i.value_type,i.units,i.valuemapid'.
 			' FROM functions f'.
 				' JOIN items i ON f.itemid=i.itemid'.
 				' JOIN hosts h ON i.hostid=h.hostid'.
 			' WHERE '.dbConditionInt('f.functionid', array_keys($macros))
 		));
 
+		$functions = CMacrosResolverHelper::resolveItemNames($functions);
+
 		// False passed to DBfetch to get data without null converted to 0, which is done by default.
 		foreach ($functions as $function) {
 			foreach ($macros[$function['functionid']] as $m => $tokens) {
+				$clock = null;
+				$value = null;
+
 				switch ($m) {
 					case 'ITEM.VALUE':
-						if ($events) {
+						if ($options['events']) {
 							$trigger = $triggers[$function['triggerid']];
-							$value = Manager::History()->getValueAt($function, $trigger['clock'], $trigger['ns']);
+							$history = Manager::History()->getValueAt($function, $trigger['clock'], $trigger['ns']);
+
+							if (is_array($history)) {
+								if (array_key_exists('clock', $history)) {
+									$clock = $history['clock'];
+								}
+								if (array_key_exists('value', $history)) {
+									$value = $history['value'];
+								}
+							}
 							break;
 						}
 						// break; is not missing here
@@ -720,14 +784,17 @@ class CMacrosResolverGeneral {
 					case 'ITEM.LASTVALUE':
 						$history = Manager::History()->getLastValues([$function], 1, ZBX_HISTORY_PERIOD);
 
-						$value = array_key_exists($function['itemid'], $history)
-							? $history[$function['itemid']][0]['value']
-							: null;
+						if (array_key_exists($function['itemid'], $history)) {
+							$clock = $history[$function['itemid']][0]['clock'];
+							$value = $history[$function['itemid']][0]['value'];
+						}
 						break;
 				}
 
-				if ($value !== null) {
-					foreach ($tokens as $token) {
+				foreach ($tokens as $token) {
+					$macro_value = UNRESOLVED_MACRO_STRING;
+
+					if ($value !== null) {
 						if (array_key_exists('function', $token)) {
 							if ($token['function'] !== 'regsub' && $token['function'] !== 'iregsub') {
 								continue;
@@ -760,9 +827,46 @@ class CMacrosResolverGeneral {
 						else {
 							$macro_value = formatHistoryValue($value, $function);
 						}
-
-						$macro_values[$function['triggerid']][$token['token']] = $macro_value;
 					}
+
+					if ($options['html']) {
+						$macro_value = str_replace(["\r\n", "\n"], [" "], $macro_value);
+						$hint_table = (new CTable())
+							->addClass('list-table')
+							->addRow([
+								new CCol($function['name_expanded']),
+								new CCol(
+									($clock !== null)
+										? zbx_date2str(DATE_TIME_FORMAT_SECONDS, $clock)
+										: UNRESOLVED_MACRO_STRING
+								),
+								new CCol($macro_value),
+								new CCol(
+									($function['value_type'] == ITEM_VALUE_TYPE_FLOAT
+											|| $function['value_type'] == ITEM_VALUE_TYPE_UINT64)
+										? new CLink(_('Graph'), (new CUrl('history.php'))
+											->setArgument('action', HISTORY_GRAPH)
+											->setArgument('itemids[]', $function['itemid'])
+											->getUrl()
+										)
+										: new CLink(_('History'), (new CUrl('history.php'))
+											->setArgument('action', HISTORY_VALUES)
+											->setArgument('itemids[]', $function['itemid'])
+											->getUrl()
+										)
+								)
+							]);
+						$macro_value = new CSpan([
+							(new CSpan())
+								->addClass('main-hint')
+								->setHint($hint_table),
+							(new CLinkAction($macro_value))
+								->addClass('hint-item')
+								->setAttribute('data-hintbox', '1')
+						]);
+					}
+
+					$macro_values[$function['triggerid']][$token['token']] = $macro_value;
 				}
 			}
 		}
@@ -843,12 +947,13 @@ class CMacrosResolverGeneral {
 	 * Get macros with values.
 	 *
 	 * @param array $data
-	 * @param array $data[n]['hostids']  the list of host ids; [<hostid1>, ...]
-	 * @param array $data[n]['macros']   the list of user macros to resolve, ['<usermacro1>' => null, ...]
+	 * @param array $data[n]['hostids']  The list of host ids; [<hostid1>, ...].
+	 * @param array $data[n]['macros']   The list of user macros to resolve, ['<usermacro1>' => null, ...].
+	 * @param bool  $unset_undefined     Unset undefined macros.
 	 *
 	 * @return array
 	 */
-	protected function getUserMacros(array $data) {
+	protected function getUserMacros(array $data, bool $unset_undefined = false) {
 		if (!$data) {
 			return $data;
 		}
@@ -873,8 +978,10 @@ class CMacrosResolverGeneral {
 		 * @var array  $host_macros
 		 * @var array  $host_macros[<hostid>]
 		 * @var array  $host_macros[<hostid>][<macro>]				macro base without curly braces
-		 * @var string $host_macros[<hostid>][<macro>]['value']		base macro value (without context); can be null
-		 * @var array  $host_macros[<hostid>][<macro>]['contexts']	context values; ['<context>' => '<value>', ...]
+		 * @var string $host_macros[<hostid>][<macro>]['value']		base macro value (without context and regex);
+		 * 															can be null
+		 * @var array  $host_macros[<hostid>][<macro>]['contexts']	context values; ['<context1>' => '<value1>', ...]
+		 * @var array  $host_macros[<hostid>][<macro>]['regex']		regex values; ['<regex1>' => '<value1>', ...]
 		 */
 		$host_macros = [];
 
@@ -882,32 +989,38 @@ class CMacrosResolverGeneral {
 			do {
 				$hostids = array_keys($hostids);
 
-				$db_host_macros = DBselect(
-					'SELECT hm.hostid,hm.macro,hm.value'.
-					' FROM hostmacro hm'.
-					' WHERE '.dbConditionInt('hm.hostid', $hostids)
-				);
-				while ($db_host_macro = DBfetch($db_host_macros)) {
+				$db_host_macros = API::UserMacro()->get([
+					'output' => ['macro', 'value', 'type', 'hostid'],
+					'hostids' => $hostids
+				]);
+
+				foreach ($db_host_macros as $db_host_macro) {
 					if ($user_macro_parser->parse($db_host_macro['macro']) != CParser::PARSE_SUCCESS) {
 						continue;
 					}
 
+					$hostid = $db_host_macro['hostid'];
 					$macro = $user_macro_parser->getMacro();
 					$context = $user_macro_parser->getContext();
+					$regex = $user_macro_parser->getRegex();
+					$value = self::getMacroValue($db_host_macro);
 
-					if (!array_key_exists($db_host_macro['hostid'], $host_macros)) {
-						$host_macros[$db_host_macro['hostid']] = [];
+					if (!array_key_exists($hostid, $host_macros)) {
+						$host_macros[$hostid] = [];
 					}
 
-					if (!array_key_exists($macro, $host_macros[$db_host_macro['hostid']])) {
-						$host_macros[$db_host_macro['hostid']][$macro] = ['value' => null, 'contexts' => []];
+					if (!array_key_exists($macro, $host_macros[$hostid])) {
+						$host_macros[$hostid][$macro] = ['value' => null, 'contexts' => [], 'regex' => []];
 					}
 
-					if ($context === null) {
-						$host_macros[$db_host_macro['hostid']][$macro]['value'] = $db_host_macro['value'];
+					if ($context === null && $regex === null) {
+						$host_macros[$hostid][$macro]['value'] = $value;
+					}
+					elseif ($regex !== null) {
+						$host_macros[$hostid][$macro]['regex'][$regex] = $value;
 					}
 					else {
-						$host_macros[$db_host_macro['hostid']][$macro]['contexts'][$context] = $db_host_macro['value'];
+						$host_macros[$hostid][$macro]['contexts'][$context] = $value;
 					}
 				}
 
@@ -936,6 +1049,9 @@ class CMacrosResolverGeneral {
 			} while ($hostids);
 		}
 
+		// Reordering only regex array.
+		$host_macros = self::sortRegexHostMacros($host_macros);
+
 		$all_macros_resolved = true;
 
 		foreach ($data as &$element) {
@@ -969,15 +1085,17 @@ class CMacrosResolverGeneral {
 		if (!$all_macros_resolved) {
 			// Global macros.
 			$db_global_macros = API::UserMacro()->get([
-				'output' => ['macro', 'value'],
+				'output' => ['macro', 'value', 'type'],
 				'globalmacro' => true
 			]);
 
 			/*
 			 * @var array  $global_macros
 			 * @var array  $global_macros[<macro>]				macro base without curly braces
-			 * @var string $global_macros[<macro>]['value']		base macro value (without context); can be null
+			 * @var string $global_macros[<macro>]['value']		base macro value (without context and regex);
+			 * 													can be null
 			 * @var array  $global_macros[<macro>]['contexts']	context values; ['<context1>' => '<value1>', ...]
+			 * @var array  $global_macros[<macro>]['regex']		regex values; ['<regex1>' => '<value1>', ...]
 			 */
 			$global_macros = [];
 
@@ -985,20 +1103,28 @@ class CMacrosResolverGeneral {
 				if ($user_macro_parser->parse($db_global_macro['macro']) == CParser::PARSE_SUCCESS) {
 					$macro = $user_macro_parser->getMacro();
 					$context = $user_macro_parser->getContext();
+					$regex = $user_macro_parser->getRegex();
+					$value = self::getMacroValue($db_global_macro);
 
 					if (!array_key_exists($macro, $global_macros)) {
-						$global_macros[$macro] = ['value' => null, 'contexts' => []];
+						$global_macros[$macro] = ['value' => null, 'contexts' => [], 'regex' => []];
 					}
 
-					if ($context === null) {
-						$global_macros[$macro]['value'] = $db_global_macro['value'];
+					if ($context === null && $regex === null) {
+						$global_macros[$macro]['value'] = $value;
+					}
+					elseif ($regex !== null) {
+						$global_macros[$macro]['regex'][$regex] = $value;
 					}
 					else {
-						$global_macros[$macro]['contexts'][$context] = $db_global_macro['value'];
+						$global_macros[$macro]['contexts'][$context] = $value;
 					}
 				}
 			}
 
+			// Reordering only regex array.
+			$global_macros = self::sortRegexGlobalMacros($global_macros);
+
 			foreach ($data as &$element) {
 				foreach ($element['macros'] as $usermacro => &$value) {
 					if ($value['value'] === null && $user_macro_parser->parse($usermacro) == CParser::PARSE_SUCCESS) {
@@ -1009,7 +1135,16 @@ class CMacrosResolverGeneral {
 							if ($context !== null && array_key_exists($context, $global_macros[$macro]['contexts'])) {
 								$value['value'] = $global_macros[$macro]['contexts'][$context];
 							}
-							elseif ($global_macros[$macro]['value'] !== null) {
+							elseif ($context !== null && count($global_macros[$macro]['regex'])) {
+								foreach ($global_macros[$macro]['regex'] as $regex => $val) {
+									if (preg_match('/'.trim($regex, '/').'/', $context) === 1) {
+										$value['value'] = $val;
+										break;
+									}
+								}
+							}
+
+							if ($value['value'] === null && $global_macros[$macro]['value'] !== null) {
 								if ($context === null) {
 									$value['value'] = $global_macros[$macro]['value'];
 								}
@@ -1025,22 +1160,23 @@ class CMacrosResolverGeneral {
 			unset($element);
 		}
 
-		foreach ($data as &$element) {
-			foreach ($element['macros'] as $usermacro => &$value) {
+		foreach ($data as $key => $element) {
+			foreach ($element['macros'] as $usermacro => $value) {
 				if ($value['value'] !== null) {
-					$value = $value['value'];
+					$data[$key]['macros'][$usermacro] = $value['value'];
 				}
 				elseif ($value['value_default'] !== null) {
-					$value = $value['value_default'];
+					$data[$key]['macros'][$usermacro] = $value['value_default'];
+				}
+				// Unresolved macro.
+				elseif ($unset_undefined) {
+					unset($data[$key]['macros'][$usermacro]);
 				}
 				else {
-					// Unresolved macro.
-					$value = $usermacro;
+					$data[$key]['macros'][$usermacro] = $usermacro;
 				}
 			}
-			unset($value);
 		}
-		unset($element);
 
 		return $data;
 	}
@@ -1068,12 +1204,24 @@ class CMacrosResolverGeneral {
 			$value_default = null) {
 		foreach ($hostids as $hostid) {
 			if (array_key_exists($hostid, $host_macros) && array_key_exists($macro, $host_macros[$hostid])) {
+				// Searching context coincidence with macro contexts.
 				if ($context !== null && array_key_exists($context, $host_macros[$hostid][$macro]['contexts'])) {
 					return [
 						'value' => $host_macros[$hostid][$macro]['contexts'][$context],
 						'value_default' => $value_default
 					];
 				}
+				// Searching context coincidence, if regex array not empty.
+				elseif ($context !== null && count($host_macros[$hostid][$macro]['regex'])) {
+					foreach ($host_macros[$hostid][$macro]['regex'] as $regex => $val) {
+						if (preg_match('/'.trim($regex, '/').'/', $context) === 1) {
+							return [
+								'value' => $val,
+								'value_default' => $value_default
+							];
+						}
+					}
+				}
 
 				if ($host_macros[$hostid][$macro]['value'] !== null) {
 					if ($context === null) {
@@ -1111,4 +1259,77 @@ class CMacrosResolverGeneral {
 
 		return ['value' => null, 'value_default' => $value_default];
 	}
+
+	/**
+	 * Get macro value refer by type.
+	 *
+	 * @static
+	 *
+	 * @param array $macro
+	 *
+	 * @return string
+	 */
+	public static function getMacroValue(array $macro): string {
+		return ($macro['type'] == ZBX_MACRO_TYPE_SECRET) ? ZBX_SECRET_MASK : $macro['value'];
+	}
+
+	/**
+	 * Sorting host macros.
+	 *
+	 * @param array $host_macros
+	 *
+	 * @return array
+	 */
+	private static function sortRegexHostMacros(array $host_macros): array {
+		foreach ($host_macros as &$macros) {
+			foreach ($macros as &$value) {
+				$value['regex'] = self::sortRegex($value['regex']);
+			}
+			unset($value);
+		}
+		unset($macros);
+
+		return $host_macros;
+	}
+
+	/**
+	 * Sorting global macros.
+	 *
+	 * @static
+	 *
+	 * @param array $global_macros
+	 *
+	 * @return array
+	 */
+	private static function sortRegexGlobalMacros(array $global_macros): array {
+		foreach ($global_macros as &$value) {
+			$value['regex'] = self::sortRegex($value['regex']);
+		}
+		unset($value);
+
+		return $global_macros;
+	}
+
+	/**
+	 * Sort regex.
+	 *
+	 * @static
+	 *
+	 * @param array $macros
+	 *
+	 * @return array
+	 */
+	private static function sortRegex(array $macros): array {
+		$keys = array_keys($macros);
+
+		usort($keys, 'strcmp');
+
+		$new_array = [];
+
+		foreach($keys as $key) {
+			$new_array[$key] = $macros[$key];
+		}
+
+		return $new_array;
+	}
 }
