@@ -19,14 +19,23 @@
 **/
 
 
+/**
+ * Setup wizzard form.
+ */
 class CSetupWizard extends CForm {
 
-	function __construct() {
-		$this->DISABLE_CANCEL_BUTTON = false;
-		$this->DISABLE_BACK_BUTTON = false;
-		$this->SHOW_RETRY_BUTTON = false;
-		$this->STEP_FAILED = false;
-		$this->frontendSetup = new CFrontendSetup();
+	protected $DISABLE_CANCEL_BUTTON = false;
+
+	protected $DISABLE_BACK_BUTTON = false;
+
+	protected $SHOW_RETRY_BUTTON = false;
+
+	protected $STEP_FAILED = false;
+
+	protected $frontend_setup;
+
+	public function __construct() {
+		$this->frontend_setup = new CFrontendSetup();
 
 		$this->stage = [
 			0 => [
@@ -58,21 +67,22 @@ class CSetupWizard extends CForm {
 		$this->eventHandler();
 
 		parent::__construct('post');
+		parent::setId('setup-form');
 	}
 
-	function getConfig($name, $default = null) {
+	private function getConfig(string $name, $default = null) {
 		return CSession::keyExists($name) ? CSession::getValue($name) : $default;
 	}
 
-	function setConfig($name, $value) {
+	private function setConfig(string $name, $value) {
 		CSession::setValue($name, $value);
 	}
 
-	function getStep() {
+	private function getStep(): int {
 		return $this->getConfig('step', 0);
 	}
 
-	function doNext() {
+	private function doNext(): bool {
 		if (isset($this->stage[$this->getStep() + 1])) {
 			$this->setConfig('step', $this->getStep('step') + 1);
 
@@ -82,7 +92,7 @@ class CSetupWizard extends CForm {
 		return false;
 	}
 
-	function doBack() {
+	private function doBack(): bool {
 		if (isset($this->stage[$this->getStep() - 1])) {
 			$this->setConfig('step', $this->getStep('step') - 1);
 
@@ -92,7 +102,7 @@ class CSetupWizard extends CForm {
 		return false;
 	}
 
-	protected function bodyToString($destroy = true) {
+	protected function bodyToString(bool $destroy = true): string {
 		$setup_left = (new CDiv())
 			->addClass(ZBX_STYLE_SETUP_LEFT)
 			->addItem((new CDiv(makeLogo(LOGO_TYPE_NORMAL)))->addClass('setup-logo'))
@@ -135,7 +145,7 @@ class CSetupWizard extends CForm {
 		return parent::bodyToString($destroy).$setup_container->toString();
 	}
 
-	function getList() {
+	private function getList(): CList {
 		$list = new CList();
 
 		foreach ($this->stage as $id => $data) {
@@ -145,19 +155,19 @@ class CSetupWizard extends CForm {
 		return $list;
 	}
 
-	function getStage() {
+	private function getStage(): array {
 		$function = $this->stage[$this->getStep()]['fnc'];
 		return $this->$function();
 	}
 
-	function stage0() {
+	private function stage0(): array {
 		preg_match('/^\d+\.\d+/', ZABBIX_VERSION, $version);
 		$setup_title = (new CDiv([new CSpan(_('Welcome to')), 'Zabbix '.$version[0]]))->addClass(ZBX_STYLE_SETUP_TITLE);
 
-		return (new CDiv($setup_title))->addClass(ZBX_STYLE_SETUP_RIGHT_BODY);
+		return [(new CDiv($setup_title))->addClass(ZBX_STYLE_SETUP_RIGHT_BODY)];
 	}
 
-	function stage1() {
+	private function stage1(): array {
 		$table = (new CTable())
 			->addClass(ZBX_STYLE_LIST_TABLE)
 			->setHeader(['', _('Current value'), _('Required'), '']);
@@ -165,7 +175,7 @@ class CSetupWizard extends CForm {
 		$messages = [];
 		$finalResult = CFrontendSetup::CHECK_OK;
 
-		foreach ($this->frontendSetup->checkRequirements() as $req) {
+		foreach ($this->frontend_setup->checkRequirements() as $req) {
 			if ($req['result'] == CFrontendSetup::CHECK_OK) {
 				$class = ZBX_STYLE_GREEN;
 				$result = 'OK';
@@ -207,15 +217,18 @@ class CSetupWizard extends CForm {
 		];
 	}
 
-	function stage2() {
-		$DB['TYPE'] = $this->getConfig('DB_TYPE');
+	private function stage2(): array {
+		$DB['TYPE'] = $this->getConfig('DB_TYPE', key(CFrontendSetup::getSupportedDatabases()));
 
 		$table = (new CFormList())
-			->addVar('tls_encryption', '0', 'tls_encryption_off')
-			->addVar('verify_host', '0', 'verify_host_off');
+			->addItem([
+				(new CVar('tls_encryption', 0))->removeId(),
+				(new CVar('verify_certificate', 0))->removeId(),
+				(new CVar('verify_host', 0))->removeId()
+			]);
 
 		$table->addRow(_('Database type'),
-			new CComboBox('type', $DB['TYPE'], 'submit()', CFrontendSetup::getSupportedDatabases())
+			new CComboBox('type', $DB['TYPE'], null, CFrontendSetup::getSupportedDatabases())
 		);
 
 		$table->addRow(_('Database host'),
@@ -236,11 +249,12 @@ class CSetupWizard extends CForm {
 				->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 		);
 
-		if ($DB['TYPE'] == ZBX_DB_POSTGRESQL) {
-			$table->addRow(_('Database schema'),
-				(new CTextBox('schema', $this->getConfig('DB_SCHEMA', '')))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
-			);
-		}
+		$table->addRow(_('Database schema'),
+			(new CTextBox('schema', $this->getConfig('DB_SCHEMA', '')))
+				->setWidth(ZBX_TEXTAREA_SMALL_WIDTH),
+			'db_schema_row',
+			ZBX_STYLE_DISPLAY_NONE
+		);
 
 		$table->addRow(_('User'),
 			(new CTextBox('user', $this->getConfig('DB_USER', 'zabbix')))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
@@ -249,58 +263,53 @@ class CSetupWizard extends CForm {
 			(new CPassBox('password', $this->getConfig('DB_PASSWORD')))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 		);
 
-		If ($DB['TYPE'] === null || $DB['TYPE'] == ZBX_DB_MYSQL || $DB['TYPE'] == ZBX_DB_POSTGRESQL) {
-			$table->addRow(_('TLS encryption'),
-				(new CCheckBox('tls_encryption'))
-					->setChecked($this->getConfig('DB_ENCRYPTION'))
-					->onChange('submit()')
-			);
-			$show_tls = true;
-		}
-		else {
-			$show_tls = false;
-		}
+		$table->addRow(_('Database TLS encryption'), [
+				(new CCheckBox('tls_encryption'))->setChecked($this->getConfig('DB_ENCRYPTION', true)),
+				(new CDiv(
+					_('Connection will not be encrypted because it uses a socket file (on Unix) or shared memory (Windows).')
+				))
+					->setId('tls_encryption_hint')
+					->addClass(ZBX_STYLE_DISPLAY_NONE)
+			],
+			'db_encryption_row',
+			ZBX_STYLE_DISPLAY_NONE
+		);
 
-		if ($show_tls && $this->getConfig('DB_ENCRYPTION')) {
-			$table->addRow(_('TLS key file'),
-				(new CTextBox('key_file', $this->getConfig('DB_KEY_FILE')))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-			);
+		$table->addRow(_('Verify database certificate'),
+			(new CCheckBox('verify_certificate'))->setChecked($this->getConfig('DB_ENCRYPTION_ADVANCED')),
+			'db_verify_host',
+			ZBX_STYLE_DISPLAY_NONE
+		);
 
-			$table->addRow(_('TLS certificate file'),
-				(new CTextBox('cert_file', $this->getConfig('DB_CERT_FILE')))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-			);
+		$table->addRow((new CLabel(_('Database TLS CA file')))->setAsteriskMark(),
+			(new CTextBox('ca_file', $this->getConfig('DB_CA_FILE')))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH),
+			'db_cafile_row',
+			ZBX_STYLE_DISPLAY_NONE
+		);
 
-			$table->addRow(_('TLS certificate authority file'),
-				(new CTextBox('ca_file', $this->getConfig('DB_CA_FILE')))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-			);
+		$table->addRow(_('Database TLS key file'),
+			(new CTextBox('key_file', $this->getConfig('DB_KEY_FILE')))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH),
+			'db_keyfile_row',
+			ZBX_STYLE_DISPLAY_NONE
+		);
 
-			$verify_host_box = new CCheckBox('verify_host');
+		$table->addRow(_('Database TLS certificate file'),
+			(new CTextBox('cert_file', $this->getConfig('DB_CERT_FILE')))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH),
+			'db_certfile_row',
+			ZBX_STYLE_DISPLAY_NONE
+		);
 
-			if ($DB['TYPE'] == ZBX_DB_MYSQL) {
-				$verify_host_box
-					->setChecked(true)
-					->setAttribute('readonly', true);
-			}
-			else {
-				$verify_host_box->setChecked($this->getConfig('DB_VERIFY_HOST'));
-			}
+		$table->addRow(_('Database host verification'),
+			(new CCheckBox('verify_host'))->setChecked($this->getConfig('DB_VERIFY_HOST')),
+			'db_verify_host_row',
+			ZBX_STYLE_DISPLAY_NONE
+		);
 
-			$table->addRow(_('With host verification'), $verify_host_box);
-
-			If ($DB['TYPE'] == ZBX_DB_MYSQL) {
-				$table->addRow(_('TLS cipher list'),
-					(new CTextBox('cipher_list', $this->getConfig('DB_CIPHER_LIST')))
-						->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-				);
-			}
-		}
-		else {
-			$table
-				->addVar('key_file', '')
-				->addVar('cert_file', '')
-				->addVar('ca_file', '')
-				->addVar('cipher_list', '');
-		}
+		$table->addRow(_('Database TLS cipher list'),
+			(new CTextBox('cipher_list', $this->getConfig('DB_CIPHER_LIST')))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH),
+			'db_cipher_row',
+			ZBX_STYLE_DISPLAY_NONE
+		);
 
 		if ($this->STEP_FAILED) {
 			global $ZBX_MESSAGES;
@@ -321,7 +330,7 @@ class CSetupWizard extends CForm {
 		];
 	}
 
-	function stage3() {
+	private function stage3(): array {
 		$table = new CFormList();
 
 		$table->addRow(_('Host'),
@@ -349,7 +358,7 @@ class CSetupWizard extends CForm {
 		];
 	}
 
-	function stage4() {
+	private function stage4(): array {
 		$db_type = $this->getConfig('DB_TYPE');
 		$databases = CFrontendSetup::getSupportedDatabases();
 
@@ -364,32 +373,41 @@ class CSetupWizard extends CForm {
 		$table->addRow((new CSpan(_('Database name')))->addClass(ZBX_STYLE_GREY), $this->getConfig('DB_DATABASE'));
 		$table->addRow((new CSpan(_('Database user')))->addClass(ZBX_STYLE_GREY), $this->getConfig('DB_USER'));
 		$table->addRow((new CSpan(_('Database password')))->addClass(ZBX_STYLE_GREY), $db_password);
-		if ($db_type == ZBX_DB_POSTGRESQL) {
+		if ($db_type === ZBX_DB_POSTGRESQL) {
 			$table->addRow((new CSpan(_('Database schema')))->addClass(ZBX_STYLE_GREY), $this->getConfig('DB_SCHEMA'));
 		}
-		$table->addRow((new CSpan(_('TLS encryption')))->addClass(ZBX_STYLE_GREY),
-			$this->getConfig('DB_ENCRYPTION') ? 'true' : 'false');
-		if ($this->getConfig('DB_ENCRYPTION')) {
-			$table->addRow((new CSpan(_('TLS key file')))->addClass(ZBX_STYLE_GREY), $this->getConfig('DB_KEY_FILE'));
-			$table->addRow((new CSpan(_('TLS certificate file')))->addClass(ZBX_STYLE_GREY),
-				$this->getConfig('DB_CERT_FILE')
-			);
-			$table->addRow((new CSpan(_('TLS certificate authority file')))->addClass(ZBX_STYLE_GREY),
+		$table->addRow((new CSpan(_('Database TLS encryption')))->addClass(ZBX_STYLE_GREY),
+			$this->getConfig('DB_ENCRYPTION') ? 'true' : 'false'
+		);
+		if ($this->getConfig('DB_ENCRYPTION_ADVANCED')) {
+			$table->addRow((new CSpan(_('Database TLS CA file')))->addClass(ZBX_STYLE_GREY),
 				$this->getConfig('DB_CA_FILE')
 			);
-			$table->addRow((new CSpan(_('With host verification')))->addClass(ZBX_STYLE_GREY),
+			$table->addRow((new CSpan(_('Database TLS key file')))->addClass(ZBX_STYLE_GREY),
+				$this->getConfig('DB_KEY_FILE')
+			);
+			$table->addRow((new CSpan(_('Database TLS certificate file')))->addClass(ZBX_STYLE_GREY),
+				$this->getConfig('DB_CERT_FILE')
+			);
+			$table->addRow((new CSpan(_('Database host verification')))->addClass(ZBX_STYLE_GREY),
 				$this->getConfig('DB_VERIFY_HOST') ? 'true' : 'false'
 			);
-			$table->addRow((new CSpan(_('TLS cipher list')))->addClass(ZBX_STYLE_GREY),
-				$this->getConfig('DB_CIPHER_LIST')
-			);
+			if ($db_type === ZBX_DB_MYSQL) {
+				$table->addRow((new CSpan(_('Database TLS cipher list')))->addClass(ZBX_STYLE_GREY),
+					$this->getConfig('DB_CIPHER_LIST')
+				);
+			}
 		}
 
 		$table->addRow(null, null);
 
 		$table->addRow((new CSpan(_('Zabbix server')))->addClass(ZBX_STYLE_GREY), $this->getConfig('ZBX_SERVER'));
-		$table->addRow((new CSpan(_('Zabbix server port')))->addClass(ZBX_STYLE_GREY), $this->getConfig('ZBX_SERVER_PORT'));
-		$table->addRow((new CSpan(_('Zabbix server name')))->addClass(ZBX_STYLE_GREY), $this->getConfig('ZBX_SERVER_NAME'));
+		$table->addRow((new CSpan(_('Zabbix server port')))->addClass(ZBX_STYLE_GREY),
+			$this->getConfig('ZBX_SERVER_PORT')
+		);
+		$table->addRow((new CSpan(_('Zabbix server name')))->addClass(ZBX_STYLE_GREY),
+			$this->getConfig('ZBX_SERVER_NAME')
+		);
 
 		return [
 			new CTag('h1', true, _('Pre-installation summary')),
@@ -400,7 +418,7 @@ class CSetupWizard extends CForm {
 		];
 	}
 
-	function stage5() {
+	private function stage5(): array {
 		$this->setConfig('ZBX_CONFIG_FILE_CORRECT', true);
 
 		$config_file_name = APP::getInstance()->getRootDir().CConfigFile::CONFIG_FILE_PATH;
@@ -469,7 +487,7 @@ class CSetupWizard extends CForm {
 		];
 	}
 
-	function dbConnect() {
+	private function dbConnect() {
 		global $DB;
 
 		if (!$this->getConfig('check_fields_result')) {
@@ -477,7 +495,7 @@ class CSetupWizard extends CForm {
 		}
 
 		$DB['TYPE'] = $this->getConfig('DB_TYPE');
-		if (is_null($DB['TYPE'])) {
+		if ($DB['TYPE'] === null) {
 			return false;
 		}
 
@@ -496,6 +514,23 @@ class CSetupWizard extends CForm {
 
 		$error = '';
 
+		// Check certificate files exists.
+		if ($DB['ENCRYPTION'] && ($DB['TYPE'] === ZBX_DB_MYSQL || $DB['TYPE'] === ZBX_DB_POSTGRESQL)) {
+			if (($this->getConfig('DB_ENCRYPTION_ADVANCED') || $DB['CA_FILE'] !== '') && !file_exists($DB['CA_FILE'])) {
+				return _s('Incorrect file path for "%1$s": %2$s.', _('Database TLS CA file'), $DB['CA_FILE']);
+			}
+
+			if ($DB['KEY_FILE'] !== '' && !file_exists($DB['KEY_FILE'])) {
+				return _s('Incorrect file path for "%1$s": %2$s.', _('Database TLS key file'), $DB['KEY_FILE']);
+			}
+
+			if ($DB['CERT_FILE'] !== '' && !file_exists($DB['CERT_FILE'])) {
+				return _s('Incorrect file path for "%1$s": %2$s.', _('Database TLS certificate file'),
+					$DB['CERT_FILE']
+				);
+			}
+		}
+
 		// During setup set debug to false to avoid displaying unwanted PHP errors in messages.
 		if (DBconnect($error)) {
 			return true;
@@ -505,7 +540,7 @@ class CSetupWizard extends CForm {
 		}
 	}
 
-	function dbClose() {
+	private function dbClose(): void {
 		global $DB;
 
 		DBclose();
@@ -513,7 +548,7 @@ class CSetupWizard extends CForm {
 		$DB = null;
 	}
 
-	function checkConnection() {
+	private function checkConnection() {
 		global $DB;
 
 		$result = true;
@@ -538,7 +573,7 @@ class CSetupWizard extends CForm {
 		return $result;
 	}
 
-	function eventHandler() {
+	private function eventHandler(): void {
 		if (hasRequest('back') && array_key_exists($this->getStep(), getRequest('back'))) {
 			$this->doBack();
 		}
@@ -546,7 +581,7 @@ class CSetupWizard extends CForm {
 		if ($this->getStep() == 1) {
 			if (hasRequest('next') && array_key_exists(1, getRequest('next'))) {
 				$finalResult = CFrontendSetup::CHECK_OK;
-				foreach ($this->frontendSetup->checkRequirements() as $req) {
+				foreach ($this->frontend_setup->checkRequirements() as $req) {
 					if ($req['result'] > $finalResult) {
 						$finalResult = $req['result'];
 					}
@@ -562,23 +597,40 @@ class CSetupWizard extends CForm {
 			}
 		}
 		elseif ($this->getStep() == 2) {
-			$this->setConfig('DB_TYPE', getRequest('type', $this->getConfig('DB_TYPE')));
-			$this->setConfig('DB_SERVER', getRequest('server', $this->getConfig('DB_SERVER', 'localhost')));
-			$this->setConfig('DB_PORT', getRequest('port', $this->getConfig('DB_PORT', '0')));
-			$this->setConfig('DB_DATABASE', getRequest('database', $this->getConfig('DB_DATABASE', 'zabbix')));
-			$this->setConfig('DB_USER', getRequest('user', $this->getConfig('DB_USER', 'root')));
-			$this->setConfig('DB_PASSWORD', getRequest('password', $this->getConfig('DB_PASSWORD', '')));
-			$this->setConfig('DB_SCHEMA', getRequest('schema', $this->getConfig('DB_SCHEMA', '')));
-			$this->setConfig('DB_ENCRYPTION',
-				getRequest('tls_encryption', $this->getConfig('DB_ENCRYPTION', true))
-			);
-			$this->setConfig('DB_VERIFY_HOST',
-				getRequest('verify_host', $this->getConfig('DB_VERIFY_HOST', true))
-			);
-			$this->setConfig('DB_KEY_FILE', getRequest('key_file', $this->getConfig('DB_KEY_FILE', '')));
-			$this->setConfig('DB_CERT_FILE', getRequest('cert_file', $this->getConfig('DB_CERT_FILE', '')));
-			$this->setConfig('DB_CA_FILE', getRequest('ca_file', $this->getConfig('DB_CA_FILE', '')));
-			$this->setConfig('DB_CIPHER_LIST', getRequest('cipher_list', $this->getConfig('DB_CIPHER_LIST', '')));
+			$input = [
+				'DB_TYPE' => getRequest('type', $this->getConfig('DB_TYPE')),
+				'DB_SERVER' => getRequest('server', $this->getConfig('DB_SERVER', 'localhost')),
+				'DB_PORT' => getRequest('port', $this->getConfig('DB_PORT', '0')),
+				'DB_DATABASE' => getRequest('database', $this->getConfig('DB_DATABASE', 'zabbix')),
+				'DB_USER' => getRequest('user', $this->getConfig('DB_USER', 'root')),
+				'DB_PASSWORD' => getRequest('password', $this->getConfig('DB_PASSWORD', '')),
+				'DB_SCHEMA' => getRequest('schema', $this->getConfig('DB_SCHEMA', '')),
+				'DB_ENCRYPTION' => (bool) getRequest('tls_encryption', $this->getConfig('DB_ENCRYPTION', false)),
+				'DB_ENCRYPTION_ADVANCED' => (bool) getRequest('verify_certificate',
+					$this->getConfig('DB_ENCRYPTION_ADVANCED', false)
+				),
+				'DB_VERIFY_HOST' => (bool) getRequest('verify_host', $this->getConfig('DB_VERIFY_HOST', false)),
+				'DB_KEY_FILE' => getRequest('key_file', $this->getConfig('DB_KEY_FILE', '')),
+				'DB_CERT_FILE' => getRequest('cert_file', $this->getConfig('DB_CERT_FILE', '')),
+				'DB_CA_FILE' => getRequest('ca_file', $this->getConfig('DB_CA_FILE', '')),
+				'DB_CIPHER_LIST' => getRequest('cipher_list', $this->getConfig('DB_CIPHER_LIST', ''))
+			];
+
+			if (!$input['DB_ENCRYPTION_ADVANCED']) {
+				$input['DB_KEY_FILE'] = '';
+				$input['DB_CERT_FILE'] = '';
+				$input['DB_CA_FILE'] = '';
+				$input['DB_CIPHER_LIST'] = '';
+			}
+			else if ($input['DB_TYPE'] === ZBX_DB_MYSQL) {
+				$input['DB_VERIFY_HOST'] = true;
+			}
+
+			if ($input['DB_TYPE'] !== ZBX_DB_POSTGRESQL) {
+				$input['DB_SCHEMA'] = '';
+			}
+
+			array_map([$this, 'setConfig'], array_keys($input), $input);
 
 			if (hasRequest('next') && array_key_exists(2, getRequest('next'))) {
 				$db_connected = $this->dbConnect();
