@@ -192,17 +192,18 @@ function inheritTagFilters($groupid, $name) {
  * Get sub-groups of selected host groups.
  *
  * @param array $groupids
- * @param array $ms_groups  [OUT] the list of groups for multiselect
+ * @param array $ms_groups  [OUT] the list of groups for multiselect.
+ * @param array $options    additional API options to select host groups.
  *
  * @return array
  */
-function getSubGroups(array $groupids, array &$ms_groups = null) {
+function getSubGroups(array $groupids, array &$ms_groups = null, array $options = []) {
 	$db_groups = $groupids
 		? API::HostGroup()->get([
 			'output' => ['groupid', 'name'],
 			'groupids' => $groupids,
 			'preservekeys' => true
-		])
+		] + $options)
 		: [];
 
 	if ($ms_groups !== null) {
@@ -216,19 +217,16 @@ function getSubGroups(array $groupids, array &$ms_groups = null) {
 	}
 
 	if ($db_groups_names) {
-		$child_groups = API::HostGroup()->get([
+		$db_groups += API::HostGroup()->get([
 			'output' => ['groupid'],
 			'search' => ['name' => $db_groups_names],
 			'searchByAny' => true,
-			'startSearch' => true
-		]);
-
-		foreach ($child_groups as $child_group) {
-			$groupids[] = $child_group['groupid'];
-		}
+			'startSearch' => true,
+			'preservekeys' => true
+		] + $options);
 	}
 
-	return $groupids;
+	return array_keys($db_groups);
 }
 
 /*
@@ -324,11 +322,10 @@ function makeProblemHostsHintBox(array $hosts, array $data, CUrl $url) {
  * @param array  $groups
  * @param string $groups[<groupid>]['groupid']
  * @param string $groups[<groupid>]['name']
- * @param array  $options                        HostGroup API call parameters.
  *
  * @return array
  */
-function enrichParentGroups(array $groups, array $options = []) {
+function enrichParentGroups(array $groups) {
 	$parents = [];
 	foreach ($groups as $group) {
 		$parent = explode('/', $group['name']);
@@ -346,18 +343,13 @@ function enrichParentGroups(array $groups, array $options = []) {
 	}
 
 	if ($parents) {
-		if (!array_key_exists('output', $options)) {
-			$options['output'] = ['groupid', 'name'];
-		}
-
-		if (!array_key_exists('filter', $options)) {
-			$options['filter'] = [];
-		}
-
-		$options['filter']['name'] = array_keys($parents);
-
-		$options['preservekeys'] = true;
-		$groups += API::HostGroup()->get($options);
+		$groups += API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
+			'filter' => [
+				'name' => array_keys($parents)
+			],
+			'preservekeys' => true
+		]);
 	}
 
 	return $groups;
