@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,24 +19,18 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 require_once dirname(__FILE__).'/js/configuration.host.list.js.php';
 
 $widget = (new CWidget())
 	->setTitle(_('Hosts'))
-	->setControls(new CList([
-		(new CForm('get'))
-			->cleanItems()
-			->setAttribute('aria-label', _('Main filter'))
-			->addItem((new CList())
-				->addItem([
-					new CLabel(_('Group'), 'groupid'),
-					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-					$data['pageFilter']->getGroupsCB()
-				])
-			),
-		(new CTag('nav', true, (new CList())
+	->setControls((new CTag('nav', true, (new CList())
 			->addItem(new CRedirectButton(_('Create host'), (new CUrl('hosts.php'))
 				->setArgument('form', 'create')
+				->setArgument('groupids', array_keys($data['filter']['groups']))
 				->getUrl()
 			))
 			->addItem(
@@ -44,22 +38,79 @@ $widget = (new CWidget())
 					->onClick('redirect("conf.import.php?rules_preset=host")')
 					->removeId()
 			)
-		))
-			->setAttribute('aria-label', _('Content controls'))
-	]));
+		))->setAttribute('aria-label', _('Content controls'))
+	);
+
+$filter_tags = $data['filter']['tags'];
+if (!$filter_tags) {
+	$filter_tags = [['tag' => '', 'value' => '', 'operator' => TAG_OPERATOR_LIKE]];
+}
+
+$filter_tags_table = (new CTable())
+	->setId('filter-tags')
+	->addRow((new CCol(
+		(new CRadioButtonList('filter_evaltype', (int) $data['filter']['evaltype']))
+			->addValue(_('And/Or'), TAG_EVAL_TYPE_AND_OR)
+			->addValue(_('Or'), TAG_EVAL_TYPE_OR)
+			->setModern(true)
+		))->setColSpan(4)
+	);
+
+$i = 0;
+foreach ($filter_tags as $tag) {
+	$filter_tags_table->addRow([
+		(new CTextBox('filter_tags['.$i.'][tag]', $tag['tag']))
+			->setAttribute('placeholder', _('tag'))
+			->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH),
+		(new CRadioButtonList('filter_tags['.$i.'][operator]', (int) $tag['operator']))
+			->addValue(_('Contains'), TAG_OPERATOR_LIKE)
+			->addValue(_('Equals'), TAG_OPERATOR_EQUAL)
+			->setModern(true),
+		(new CTextBox('filter_tags['.$i.'][value]', $tag['value']))
+			->setAttribute('placeholder', _('value'))
+			->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH),
+		(new CCol(
+			(new CButton('filter_tags['.$i.'][remove]', _('Remove')))
+				->addClass(ZBX_STYLE_BTN_LINK)
+				->addClass('element-table-remove')
+		))->addClass(ZBX_STYLE_NOWRAP)
+	], 'form_row');
+
+	$i++;
+}
+$filter_tags_table->addRow(
+	(new CCol(
+		(new CButton('filter_tags_add', _('Add')))
+			->addClass(ZBX_STYLE_BTN_LINK)
+			->addClass('element-table-add')
+	))->setColSpan(3)
+);
 
 // filter
 $filter = new CFilter(new CUrl('hosts.php'));
-
 $filter
 	->setProfile($data['profileIdx'])
 	->setActiveTab($data['active_tab'])
 	->addFilterTab(_('Filter'), [
 		(new CFormList())
-			->addRow(_('Name'),
-				(new CTextBox('filter_host', $data['filter']['host']))
-					->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-					->setAttribute('autofocus', 'autofocus')
+			->addRow(
+				(new CLabel(_('Host groups'), 'filter_groups__ms')),
+				(new CMultiSelect([
+					'name' => 'filter_groups[]',
+					'object_name' => 'hostGroup',
+					'data' => $data['filter']['groups'],
+					'popup' => [
+						'parameters' => [
+							'srctbl' => 'host_groups',
+							'srcfld1' => 'groupid',
+							'dstfrm' => $filter->getName(),
+							'dstfld1' => 'filter_groups_',
+							'real_hosts' => 1,
+							'editable' => 1,
+							'enrich_parent_groups' => 1
+						]
+					]
+				]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 			)
 			->addRow(
 				(new CLabel(_('Templates'), 'filter_templates__ms')),
@@ -78,6 +129,19 @@ $filter
 					]
 				]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 			)
+			->addRow(_('Name'),
+				(new CTextBox('filter_host', $data['filter']['host']))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
+			)
+			->addRow(_('DNS'),
+				(new CTextBox('filter_dns', $data['filter']['dns']))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
+			)
+			->addRow(_('IP'),
+				(new CTextBox('filter_ip', $data['filter']['ip']))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
+			)
+			->addRow(_('Port'),
+				(new CTextBox('filter_port', $data['filter']['port']))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
+			),
+		(new CFormList())
 			->addRow(_('Monitored by'),
 				(new CRadioButtonList('filter_monitored_by', (int) $data['filter']['monitored_by']))
 					->addValue(_('Any'), ZBX_MONITORED_BY_ANY)
@@ -102,17 +166,8 @@ $filter
 						]
 					]
 				]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-			),
-		(new CFormList())
-			->addRow(_('DNS'),
-				(new CTextBox('filter_dns', $data['filter']['dns']))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-			)
-			->addRow(_('IP'),
-				(new CTextBox('filter_ip', $data['filter']['ip']))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-			)
-			->addRow(_('Port'),
-				(new CTextBox('filter_port', $data['filter']['port']))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 			)
+			->addRow(_('Tags'), $filter_tags_table)
 	]);
 
 $widget->addItem($filter);
@@ -133,11 +188,14 @@ $table = (new CTableInfo())
 		_('Discovery'),
 		_('Web'),
 		_('Interface'),
+		($data['filter']['monitored_by'] == ZBX_MONITORED_BY_PROXY
+				|| $data['filter']['monitored_by'] == ZBX_MONITORED_BY_ANY) ? _('Proxy') : null,
 		_('Templates'),
 		make_sorting_header(_('Status'), 'status', $data['sortField'], $data['sortOrder'], 'hosts.php'),
 		_('Availability'),
 		_('Agent encryption'),
-		_('Info')
+		_('Info'),
+		_('Tags')
 	]);
 
 $current_time = time();
@@ -157,21 +215,23 @@ foreach ($data['hosts'] as $host) {
 
 	$description = [];
 
-	if ($host['proxy_hostid'] != 0) {
-		$description[] = $data['proxies'][$host['proxy_hostid']]['host'];
-		$description[] = NAME_DELIMITER;
-	}
 	if ($host['discoveryRule']) {
-		$description[] = (new CLink(
-			$host['discoveryRule']['name'], 'host_prototypes.php?parent_discoveryid='.$host['discoveryRule']['itemid']
+		$description[] = (new CLink(CHtml::encode($host['discoveryRule']['name']),
+			(new CUrl('host_prototypes.php'))->setArgument('parent_discoveryid', $host['discoveryRule']['itemid'])
 		))
 			->addClass(ZBX_STYLE_LINK_ALT)
 			->addClass(ZBX_STYLE_ORANGE);
 		$description[] = NAME_DELIMITER;
 	}
+	elseif ($host['flags'] == ZBX_FLAG_DISCOVERY_CREATED) {
+		// Discovered host which does not contain info about parent discovery rule is inaccessible for current user.
+		$description[] = (new CSpan(_('Inaccessible discovery rule')))->addClass(ZBX_STYLE_ORANGE);
+		$description[] = NAME_DELIMITER;
+	}
 
-	$description[] = new CLink(CHtml::encode($host['name']),
-		'hosts.php?form=update&hostid='.$host['hostid'].url_param('groupid')
+	$description[] = new CLink(CHtml::encode($host['name']), (new CUrl('hosts.php'))
+				->setArgument('form', 'update')
+				->setArgument('hostid', $host['hostid'])
 	);
 
 	$maintenance_icon = false;
@@ -194,26 +254,27 @@ foreach ($data['hosts'] as $host) {
 		$statusCaption = _('Enabled');
 		$statusClass = ZBX_STYLE_GREEN;
 		$confirm_message = _('Disable host?');
-		$statusUrl = 'hosts.php?hosts[]='.$host['hostid'].'&action=host.massdisable'.url_param('groupid');
+		$status_url = (new CUrl('hosts.php'))
+			->setArgument('action', 'host.massdisable')
+			->setArgument('hosts', [$host['hostid']]);
 	}
 	else {
 		$statusCaption = _('Disabled');
-		$statusUrl = 'hosts.php?hosts[]='.$host['hostid'].'&action=host.massenable'.url_param('groupid');
+		$status_url = (new CUrl('hosts.php'))
+			->setArgument('action', 'host.massenable')
+			->setArgument('hosts', [$host['hostid']]);
 		$confirm_message = _('Enable host?');
 		$statusClass = ZBX_STYLE_RED;
 	}
 
-	$status = (new CLink($statusCaption, $statusUrl))
+	$status = (new CLink($statusCaption, $status_url->getUrl()))
 		->addClass(ZBX_STYLE_LINK_ACTION)
 		->addClass($statusClass)
 		->addConfirmation($confirm_message)
 		->addSID();
 
 	if ($maintenance_icon) {
-		$status = [$maintenance_icon, $status];
-	}
-	elseif (count($data['maintenances'])) {
-		$status->addClass(ZBX_STYLE_ICON_NONE);
+		$description[] = $maintenance_icon;
 	}
 
 	order_result($host['parentTemplates'], 'name');
@@ -232,8 +293,10 @@ foreach ($data['hosts'] as $host) {
 
 		if (array_key_exists($template['templateid'], $data['writable_templates'])) {
 			$caption = [
-				(new CLink(
-					CHtml::encode($template['name']), 'templates.php?form=update&templateid='.$template['templateid']
+				(new CLink(CHtml::encode($template['name']),
+					(new CUrl('templates.php'))
+						->setArgument('form', 'update')
+						->setArgument('templateid', $template['templateid'])
 				))
 					->addClass(ZBX_STYLE_LINK_ALT)
 					->addClass(ZBX_STYLE_GREY)
@@ -254,7 +317,9 @@ foreach ($data['hosts'] as $host) {
 			foreach ($parent_templates as $parent_template) {
 				if (array_key_exists($parent_template['templateid'], $data['writable_templates'])) {
 					$caption[] = (new CLink(CHtml::encode($parent_template['name']),
-						'templates.php?form=update&templateid='.$parent_template['templateid']
+						(new CUrl('templates.php'))
+							->setArgument('form', 'update')
+							->setArgument('templateid', $parent_template['templateid'])
 					))
 						->addClass(ZBX_STYLE_LINK_ALT)
 						->addClass(ZBX_STYLE_GREY);
@@ -323,42 +388,75 @@ foreach ($data['hosts'] as $host) {
 			$out_encryption[] = (new CSpan(_('CERT')))->addClass(ZBX_STYLE_STATUS_GREY);
 		}
 
-		$encryption = (new CDiv([$in_encryption, ' ', $out_encryption]))->addClass(ZBX_STYLE_STATUS_CONTAINER);
+		$encryption = (new CDiv([new CSpan($in_encryption), ' ', new CSpan($out_encryption)]))
+			->addClass(ZBX_STYLE_STATUS_CONTAINER)
+			->addClass(ZBX_STYLE_NOWRAP);
 	}
 
 	$table->addRow([
 		new CCheckBox('hosts['.$host['hostid'].']', $host['hostid']),
 		(new CCol($description))->addClass(ZBX_STYLE_NOWRAP),
 		[
-			new CLink(_('Applications'), 'applications.php?groupid='.$data['groupId'].'&hostid='.$host['hostid']),
+			new CLink(_('Applications'),
+				(new CUrl('applications.php'))
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$host['hostid']])
+			),
 			CViewHelper::showNum($host['applications'])
 		],
 		[
-			new CLink(_('Items'), 'items.php?filter_set=1&hostid='.$host['hostid']),
+			new CLink(_('Items'),
+				(new CUrl('items.php'))
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$host['hostid']])
+			),
 			CViewHelper::showNum($host['items'])
 		],
 		[
-			new CLink(_('Triggers'), 'triggers.php?groupid='.$data['groupId'].'&hostid='.$host['hostid']),
+			new CLink(_('Triggers'),
+				(new CUrl('triggers.php'))
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$host['hostid']])
+			),
 			CViewHelper::showNum($host['triggers'])
 		],
 		[
-			new CLink(_('Graphs'), 'graphs.php?groupid='.$data['groupId'].'&hostid='.$host['hostid']),
+			new CLink(_('Graphs'),
+				(new CUrl('graphs.php'))
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$host['hostid']])
+			),
 			CViewHelper::showNum($host['graphs'])
 		],
 		[
-			new CLink(_('Discovery'), 'host_discovery.php?hostid='.$host['hostid']),
+			new CLink(_('Discovery'),
+				(new CUrl('host_discovery.php'))
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$host['hostid']])
+			),
 			CViewHelper::showNum($host['discoveries'])
 		],
 		[
-			new CLink(_('Web'), 'httpconf.php?&hostid='.$host['hostid']),
+			new CLink(_('Web'),
+				(new CUrl('httpconf.php'))
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$host['hostid']])
+			),
 			CViewHelper::showNum($host['httpTests'])
 		],
 		$hostInterface,
+		($data['filter']['monitored_by'] == ZBX_MONITORED_BY_PROXY
+				|| $data['filter']['monitored_by'] == ZBX_MONITORED_BY_ANY)
+			? ($host['proxy_hostid'] != 0)
+				? $data['proxies'][$host['proxy_hostid']]['host']
+				: ''
+			: null,
 		$hostTemplates,
 		$status,
 		getHostAvailabilityTable($host),
 		$encryption,
-		makeInformationList($info_icons)
+		makeInformationList($info_icons),
+		$data['tags'][$host['hostid']]
 	]);
 }
 
@@ -373,8 +471,7 @@ $form->addItem([
 				(new CUrl('zabbix.php'))
 					->setArgument('action', 'export.hosts.xml')
 					->setArgument('backurl', (new CUrl('hosts.php'))
-						->setArgument('groupid', $data['pageFilter']->groupid)
-						->setArgument('page', getPageNumber())
+						->setArgument('page', $data['page'] == 1 ? null : $data['page'])
 						->getUrl())
 					->getUrl()
 			],
@@ -386,4 +483,4 @@ $form->addItem([
 
 $widget->addItem($form);
 
-return $widget;
+$widget->show();
