@@ -23,6 +23,11 @@
  * @var CView $this
  */
 
+$interface_ids_by_types = [];
+foreach ($data['interfaces'] as $interface) {
+	$interface_ids_by_types[$interface['type']][] = $interface['interfaceid'];
+}
+
 include dirname(__FILE__).'/common.item.edit.js.php';
 include dirname(__FILE__).'/item.preprocessing.js.php';
 include dirname(__FILE__).'/editabletable.js.php';
