@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,6 +18,11 @@
 **/
 
 
+// Global constants.
+
+// Sync with SASS variable: $ui-transition-duration.
+const UI_TRANSITION_DURATION = 300;
+
 // Array indexOf method for javascript<1.6 compatibility
 if (!Array.prototype.indexOf) {
 	Array.prototype.indexOf = function (searchElement) {
@@ -71,9 +76,10 @@ var PageRefresh = {
 			return false;
 		}
 
-		this.delayLeft -= 1000;
-		if (this.delayLeft < 0) {
-			if (IE || ED) {
+		this.delayLeft = Math.max(-1, this.delayLeft - 1000);
+
+		if (this.delayLeft < 0 && !overlays_stack.length) {
+			if (ED) {
 				sessionStorage.scrollTop = jQuery(window).scrollTop();
 			}
 
@@ -103,94 +109,7 @@ var PageRefresh = {
 };
 
 /*
- * Main menu
- */
-var MMenu = {
-	menus:			{'view': 0, 'cm': 0, 'reports': 0, 'config': 0, 'admin': 0},
-	def_label:		null,
-	sub_active: 	false,
-	timeout_reset:	null,
-	timeout_change:	null,
-
-	init: function() {
-		// Detects when none of the selected elements are focused.
-		var elems = jQuery('.top-nav a, .top-subnav a').on('keydown', function(event) {
-			clearTimeout(this.timeout_reset);
-
-			if (event.which == 9) {
-				setTimeout(function() {
-					if (elems.toArray().indexOf(document.querySelector(':focus')) == -1) {
-						clearTimeout(this.timeout_reset);
-						this.timeout_reset = setTimeout(function() {
-							if (elems.toArray().indexOf(document.querySelector(':focus')) == -1){
-								MMenu.showSubMenu(MMenu.def_label)
-							}
-						}, 2500);
-					}
-				});
-			}
-		});
-	},
-
-	mouseOver: function(show_label) {
-		clearTimeout(this.timeout_reset);
-		this.timeout_change = setTimeout('MMenu.showSubMenu("' + show_label + '", true)', 10);
-		PageRefresh.restart();
-	},
-
-	keyUp: function(show_label, event) {
-		if (event.which == 13) {
-			clearTimeout(this.timeout_reset);
-			this.timeout_change = setTimeout('MMenu.showSubMenu("' + show_label + '", true)', 10);
-			PageRefresh.restart();
-		}
-	},
-
-	submenu_mouseOver: function() {
-		clearTimeout(this.timeout_reset);
-		clearTimeout(this.timeout_change);
-		PageRefresh.restart();
-	},
-
-	mouseOut: function() {
-		clearTimeout(this.timeout_change);
-		this.timeout_reset = setTimeout('MMenu.showSubMenu("' + this.def_label + '")', 2500);
-	},
-
-	showSubMenu: function(show_label, focus_subitem) {
-		var sub_menu = $('sub_' + show_label),
-			focus_subitem = focus_subitem || false;
-
-		if (sub_menu !== null) {
-			$(show_label).className = 'selected';
-			sub_menu.show();
-
-			if (focus_subitem) {
-				jQuery('li:first > a', sub_menu).focus();
-			}
-
-			for (var key in this.menus) {
-				if (key == show_label) {
-					continue;
-				}
-
-				var menu_cell = $(key);
-				if (menu_cell !== null) {
-					menu_cell.className = '';
-					jQuery('a', menu_cell).blur();
-				}
-
-				var sub_menu_cell = $('sub_' + key);
-				if (sub_menu_cell !== null) {
-					sub_menu_cell.hide();
-				}
-			}
-		}
-	}
-};
-
-/*
- * Audio control system
+ * Audio control system.
  */
 var AudioControl = {
 
@@ -213,36 +132,26 @@ var AudioControl = {
 	playOnce: function(name) {
 		this.stop();
 
-		if (IE) {
-			this.create(name, false);
+		var obj = jQuery('#audio');
+
+		if (obj.length > 0 && obj.data('name') === name) {
+			obj.trigger('play');
 		}
 		else {
-			var obj = jQuery('#audio');
-
-			if (obj.length > 0 && obj.data('name') === name) {
-				obj.trigger('play');
-			}
-			else {
-				this.create(name, false);
-			}
+			this.create(name, false);
 		}
 	},
 
 	playLoop: function(name, delay) {
 		this.stop();
 
-		if (IE) {
-			this.create(name, true);
+		var obj = jQuery('#audio');
+
+		if (obj.length > 0 && obj.data('name') === name) {
+			obj.trigger('play');
 		}
 		else {
-			var obj = jQuery('#audio');
-
-			if (obj.length > 0 && obj.data('name') === name) {
-				obj.trigger('play');
-			}
-			else {
-				this.create(name, true);
-			}
+			this.create(name, true);
 		}
 
 		AudioControl.loop(new Date().getTime() + delay * 1000);
@@ -254,67 +163,29 @@ var AudioControl = {
 		if (obj !== null) {
 			clearTimeout(AudioControl.timeoutHandler);
 
-			if (IE) {
-				obj.setAttribute('loop', false);
-				obj.setAttribute('playcount', 0);
-
-				try {
-					obj.stop();
-				}
-				catch (e) {
-					setTimeout(
-						function() {
-							try {
-								document.getElementById('audio').stop();
-							}
-							catch (e) {
-							}
-						},
-						100
-					);
-				}
-			}
-			else {
-				jQuery(obj).trigger('pause');
-			}
+			jQuery(obj).trigger('pause');
 		}
 	},
 
 	create: function(name, loop) {
-		if (IE) {
-			jQuery('#audio').remove();
+		var obj = jQuery('#audio');
 
-			jQuery('body').append(jQuery('<embed>', {
+		if (obj.length == 0 || obj.data('name') !== name) {
+			obj.remove();
+
+			var audioOptions = {
 				id: 'audio',
 				'data-name': name,
 				src: 'audio/' + name,
-				enablejavascript: true,
-				autostart: true,
-				loop: true,
-				playcount: loop ? 9999999 : 1,
-				height: 0
-			}));
-		}
-		else {
-			var obj = jQuery('#audio');
+				preload: 'auto',
+				autoplay: true
+			};
 
-			if (obj.length == 0 || obj.data('name') !== name) {
-				obj.remove();
-
-				var audioOptions = {
-					id: 'audio',
-					'data-name': name,
-					src: 'audio/' + name,
-					preload: 'auto',
-					autoplay: true
-				};
-
-				if (loop) {
-					audioOptions.loop = true;
-				}
-
-				jQuery('body').append(jQuery('<audio>', audioOptions));
+			if (loop) {
+				audioOptions.loop = true;
 			}
+
+			jQuery('body').append(jQuery('<audio>', audioOptions));
 		}
 	}
 };
@@ -439,7 +310,7 @@ var hintBox = {
 	createBox: function(e, target, hintText, className, isStatic, styles, appendTo) {
 		var hintboxid = hintBox.getUniqueId(),
 			box = jQuery('<div></div>', {'data-hintboxid': hintboxid}).addClass('overlay-dialogue'),
-			appendTo = appendTo || 'body';
+			appendTo = appendTo || '.wrapper';
 
 		if (styles) {
 			// property1: value1; property2: value2; property(n): value(n)
@@ -460,7 +331,7 @@ var hintBox = {
 		}
 
 		if (!empty(className)) {
-			box.append(jQuery('<span></span>').addClass(className).html(hintText));
+			box.append(jQuery('<div></div>').addClass(className).html(hintText));
 		}
 		else {
 			box.html(hintText);
@@ -526,79 +397,71 @@ var hintBox = {
 		target.hintBoxItem.show();
 
 		if (target.isStatic) {
-			overlayDialogueOnLoad(true, target.hintBoxItem);
+			Overlay.prototype.recoverFocus.call({'$dialogue': target.hintBoxItem});
+			Overlay.prototype.containFocus.call({'$dialogue': target.hintBoxItem});
 		}
 	},
 
 	positionHint: function(e, target) {
-		var wWidth = jQuery(window).width(),
-			wHeight = jQuery(window).height(),
-			scrollTop = jQuery(window).scrollTop(),
-			scrollLeft = jQuery(window).scrollLeft(),
-			hint_width = jQuery(target.hintBoxItem).outerWidth(),
-			hint_height = jQuery(target.hintBoxItem).outerHeight(),
-			top, left;
-
-		// uses stored clientX on afterload positioning when there is no event
 		if (e.clientX) {
 			target.clientX = e.clientX;
 			target.clientY = e.clientY;
 		}
 
-		// doesn't fit in the screen horizontally
-		if (hint_width + 10 > wWidth) {
-			left = scrollLeft + 2;
+		var $host = target.hintBoxItem.offsetParent(),
+			host_offset = $host.offset(),
+			// Usable area relative to host.
+			host_x_min = $host.scrollLeft(),
+			host_x_max = Math.min($host[0].scrollWidth,
+				$(window).width() + $(window).scrollLeft() - host_offset.left + $host.scrollLeft()
+			) - 1,
+			host_y_min = $host.scrollTop(),
+			host_y_max = Math.min($host[0].scrollHeight,
+				$(window).height() + $(window).scrollTop() - host_offset.top + $host.scrollTop()
+			) - 1,
+			// Event coordinates relative to host.
+			event_x = target.clientX - host_offset.left + $host.scrollLeft(),
+			event_y = target.clientY - host_offset.top + $host.scrollTop(),
+			event_offset = 10,
+			// Hint box width and height.
+			hint_width = target.hintBoxItem.outerWidth(),
+			hint_height = target.hintBoxItem.outerHeight(),
+			/*
+				Fix popup width and height since browsers will tend to reduce the size of the popup, if positioned further
+				than the width of window when horizontal scrolling is active.
+			*/
+			css = {
+				width: target.hintBoxItem.width(),
+				height: target.hintBoxItem.height()
+			};
+
+		if (event_x + event_offset + hint_width <= host_x_max) {
+			css.left = event_x + event_offset;
 		}
-		// 10px to right if fit
-		else if (wWidth - target.clientX - 10 > hint_width) {
-			left = scrollLeft + target.clientX + 10;
-		}
-		// 10px from screen right side
 		else {
-			left = scrollLeft + wWidth - 10 - hint_width;
+			css.right = -$host.scrollLeft();
 		}
 
-		// 10px below if fit
-		if (wHeight - target.clientY - hint_height - 10 > 0) {
-			top = scrollTop + target.clientY + 10;
+		if (event_y + event_offset + hint_height <= host_y_max) {
+			css.top = event_y + event_offset;
 		}
-		// 10px above if fit
-		else if (target.clientY - hint_height - 10 > 0) {
-			top = scrollTop + target.clientY - hint_height - 10;
+		else if (event_y - event_offset - hint_height >= host_y_min) {
+			css.top = event_y - event_offset - hint_height;
 		}
-		// 10px below as fallback
 		else {
-			top = scrollTop + target.clientY + 10;
-		}
+			css.top = Math.max(host_y_min, Math.min(host_y_max - hint_height, event_y + event_offset));
 
-		// fallback if doesn't fit verticaly but could fit if aligned to right or left
-		if ((top - scrollTop + hint_height > wHeight)
-				&& (target.clientX - 10 > hint_width || wWidth - target.clientX - 10 > hint_width)) {
+			if (css.right !== undefined) {
+				delete css.right;
 
-			// align to left if fit
-			if (wWidth - target.clientX - 10 > hint_width) {
-				left = scrollLeft + target.clientX + 10;
-			}
-			// align to right
-			else {
-				left = scrollLeft + target.clientX - hint_width - 10;
-			}
-
-			// 10px from bottom if fit
-			if (wHeight - 10 > hint_height) {
-				top = scrollTop + wHeight - hint_height - 10;
-			}
-			// 10px from top
-			else {
-				top = scrollTop + 10;
+				css.left = ((event_x - event_offset - hint_width >= host_x_min)
+					? event_x - event_offset - hint_width
+					: event_x + event_offset
+				);
 			}
 		}
 
-		target.hintBoxItem.css({
-			top: top + 'px',
-			left: left + 'px',
-			zIndex: 100
-		});
+		target.hintBoxItem.css(css);
 	},
 
 	hideHint: function(target, hideStatic) {
@@ -680,7 +543,7 @@ function updateUserProfile(idx, value_int, idx2) {
 	});
 }
 
-function changeWidgetState(obj, widgetId, url) {
+function changeWidgetState(obj, widgetId, idx) {
 	var widgetObj = jQuery('#' + widgetId + '_widget'),
 		css = switchElementClass(obj, 'btn-widget-collapse', 'btn-widget-expand'),
 		state = 0;
@@ -697,13 +560,9 @@ function changeWidgetState(obj, widgetId, url) {
 	}
 
 	obj.title = (state == 1) ? t('S_COLLAPSE') : t('S_EXPAND');
-
-	sendAjaxData(url, {
-		data: {
-			widget: widgetId,
-			state: state
-		}
-	});
+	if (idx !== '' && typeof idx !== 'undefined') {
+		updateUserProfile(idx, state, []);
+	}
 }
 
 /**
@@ -841,12 +700,13 @@ function getConditionFormula(conditions, evalType) {
 	 * Creates a table with dynamic add/remove row buttons.
 	 *
 	 * Supported options:
-	 * - template		- row template selector
-	 * - row			- element row selector
-	 * - add			- add row button selector
-	 * - remove			- remove row button selector
-	 * - counter 		- number to start row enumeration from
-	 * - dataCallback	- function to generate the data passed to the template
+	 * - template				- row template selector
+	 * - row					- element row selector
+	 * - add					- add row button selector
+	 * - remove					- remove row button selector
+	 * - counter 				- number to start row enumeration from
+	 * - dataCallback			- function to generate the data passed to the template
+	 * - remove_next_sibling	- remove also next element
 	 *
 	 * Triggered events:
 	 * - tableupdate.dynamicRows 	- after adding or removing a row.
@@ -862,6 +722,8 @@ function getConditionFormula(conditions, evalType) {
 			row: '.form_row',
 			add: '.element-table-add',
 			remove: '.element-table-remove',
+			remove_next_sibling: false,
+			disable: '.element-table-disable',
 			counter: null,
 			beforeRow: null,
 			dataCallback: function(data) {
@@ -872,6 +734,7 @@ function getConditionFormula(conditions, evalType) {
 		return this.each(function() {
 			var table = $(this);
 
+			// If options.remove_next_sibling is true, counter counts each row making the next index twice as large (bug).
 			table.data('dynamicRows', {
 				counter: (options.counter !== null) ? options.counter : $(options.row, table).length
 			});
@@ -894,6 +757,12 @@ function getConditionFormula(conditions, evalType) {
 				// remove the parent row
 				removeRow(table, $(this).closest(options.row), options);
 			});
+
+			// disable buttons
+			table.on('click', options.disable, function() {
+				// disable the parent row
+				disableRow($(this).closest(options.row));
+			});
 		});
 	};
 
@@ -925,11 +794,24 @@ function getConditionFormula(conditions, evalType) {
 	 * @param {object} options
 	 */
 	function removeRow(table, row, options) {
+		if (options.remove_next_sibling) {
+			row.next().remove();
+		}
 		row.remove();
 
 		table.trigger('tableupdate.dynamicRows', options);
 		table.trigger('afterremove.dynamicRows', options);
 	}
+
+	/**
+	 * Disables the given row.
+	 *
+	 * @param {jQuery} row
+	 */
+	function disableRow(row) {
+		row.find('input').prop('readonly', true);
+		row.find('button').prop('disabled', true);
+	}
 }(jQuery));
 
 jQuery(function ($) {
@@ -967,15 +849,10 @@ jQuery(function ($) {
 
 				var css = {};
 
-				if (IE9) {
-					css['-ms-transform-origin'] = transform;
-				}
-				else {
-					css['transform-origin'] = transform;
-					css['-webkit-transform-origin'] = transform;
-					css['-moz-transform-origin'] = transform;
-					css['-o-transform-origin'] = transform;
-				}
+				css['transform-origin'] = transform;
+				css['-webkit-transform-origin'] = transform;
+				css['-moz-transform-origin'] = transform;
+				css['-o-transform-origin'] = transform;
 
 				var divInner = $('<div>', {
 					'class': 'vertical_rotation_inner'
@@ -1004,86 +881,27 @@ jQuery(function ($) {
 		});
 	};
 
-	if ((IE || ED) && typeof sessionStorage.scrollTop !== 'undefined') {
+	if (ED && typeof sessionStorage.scrollTop !== 'undefined') {
 		$(window).scrollTop(sessionStorage.scrollTop);
+		sessionStorage.removeItem('scrollTop');
 	}
 });
 
+window.addEventListener('load', e => {
 
-jQuery(function ($) {
-	"use strict";
+	/**
+	 * SideBar initialization.
+	 */
+	const sidebar = document.querySelector('.sidebar');
 
-	function calcRows($obj, options) {
-		var max_height = (options && 'maxHeight' in options) ? options.maxHeight : null,
-			clone = $obj
-				.clone(false)
-				.css({'position': 'absolute', 'z-index': '-1'})
-				.removeClass('patternselect')
-				.insertAfter($obj),
-			padding = parseInt(clone.css('padding-top'), 10) + parseInt(clone.css('padding-bottom'), 10),
-			line_height = parseInt(clone.css('font-size'), 10) * 1.14,
-			rows = 0;
+	if (sidebar !== null) {
+		ZABBIX.MenuMain = new CMenu(document.querySelector('.menu-main'));
+		ZABBIX.UserMain = new CMenu(document.querySelector('.menu-user'));
 
-		do {
-			rows++;
-			clone.height(rows * line_height);
-		}
-		while (clone[0].scrollHeight - padding > clone.innerHeight()
-			&& (max_height === null || rows * line_height + padding <= max_height));
-		clone.remove();
-
-		return rows;
-	}
-
-	var methods = {
-		init: function(options) {
-			options = $.extend({}, options);
-
-			this.each(function() {
-				if (typeof $(this).data('autogrow') === 'undefined') {
-					$(this)
-						.css({
-							'resize': 'none',
-							'overflow-x': 'hidden',
-							'white-space': 'pre-line'
-						})
-						.on('paste change keyup', function() {
-							var rows = calcRows($(this), options);
-
-							if (options && 'pair' in options) {
-								var pair_rows = calcRows($(options.pair), options);
-								if (pair_rows > rows) {
-									rows = pair_rows;
-								}
-								$(options.pair).attr('rows', rows);
-							}
-
-							$(this).attr('rows', rows);
-						})
-						.data('autogrow', options)
-						.trigger('keyup');
-				}
-
-				if ($(this).prop('maxlength') !== 'undefined' && !CR && !GK) {
-					$(this).bind('paste contextmenu change keydown keypress keyup', function() {
-						if ($(this).val().length > $(this).attr('maxlength')) {
-							$(this).val($(this).val().substr(0, $(this).attr('maxlength')));
-						}
-					});
-				}
-
-				if (options && 'pair' in options) {
-					$(options.pair).css({'resize': 'none'});
-				}
+		ZABBIX.Sidebar = new CSidebar(sidebar)
+			.on('viewmodechange', (e) => {
+				updateUserProfile('web.sidebar.mode', e.detail.view_mode, []);
+				window.dispatchEvent(new Event('resize'));
 			});
-		}
-	};
-
-	$.fn.autoGrowTextarea = function(method, options) {
-		if (methods[method]) {
-			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
-		}
-
-		return methods.init.apply(this, arguments);
-	};
+	}
 });
