@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 require_once dirname(__FILE__).'/include/gettextwrapper.inc.php';
 require_once dirname(__FILE__).'/include/js.inc.php';
 require_once dirname(__FILE__).'/include/locales.inc.php';
+require_once dirname(__FILE__).'/include/translateDefines.inc.php';
 
 // if we must provide language constants on language different from English
 if (isset($_GET['lang'])) {
@@ -45,8 +46,6 @@ if (isset($_GET['lang'])) {
 	setlocale(LC_NUMERIC, ['C', 'POSIX', 'en', 'en_US', 'en_US.UTF-8', 'English_United States.1252', 'en_GB', 'en_GB.UTF-8']);
 }
 
-require_once dirname(__FILE__).'/include/translateDefines.inc.php';
-
 // available scripts 'scriptFileName' => 'path relative to js/'
 $availableJScripts = [
 	'common.js' => '',
@@ -58,23 +57,35 @@ $availableJScripts = [
 	'dom.js' => '',
 	'servercheck.js' => '',
 	'flickerfreescreen.js' => '',
+	'multilineinput.js' => '',
 	'multiselect.js' => '',
 	'colorpicker.js' => '',
 	'chkbxrange.js' => '',
 	'csvggraphwidget.js' => '',
 	'layout.mode.js' => '',
+	'textareaflexible.js' => '',
+	'inputsecret.js' => '',
+	'macrovalue.js' => '',
 	// vendors
-	'prototype.js' => 'vendors/',
 	'jquery.js' => 'vendors/',
 	'jquery-ui.js' => 'vendors/',
 	// classes
+	'component.z-select.js' => '',
+	'class.base-component.js' => '',
 	'class.bbcode.js' => '',
 	'class.calendar.js' => '',
 	'class.cclock.js' => '',
 	'class.cdate.js' => '',
 	'class.cdebug.js' => '',
 	'class.cmap.js' => '',
-	'class.cmessages.js' => '',
+	'class.localstorage.js' => '',
+	'class.menu.js' => '',
+	'class.menu-item.js' => '',
+	'class.notifications.js' => '',
+	'class.notification.js' => '',
+	'class.notification.collection.js' => '',
+	'class.notifications.audio.js' => '',
+	'class.browsertab.js' => '',
 	'class.cnavtree.js' => '',
 	'class.cookie.js' => '',
 	'class.coverride.js' => '',
@@ -84,27 +95,41 @@ $availableJScripts = [
 	'class.csvggraph.js' => '',
 	'class.ctree.js' => '',
 	'class.curl.js' => '',
+	'class.overlaycollection.js' => '',
+	'class.overlay.js' => '',
 	'class.cverticalaccordion.js' => '',
 	'class.mapWidget.js' => '',
+	'class.scrollable.js' => '',
+	'class.sidebar.js' => '',
 	'class.svg.canvas.js' => 'vector/',
 	'class.svg.map.js' => 'vector/',
 	'class.cviewswitcher.js' => '',
 	'class.pmaster.js' => '',
 	'class.rpc.js' => '',
+	'class.template.js' => '',
 	'init.js' => '',
 	// templates
 	'sysmap.tpl.js' => 'templates/',
 	// page-specific scripts
 	'items.js' => 'pages/',
+	'setup.js' => 'pages/',
+	'popup.condition.common.js' => 'pages/',
+	'popup.operation.common.js' => 'pages/'
 ];
 
 $tranStrings = [
 	'gtlc.js' => [
 		'S_MINUTE_SHORT' => _x('m', 'minute short')
 	],
+	'class.overlay.js' => [
+		'Operation details' => _('Operation details')
+	],
 	'dashboard.grid.js' => [
 		'Edit widget' => _('Edit widget'),
 		'Add widget' => _('Add widget'),
+		'Paste widget' => _('Paste widget'),
+		'S_COPY' => _('Copy'),
+		'S_PASTE' => _('Paste'),
 		'Apply' => _('Apply'),
 		'Add' => _('Add'),
 		'Edit' => _('Edit'),
@@ -113,10 +138,16 @@ $tranStrings = [
 		'You have unsaved changes.' => _('You have unsaved changes.'),
 		'Are you sure, you want to leave this page?' => _('Are you sure, you want to leave this page?'),
 		'Cannot add widgets in kiosk mode' => _('Cannot add widgets in kiosk mode'),
+		'You do not have permissions to edit dashboard' => _('You do not have permissions to edit dashboard'),
 		'Add a new widget' => _('Add a new widget'),
-		'Release to create a new widget.' => _('Release to create a new widget.'),
+		'Release to create a widget.' => _('Release to create a widget.'),
 		'Click and drag to desired size.' => _('Click and drag to desired size.'),
-		'Adjust widget refresh interval' => _('Adjust widget refresh interval')
+		'Previous page' => _('Previous page'),
+		'Next page' => _('Next page'),
+		'Widget is too small for the specified number of columns and rows.' =>
+			_('Widget is too small for the specified number of columns and rows.'),
+		'Cannot add widget: not enough free space on the dashboard.' =>
+			_('Cannot add widget: not enough free space on the dashboard.')
 	],
 	'functions.js' => [
 		'Cancel' => _('Cancel'),
@@ -128,7 +159,10 @@ $tranStrings = [
 		'S_DAY_SHORT' => _x('d', 'day short'),
 		'S_HOUR_SHORT' => _x('h', 'hour short'),
 		'S_MINUTE_SHORT' => _x('m', 'minute short'),
-		'Do you wish to replace the conditional expression?' => _('Do you wish to replace the conditional expression?')
+		'Do you wish to replace the conditional expression?' => _('Do you wish to replace the conditional expression?'),
+		'Success message' => _('Success message'),
+		'Error message' => _('Error message'),
+		'Warning message' => _('Warning message')
 	],
 	'class.calendar.js' => [
 		'S_CALENDAR' => _('Calendar'),
@@ -190,10 +224,13 @@ $tranStrings = [
 		'S_NO_IMAGES' => 'You need to have at least one image uploaded to create map element. Images can be uploaded in Administration->General->Images section.',
 		'S_COLOR_IS_NOT_CORRECT' => _('Colour "%1$s" is not correct: expecting hexadecimal colour code (6 symbols).')
 	],
-	'class.cmessages.js' => [
+	'class.notifications.js' => [
+		'S_PROBLEM_ON' => _('Problem on'),
+		'S_RESOLVED' => _('Resolved'),
 		'S_MUTE' => _('Mute'),
+		'S_CANNOT_SUPPORT_NOTIFICATION_AUDIO' => _('Cannot support notification audio for this device.'),
 		'S_UNMUTE' => _('Unmute'),
-		'S_CLEAR' => _('Clear'),
+		'S_CLOSE' => _('Close'),
 		'S_SNOOZE' => _('Snooze')
 	],
 	'class.cookie.js' => [
@@ -212,6 +249,12 @@ $tranStrings = [
 		'S_COLLAPSE' => _('Collapse'),
 		'S_CLOSE' => _('Close')
 	],
+	'multilineinput.js' => [
+		'S_N_SYMBOLS_REMAINING' => _('%1$s symbols remaining'),
+		'S_CLICK_TO_VIEW_OR_EDIT' => _('Click to view or edit'),
+		'S_APPLY' => _('Apply'),
+		'S_CANCEL' => _('Cancel')
+	],
 	'multiselect.js' => [
 		'No matches found' => _('No matches found'),
 		'More matches found...' => _('More matches found...'),
@@ -236,9 +279,9 @@ $tranStrings = [
 		'Create new' => _('Create new'),
 		'Create trigger' => _('Create trigger'),
 		'Create dependent item' => _('Create dependent item'),
+		'Create dependent discovery rule' => _('Create dependent discovery rule'),
 		'Delete' => _('Delete'),
 		'Delete dashboard?' => _('Delete dashboard?'),
-		'Description' => _('Description'),
 		'Do you wish to replace the conditional expression?' => _('Do you wish to replace the conditional expression?'),
 		'Edit trigger' => _('Edit trigger'),
 		'Insert expression' => _('Insert expression'),
@@ -248,8 +291,9 @@ $tranStrings = [
 		'Go to' => _('Go to'),
 		'Graphs' => _('Graphs'),
 		'History' => _('History'),
-		'Host inventory' => _('Host inventory'),
-		'Host screens' => _('Host screens'),
+		'Host' => _('Host'),
+		'Inventory' => _('Inventory'),
+		'Screens' => _('Screens'),
 		'Latest data' => _('Latest data'),
 		'Latest values' => _('Latest values'),
 		'Last hour graph' => _('Last hour graph'),
@@ -263,6 +307,7 @@ $tranStrings = [
 		'S_TRIGGER' => _('Trigger'),
 		'URL' => _('URL'),
 		'URLs' => _('URLs'),
+		'Web' => _('Web'),
 		'No refresh' => _('No refresh'),
 		'10 seconds' => _n('%1$s second', '%1$s seconds', 10),
 		'30 seconds' => _n('%1$s second', '%1$s seconds', 30),
@@ -270,11 +315,17 @@ $tranStrings = [
 		'2 minutes' => _n('%1$s minute', '%1$s minutes', 2),
 		'10 minutes' => _n('%1$s minute', '%1$s minutes', 10),
 		'15 minutes' => _n('%1$s minute', '%1$s minutes', 15),
-		'S_SELECTED_SR' => _x('%1$s, selected', 'screen reader')
+		'S_SELECTED_SR' => _x('%1$s, selected', 'screen reader'),
+		'Download image' => _('Download image')
+	],
+	'init.js' => [
+		'Debug' => _('Debug'),
+		'Hide debug' => _('Hide debug')
 	],
 	'items.js' => [
 		'To set a host interface select a single item type for all items' => _('To set a host interface select a single item type for all items'),
-		'No interface found' => _('No interface found')
+		'No interface found' => _('No interface found'),
+		'Item type does not use interface' => _('Item type does not use interface')
 	],
 	'class.cnavtree.js' => [
 		'Edit' => _('Edit'),
@@ -296,22 +347,38 @@ $tranStrings = [
 	],
 	'common.js' => [
 		'Cancel' => _('Cancel')
+	],
+	'component.z-select.js' => [
+		'All' => _('All')
+	],
+	'macrovalue.js' => [
+		'Set new value' => _('Set new value')
 	]
 ];
 
+$js = '';
 if (empty($_GET['files'])) {
+
 	$files = [
-		'prototype.js',
 		'jquery.js',
 		'jquery-ui.js',
 		'common.js',
+		'component.z-select.js',
+		'class.base-component.js',
 		'class.cdebug.js',
+		'class.overlaycollection.js',
+		'class.overlay.js',
 		'class.cdate.js',
 		'class.cookie.js',
 		'class.curl.js',
+		'class.menu.js',
+		'class.menu-item.js',
 		'class.rpc.js',
 		'class.bbcode.js',
 		'class.csuggest.js',
+		'class.scrollable.js',
+		'class.sidebar.js',
+		'class.template.js',
 		'main.js',
 		'chkbxrange.js',
 		'functions.js',
@@ -319,20 +386,31 @@ if (empty($_GET['files'])) {
 		'init.js'
 	];
 
+	require_once dirname(__FILE__).'/include/defines.inc.php';
+
+	if (array_key_exists(ZBX_SESSION_NAME, $_COOKIE)) {
+		$js .= 'window.ZBX_SESSION_NAME = "'.crc32($_COOKIE[ZBX_SESSION_NAME]).'";';
+		$files[] = 'class.localstorage.js';
+	}
+
 	// load frontend messaging only for some pages
 	if (isset($_GET['showGuiMessaging']) && $_GET['showGuiMessaging']) {
-		$files[] = 'class.cmessages.js';
+		$files[] = 'class.browsertab.js';
+		$files[] = 'class.notification.collection.js';
+		$files[] = 'class.notifications.audio.js';
+		$files[] = 'class.notification.js';
+		$files[] = 'class.notifications.js';
 	}
 }
 else {
 	$files = $_GET['files'];
 }
 
-$js = 'if (typeof(locale) == "undefined") { var locale = {}; }'."\n";
+$js .= 'if (typeof(locale) === "undefined") { var locale = {}; }'."\n";
 foreach ($files as $file) {
 	if (isset($tranStrings[$file])) {
 		foreach ($tranStrings[$file] as $origStr => $str) {
-			$js .= "locale['".$origStr."'] = ".zbx_jsvalue($str).";";
+			$js .= 'locale[\'' . $origStr . '\'] = ' . zbx_jsvalue($str) . ';';
 		}
 	}
 }
@@ -343,6 +421,7 @@ foreach ($files as $file) {
 	}
 }
 
+
 $etag = md5($js);
 /**
  * strpos function allow to check ETag value to fix cases when web server compression is used:
@@ -358,20 +437,6 @@ if (array_key_exists('HTTP_IF_NONE_MATCH', $_SERVER) && strpos($_SERVER['HTTP_IF
 	exit;
 }
 
-if (in_array('prototype.js', $files)) {
-	// This takes care of the Array toJSON incompatibility with JSON.stringify.
-	$js .=
-		'var _json_stringify = JSON.stringify;'.
-		'JSON.stringify = function(value) {'.
-			'var _array_tojson = Array.prototype.toJSON,'.
-				'ret;'.
-			'delete Array.prototype.toJSON;'.
-			'ret = _json_stringify(value);'.
-			'Array.prototype.toJSON = _array_tojson;'.
-			'return ret;'.
-		'};';
-}
-
 header('Content-Type: application/javascript; charset=UTF-8');
 // breaks if "zlib.output_compression = On"
 // header('Content-length: '.$jsLength);
