@@ -27,10 +27,8 @@ require_once dirname(__FILE__).'/include/items.inc.php';
 $page['title'] = _('Overview');
 $page['file'] = 'overview.php';
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
-$page['scripts'] = ['layout.mode.js'];
-
-CView::$has_web_layout_mode = true;
-$page['web_layout_mode'] = CView::getLayoutMode();
+$page['scripts'] = ['layout.mode.js', 'multiselect.js'];
+$page['web_layout_mode'] = CViewHelper::loadLayoutMode();
 
 define('ZBX_PAGE_DO_REFRESH', 1);
 define('SHOW_TRIGGERS', 0);
@@ -40,12 +38,13 @@ require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'groupid'     => [T_ZBX_INT, O_OPT, P_SYS, DB_ID,							null],
 	'view_style'  => [T_ZBX_INT, O_OPT, P_SYS, IN(STYLE_LEFT.','.STYLE_TOP),	null],
 	'type'        => [T_ZBX_INT, O_OPT, P_SYS, IN(SHOW_TRIGGERS.','.SHOW_DATA), null],
 	// filter
 	'filter_rst' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'filter_set' =>			[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_groupids' =>	[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
+	'filter_hostids' =>		[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
 	'show_triggers' =>		[T_ZBX_INT, O_OPT, null,	null,		null],
 	'ack_status' =>			[T_ZBX_INT, O_OPT, P_SYS,	null,		null],
 	'show_severity' =>		[T_ZBX_INT, O_OPT, P_SYS,	null,		null],
@@ -58,19 +57,14 @@ $fields = [
 ];
 check_fields($fields);
 
-/*
- * Permissions
- */
-if (getRequest('groupid') && !isReadableHostGroups([getRequest('groupid')])) {
-	access_deny();
-}
-
 $config = select_config();
 if (hasRequest('filter_set')) {
 	CProfile::update('web.overview.filter.show_triggers', getRequest('show_triggers', TRIGGERS_OPTION_RECENT_PROBLEM),
 		PROFILE_TYPE_INT
 	);
-	CProfile::update('web.overview.filter.show_suppressed', getRequest('show_suppressed', 0), PROFILE_TYPE_INT);
+	CProfile::update('web.overview.filter.show_suppressed', getRequest('show_suppressed', ZBX_PROBLEM_SUPPRESSED_FALSE),
+		PROFILE_TYPE_INT
+	);
 	CProfile::update('web.overview.filter.show_severity', getRequest('show_severity', TRIGGER_SEVERITY_NOT_CLASSIFIED),
 		PROFILE_TYPE_INT
 	);
@@ -97,6 +91,8 @@ if (hasRequest('filter_set')) {
 	}
 	CProfile::updateArray('web.overview.filter.inventory.field', $inventoryFields, PROFILE_TYPE_STR);
 	CProfile::updateArray('web.overview.filter.inventory.value', $inventoryValues, PROFILE_TYPE_STR);
+	CProfile::updateArray('web.overview.filter.groupids', getRequest('filter_groupids', []), PROFILE_TYPE_ID);
+	CProfile::updateArray('web.overview.filter.hostids', getRequest('filter_hostids', []), PROFILE_TYPE_ID);
 }
 elseif (hasRequest('filter_rst')) {
 	DBStart();
@@ -110,6 +106,8 @@ elseif (hasRequest('filter_rst')) {
 	CProfile::delete('web.overview.filter.application');
 	CProfile::deleteIdx('web.overview.filter.inventory.field');
 	CProfile::deleteIdx('web.overview.filter.inventory.value');
+	CProfile::deleteIdx('web.overview.filter.groupids');
+	CProfile::deleteIdx('web.overview.filter.hostids');
 	DBend();
 }
 
@@ -124,48 +122,41 @@ if (hasRequest('view_style')) {
 	CProfile::update('web.overview.view_style', getRequest('view_style'), PROFILE_TYPE_INT);
 }
 
-$showTriggers = CProfile::get('web.overview.filter.show_triggers', TRIGGERS_OPTION_RECENT_PROBLEM);
-
 /*
  * Display
  */
-$page_filter = new CPageFilter([
-	'groups' => [
-		($type == SHOW_TRIGGERS ? 'with_monitored_triggers' : 'with_monitored_items') => true
-	],
-	'hosts' => [
-		'monitored_hosts' => true,
-		($type == SHOW_TRIGGERS ? 'with_monitored_triggers' : 'with_monitored_items') => true
-	],
-	'hostid' => getRequest('hostid'),
-	'groupid' => getRequest('groupid')
-]);
-
 $data = [
 	'type' => $type,
 	'view_style' => CProfile::get('web.overview.view_style', STYLE_TOP),
 	'config' => $config,
-	'pageFilter' => $page_filter,
-	'groupid' => $page_filter->groupid,
-	'hostid' => $page_filter->hostid,
 	'profileIdx' => 'web.overview.filter',
-	'active_tab' => CProfile::get('web.overview.filter.active', 1)
+	'active_tab' => CProfile::get('web.overview.filter.active', 1),
+	'db_hosts' => [],
+	'db_triggers' => [],
+	'dependencies' => [],
+	'triggers_by_name' => [],
+	'hosts_by_name' => [],
+	'exceeded_hosts' => false,
+	'exceeded_trigs' => false
 ];
 
 // fetch trigger data
 if ($type == SHOW_TRIGGERS) {
 	// filter data
 	$filter = [
-		'showTriggers' => $showTriggers,
+		'showTriggers' => CProfile::get('web.overview.filter.show_triggers', TRIGGERS_OPTION_RECENT_PROBLEM),
 		'ackStatus' => CProfile::get('web.overview.filter.ack_status', 0),
 		'showSeverity' => CProfile::get('web.overview.filter.show_severity', TRIGGER_SEVERITY_NOT_CLASSIFIED),
 		'statusChange' => CProfile::get('web.overview.filter.status_change', 0),
 		'statusChangeDays' => CProfile::get('web.overview.filter.status_change_days', 14),
 		'txtSelect' => CProfile::get('web.overview.filter.txt_select', ''),
 		'application' => CProfile::get('web.overview.filter.application', ''),
-		'show_suppressed' => CProfile::get('web.overview.filter.show_suppressed', 0),
+		'show_suppressed' => CProfile::get('web.overview.filter.show_suppressed', ZBX_PROBLEM_SUPPRESSED_FALSE),
+		'groupids' => CProfile::getArray('web.overview.filter.groupids', []),
+		'hostids' => CProfile::getArray('web.overview.filter.hostids', []),
 		'inventory' => []
 	];
+
 	foreach (CProfile::getArray('web.overview.filter.inventory.field', []) as $i => $field) {
 		$filter['inventory'][] = [
 			'field' => $field,
@@ -173,63 +164,115 @@ if ($type == SHOW_TRIGGERS) {
 		];
 	}
 
-	if ($data['pageFilter']->groupsSelected) {
-		$host_options = [];
+	$trigger_options = [
+		'search' => ($filter['txtSelect'] !== '') ? ['description' => $filter['txtSelect']] : null,
+		'only_true' => ($filter['showTriggers'] == TRIGGERS_OPTION_RECENT_PROBLEM) ? true : null,
+		'filter' => [
+			'value' => ($filter['showTriggers'] == TRIGGERS_OPTION_IN_PROBLEM) ? TRIGGER_VALUE_TRUE : null
+		],
+		'skipDependent' => ($filter['showTriggers'] == TRIGGERS_OPTION_ALL) ? null : true
+	];
 
-		if ($filter['inventory']) {
-			$host_options['searchInventory'] = [];
+	$problem_options = [
+		'show_recent' => ($filter['showTriggers'] == TRIGGERS_OPTION_RECENT_PROBLEM) ? true : null,
+		'show_suppressed' => $filter['show_suppressed'],
+		'acknowledged' => ($filter['ackStatus'] == EXTACK_OPTION_UNACK) ? false : null,
+		'min_severity' => $filter['showSeverity'],
+		'time_from' => $filter['statusChange'] ? (time() - $filter['statusChangeDays'] * SEC_PER_DAY) : null
+	];
 
-			foreach ($filter['inventory'] as $field) {
-				$host_options['searchInventory'][$field['field']][] = $field['value'];
-			}
+	$filter['groups'] = $filter['groupids']
+		? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
+			'groupids' => $filter['groupids'],
+			'preservekeys' => true
+		]), ['groupid' => 'id'])
+		: [];
+
+	$groupids = $filter['groups'] ? getSubGroups(array_keys($filter['groups'])) : [];
+
+	$filter['hosts'] = $filter['hostids']
+		? CArrayHelper::renameObjectsKeys(API::Host()->get([
+			'output' => ['hostid', 'name'],
+			'hostids' => $filter['hostids'],
+			'monitored_hosts' => true,
+			'with_monitored_triggers' => true,
+			'preservekeys' => true
+		]), ['hostid' => 'id'])
+		: [];
+
+	unset($filter['groupids'], $filter['hostids']);
+
+	$host_options = [];
+	if ($filter['inventory']) {
+		$host_options['searchInventory'] = [];
+		foreach ($filter['inventory'] as $field) {
+			$host_options['searchInventory'][$field['field']][] = $field['value'];
 		}
-
-		$trigger_options = [
-			'search' => ($filter['txtSelect'] !== '') ? ['description' => $filter['txtSelect']] : null,
-			'only_true' => ($filter['showTriggers'] == TRIGGERS_OPTION_RECENT_PROBLEM) ? true : null,
-			'filter' => [
-				'value' => ($filter['showTriggers'] == TRIGGERS_OPTION_IN_PROBLEM) ? TRIGGER_VALUE_TRUE : null
-			],
-			'skipDependent' => ($filter['showTriggers'] == TRIGGERS_OPTION_ALL) ? null : true
-		];
-
-		$problem_options = [
-			'show_recent' => ($filter['showTriggers'] == TRIGGERS_OPTION_RECENT_PROBLEM) ? true : null,
-			'show_suppressed' => $filter['show_suppressed'],
-			'acknowledged' => ($filter['ackStatus'] == EXTACK_OPTION_UNACK) ? false : null,
-			'min_severity' => $filter['showSeverity'],
-			'time_from' => $filter['statusChange'] ? (time() - $filter['statusChangeDays'] * SEC_PER_DAY) : null
-		];
-
-		$groupids = ($data['pageFilter']->groupids !== null) ? $data['pageFilter']->groupids : [];
-
-		list($hosts, $triggers) = getTriggersOverviewData($groupids, $filter['application'], $host_options,
-			$trigger_options, $problem_options
-		);
 	}
-	else {
-		$hosts = [];
-		$triggers = [];
+	if ($filter['hosts']) {
+		$host_options['hostids'] = array_keys($filter['hosts']);
 	}
 
+	list($data['db_hosts'], $data['db_triggers'], $data['dependencies'], $data['triggers_by_name'],
+		$data['hosts_by_name'], $data['exceeded_hosts'], $data['exceeded_trigs']
+	) = getTriggersOverviewData($groupids, $filter['application'], $host_options, $trigger_options, $problem_options);
+
 	$data['filter'] = $filter;
-	$data['hosts'] = $hosts;
-	$data['triggers'] = $triggers;
 
-	$overviewView = new CView('monitoring.overview.triggers', $data);
+	// Render view.
+	echo (new CView('monitoring.overview.triggers', $data))->getOutput();
 }
-// fetch item data
+// Fetch item data.
 else {
 	$data['filter'] = [
 		'application' => CProfile::get('web.overview.filter.application', ''),
-		'show_suppressed' => CProfile::get('web.overview.filter.show_suppressed', 0)
+		'hostids' => CProfile::getArray('web.overview.filter.hostids', []),
+		'groupids' => CProfile::getArray('web.overview.filter.groupids', []),
+		'show_suppressed' => CProfile::get('web.overview.filter.show_suppressed', ZBX_PROBLEM_SUPPRESSED_FALSE)
 	];
 
-	$overviewView = new CView('monitoring.overview.items', $data);
+	$data['filter']['groups'] = $data['filter']['groupids']
+		? CArrayHelper::renameObjectsKeys(API::HostGroup()->get([
+			'output' => ['groupid', 'name'],
+			'groupids' => $data['filter']['groupids'],
+			'preservekeys' => true
+		]), ['groupid' => 'id'])
+		: [];
+
+	$data['filter']['hosts'] = $data['filter']['hostids']
+		? CArrayHelper::renameObjectsKeys(API::Host()->get([
+			'output' => ['hostid', 'name'],
+			'hostids' => $data['filter']['hostids'],
+			'preservekeys' => true
+		]), ['hostid' => 'id'])
+		: [];
+
+	unset($data['filter']['groupids'], $data['filter']['hostids']);
+
+	$groupids = $data['filter']['groups'] ? getSubGroups(array_keys($data['filter']['groups'])) : null;
+	$hostids = $data['filter']['hosts'] ? array_keys($data['filter']['hosts']) : null;
+
+	if ($data['view_style'] == STYLE_TOP) {
+		list($db_items, $db_hosts, $items_by_name, $has_hidden_data) = getDataOverviewTop($groupids, $hostids,
+			$data['filter']['application']
+		);
+	}
+	else {
+		list($db_items, $db_hosts, $items_by_name, $has_hidden_data) = getDataOverviewLeft($groupids, $hostids,
+			$data['filter']['application']
+		);
+	}
+
+	$data['visible_items'] = getDataOverviewCellData($db_hosts, $db_items, $items_by_name,
+		$data['filter']['show_suppressed']
+	);
+	$data['db_hosts'] = $db_hosts;
+	$data['items_by_name'] = $items_by_name;
+	$data['has_hidden_data'] = $has_hidden_data;
+
+	// Render view.
+	echo (new CView('monitoring.overview.items', $data))->getOutput();
 }
 
-// render view
-$overviewView->render();
-$overviewView->show();
-
 require_once dirname(__FILE__).'/include/page_footer.php';
