@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,32 +19,39 @@
 **/
 
 
-require_once dirname(__FILE__).'/include/classes/core/Z.php';
+require_once dirname(__FILE__).'/include/classes/core/APP.php';
 
 $page['file'] = 'setup.php';
 
 try {
-	Z::getInstance()->run(ZBase::EXEC_MODE_SETUP);
+	APP::getInstance()->run(APP::EXEC_MODE_SETUP);
 }
 catch (Exception $e) {
-	(new CView('general.warning', [
+	echo (new CView('general.warning', [
 		'header' => $e->getMessage(),
 		'messages' => [],
 		'theme' => ZBX_DEFAULT_THEME
-	]))->render();
+	]))->getOutput();
 
 	exit;
 }
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'type' =>				[T_ZBX_STR, O_OPT, null,	IN('"'.ZBX_DB_MYSQL.'","'.ZBX_DB_POSTGRESQL.'","'.ZBX_DB_ORACLE.'","'.ZBX_DB_DB2.'"'), null],
+	'type' =>				[T_ZBX_STR, O_OPT, null,	IN('"'.ZBX_DB_MYSQL.'","'.ZBX_DB_POSTGRESQL.'","'.ZBX_DB_ORACLE.'"'), null],
 	'server' =>				[T_ZBX_STR, O_OPT, null,	null,				null],
 	'port' =>				[T_ZBX_INT, O_OPT, null,	BETWEEN(0, 65535),	null, _('Database port')],
 	'database' =>			[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,			null, _('Database name')],
 	'user' =>				[T_ZBX_STR, O_OPT, null,	null,				null],
 	'password' =>			[T_ZBX_STR, O_OPT, null,	null, 				null],
 	'schema' =>				[T_ZBX_STR, O_OPT, null,	null, 				null],
+	'tls_encryption' =>		[T_ZBX_INT, O_OPT, null,	IN([0,1]),			null],
+	'verify_certificate' =>	[T_ZBX_INT, O_OPT, null,	IN([0,1]),			null],
+	'verify_host' =>		[T_ZBX_INT, O_OPT, null,	IN([0,1]),			null],
+	'key_file' =>			[T_ZBX_STR, O_OPT, null,	null, 				null],
+	'cert_file' =>			[T_ZBX_STR, O_OPT, null,	null, 				null],
+	'ca_file' =>			[T_ZBX_STR, O_OPT, null,	null, 				null],
+	'cipher_list' =>		[T_ZBX_STR, O_OPT, null,	null, 				null],
 	'zbx_server' =>			[T_ZBX_STR, O_OPT, null,	null,				null],
 	'zbx_server_name' =>	[T_ZBX_STR, O_OPT, null,	null,				null],
 	'zbx_server_port' =>	[T_ZBX_INT, O_OPT, null,	BETWEEN(0, 65535),	null, _('Port')],
@@ -104,6 +111,7 @@ if (!CWebUser::$data) {
 	->addJsFile((new CUrl('jsLoader.php'))
 		->setArgument('ver', ZABBIX_VERSION)
 		->setArgument('lang', CWebUser::$data['lang'])
+		->setArgument('files', ['setup.js'])
 		->getUrl()
 	)
 	->display();
@@ -111,13 +119,17 @@ if (!CWebUser::$data) {
 /*
  * Displaying
  */
-$link = (new CLink('GPL v2', 'http://www.zabbix.com/license.php'))
+$link = (new CLink('GPL v2', 'https://www.zabbix.com/license'))
 	->setTarget('_blank')
 	->addClass(ZBX_STYLE_GREY)
 	->addClass(ZBX_STYLE_LINK_ALT);
 $sub_footer = (new CDiv(['Licensed under ', $link]))->addClass(ZBX_STYLE_SIGNIN_LINKS);
 
-(new CTag('body', true, [(new CTag('main', true, [$ZBX_SETUP_WIZARD, $sub_footer])), makePageFooter()]))
+(new CTag('body', true,
+	(new CDiv([
+		(new CTag('main', true, [$ZBX_SETUP_WIZARD, $sub_footer])), makePageFooter()])
+	)->addClass(ZBX_STYLE_LAYOUT_WRAPPER)
+))
 	->setAttribute('lang', CWebUser::getLang())
 	->show();
 ?>
